#include "sd.h"

SD3DBox::SD3DBox(void)
{
}

SD3DBox::~SD3DBox(void)
{
}

FITSRESULT	SD3DBox::fits(SD3DBox *pBox)
{
	// Test if this box fits in the passed in box
	Point a, b, a_sml, b_sml;
	pBox->get(&a, &b);
	this->get(&a_sml, &b_sml);

	// Get the extents of the containing box
	Point t;
	t = a - b/2;
	b = a + b/2;
	a = t;

	// Get the extents of the this box
	t = a_sml - b_sml/2;
	b_sml = a_sml + b_sml/2;
	a_sml = t;

	// Check whether there is x axis overlap
	if( ( a_sml.x < a.x && b_sml.x < a.x ) || ( a_sml.x > b.x && b_sml.x > b.x ) )
		return OUTSIDE;

	// Check whether there is y axis overlap
	if( ( a_sml.y < a.y && b_sml.y < a.y ) || ( a_sml.y > b.y && b_sml.y > b.y ) )
		return OUTSIDE;

	// Check whether there is z axis overlap
	if( ( a_sml.z < a.z && b_sml.z < a.z ) || ( a_sml.z > b.z && b_sml.z > b.z ) )
		return OUTSIDE;

	// The cubes overlap, check if the passed in one contains this box
	if( (a_sml.x >= a.x && b_sml.x <= b.x )
		&& (a_sml.y >= a.y && b_sml.y <= b.y )
		&& (a_sml.z >= a.z && b_sml.z <= b.z ) )
		return CONTAINED;

	// The boxes overlapped so return intersects
	return INTERSECTS;
}

int SD3DBox::whichOctants(Point* p, bool* intersections)
{
	
	// Get the extents of this box
	Point a, b;
	a = m_location - m_dimensions/2;
	b = m_location + m_dimensions/2;	

	bool minusX = false;
	bool plusX = false;
	bool minusY = false;
	bool plusY = false;
	bool minusZ = false;
	bool plusZ = false;

	// Check if the box lies on the -X and +X plane sides
	if( a.x < p->x )
		minusX = true;
	if( b.x > p->x )
		plusX = true;
	// Check if the box lies on the -Y and +Y plane sides
	if( a.y < p->y )
		minusY = true;
	if( b.y > p->y )
		plusY = true;
	// Check if the box lies on the -Z and +Z plane sides
	if( a.z < p->z )
		minusZ = true;
	if( b.z > p->z )
		plusZ = true;

	// Set the various octants
	if( minusX && minusY && minusZ )
		intersections[0] = true;
	if( plusX && minusY && minusZ )
		intersections[PLUSXPLANE] = true;
	if( minusX && plusY && minusZ )
		intersections[PLUSYPLANE] = true;
	if( minusX && minusY && plusZ )
		intersections[PLUSZPLANE] = true;
	if( plusX && minusY && plusZ )
		intersections[PLUSXPLANE | PLUSZPLANE] = true;
	if( plusX && plusY && minusZ )
		intersections[PLUSXPLANE | PLUSYPLANE] = true;
	if( minusX && plusY && plusZ )
		intersections[PLUSYPLANE | PLUSZPLANE] = true;
	if( plusX && plusY && plusZ )
		intersections[PLUSXPLANE | PLUSYPLANE | PLUSZPLANE] = true;

	// Check for a single intersection
	int i;
	int indexOfIntersection = -1;
	for(i=0; i<8; i++)
	{
		if ( intersections[i] )
		{
			if(indexOfIntersection == -1)
			{
				indexOfIntersection = i;
			}
			else
				return -1;
		}
	}

	return indexOfIntersection;
}