#pragma once

enum TREETYPE
{
	OCTREE
};

#include <list>
using namespace std;

class SDSceneNode;
class SDNodeList;

// A class to encapsulate the multiple references to a node in a tree
// For example:
// An irregular object may not fit completely into a node of a
// tree but it may be better defined by adding it to only a subset of
// the child tree nodes. Each child node that contains a piece of the 
// the object contains will then contain reference to the object. 

// Note that this means an object may be referenced by all the child
// pieces and the division was not efficient. If the object is referenced
// in N or more children, then adding the node should fail and it should be added
// to the parent. Adjust N to suit your culling efficiency requirements.

class SDTreeNode
{
protected:
	// The list of all nodes available to this tree
	SDNodeList*		m_pNodes;

	// Index of the Scene nodes in the node list that are branched off of this part of the tree
	list<int>*		m_pNodeRefList;

	// The depth of this tree node
	unsigned int	m_depth;

public:
	SDTreeNode(SDNodeList* pList, unsigned int depth);
	virtual ~SDTreeNode(void);

	// Gets the type of the tree
	virtual TREETYPE getTreeType() = 0;

	// called when a node moves position so the tree can update its location
	virtual void nodeMoved( int nodeIndex ) = 0;

	// Adds a node to the tre. Returns false if it was unable to add the node
	virtual bool addNode(int nodeIndex) = 0;

	// Removes a node from this tree
	virtual bool removeNode( int nodeIndex ) = 0;	

};
