#include ".\BinaryFileStream.h"

//---------BinaryFileStreamWriter ---------------------------------------------------
BinaryFileStreamWriter::BinaryFileStreamWriter(const char* filename) {	
	_filename = new char[ strlen(filename) + 1];
	strcpy(_filename, filename);
	_stream.open(filename, ios_base::binary);
}

BinaryFileStreamWriter::~BinaryFileStreamWriter(void) {
	close();
}
void BinaryFileStreamWriter::close() {
	_stream.close();
	if(_filename)
	{
		delete _filename;
		_filename = NULL;
	}

}

int BinaryFileStreamWriter::seek(long position) {
	_stream.seekp(position);
	return _stream.tellp();
}
int BinaryFileStreamWriter::seek(long offset, SeekFrom from) {
	_stream.seekp(offset, (ios_base::seekdir)from);
	return _stream.tellp();
}

void BinaryFileStreamWriter::writeChar(char c) {
	_stream << c;
}

void BinaryFileStreamWriter::writeString(char* string) {
	_stream.write(string, (std::streamsize)strlen(string) );
	int length = (int)strlen(string);
	if (string[length-1] != 0)
		_stream << (char)0;
}
void BinaryFileStreamWriter::writeString(char* string, int length) {
	_stream.write(string, length);
}
void BinaryFileStreamWriter::writeString(std::string string) {
	_stream.write(string.c_str(), (std::streamsize)string.length());
}
void BinaryFileStreamWriter::writeInteger(int number) {
	_stream.write( (char*)(&number), sizeof(number));
}

void BinaryFileStreamWriter::writeDouble(double number) {
	_stream.write( (char*)(&number), sizeof(number));
}
void BinaryFileStreamWriter::writeFloat(float number) {
	_stream.write( (char*)(&number), sizeof(number));
}
void BinaryFileStreamWriter::writeShort(short number) {
	_stream.write( (char*)(&number), sizeof(number));
}
//---------BinaryFileStreamReader ---------------------------------------------------
BinaryFileStreamReader::BinaryFileStreamReader(const char* filename) {
	_filename = new char[ strlen(filename) + 1];
	strcpy(_filename, filename);
	_stream.open(filename, ios_base::binary);
}
BinaryFileStreamReader::~BinaryFileStreamReader(void) {
	close();
}
void BinaryFileStreamReader::close() {
	_stream.close();
	if(_filename)
	{
		delete _filename;
		_filename = NULL;
	}
}
int BinaryFileStreamReader::seek(long position) {
	_stream.seekg(position);
	return _stream.tellg();
}
int BinaryFileStreamReader::seek(long offset, SeekFrom from) {
	_stream.seekg(offset, (ios_base::seekdir)from);
	return _stream.tellg();
}

char BinaryFileStreamReader::readChar() {
	char character;
	_stream >> character;
	return character;
}

int BinaryFileStreamReader::readString(string* pStr) {	
	buffer[MAX_READ_STRING_LENGTH] = 0;
	
	_stream.get(buffer, MAX_READ_STRING_LENGTH, '\0');	// Gets up to but NOT including the delimiter
	int charsRead = (int)strlen(buffer);

	// Get the delimiter if it reached the delimiter
	if( charsRead != MAX_READ_STRING_LENGTH )
		_stream.get();										
	*pStr = buffer;

	return charsRead;
}

bool BinaryFileStreamReader::readString(string* pStr, int length) {

	if(length < 0 || length > MAX_READ_STRING_LENGTH)
		return false;

	buffer[length] = 0;

	_stream.read(buffer, length);

	*pStr = buffer;

	return true;
}

int BinaryFileStreamReader::readInteger() {
	int number;
	_stream.read( (char*)(&number), sizeof(number));
	return number;
}

double BinaryFileStreamReader::readDouble() {
	double d;
	_stream.read( (char*)(&d), sizeof(d));
	return d;
}
float BinaryFileStreamReader::readFloat() {
	float f;
	_stream.read( (char*)(&f), sizeof(f));
	return f;
}
short BinaryFileStreamReader::readShort() {
	short s;
	_stream.read( (char*)(&s), sizeof(s));
	return s;
}