#include "includes.h"

/*static*/ RGBA RGBA::Null = {0, 0, 0, 0};

/*static*/ Console Console::s_console;

extern int centerX, centerY;
extern int windowWidth, windowHeight;
extern int windowLeft, windowTop;
extern HWND hWnd;
extern void getWindowInfo();

CommandResult CALLBACK Console::ConsoleCommandWatcher(void* object, string command, vector<string> parameters) {
	Console *c = (Console*)object;
	if (command == "clear") {
		string clearWhat = "";
		if (parameters.size() > 0)
			clearWhat = parameters[0];
		else
			clearWhat = "log";

		if (clearWhat == "log") {
			c->clear();
		} else if (clearWhat == "history") {
			c->clearCommandHistory();
			return CR_SUCCESS_NO_HISTORY;
		} else if(clearWhat =="all") {
			c->clear();
			c->clearCommandHistory();
			return CR_SUCCESS_NO_HISTORY;
		} else {
			c->writeline("USAGE: clear [log | history]");
			return CR_USAGE_ERROR;
		}
	} else if (command == "info") {
		c->writeline("COMP 4002 Project");
		c->writeline("OpenGL 3D Engine");
		c->writeline("------------------------------------------------------------------------");
		c->writeline("Use PgUp / PgDn to scroll the console");
		c->writeline("Thats right! I said scroll.... We're scrollin baby!! :D");
		c->writeline("------------------------------------------------------------------------");
		c->write (
			"Command                     Description\n"
			"INFO                        Prints this information.\n"
			"GET <variable>              Retrieves the value of the variable\n"
			"SET <variable> <value>      Sets the variable with the value\n"
			"UNSET <variable>            Unsets the variable\n"
			"CLEAR [log | history | all] Clears the console\n"
			"DUMP <filename>             Dumps the contents of the console into a file");
		c->writeline("");

	} else if (command =="dump") {
		ofstream *stream = new ofstream(parameters[0].c_str());
		for(unsigned int i = 0; i < c->m_log.size(); i++) {
			string line = c->m_log[i];
			stream->write(line.c_str(), (std::streamsize)line.length());
			stream->write("\n", 1);
		}
		stream->close();
		c->write ("Console contents dumped to file: '%s'", parameters[0].c_str());
	} else if (command == "window") {
		getWindowInfo();
		c->write("Window: %x\n"
			"Position:    [%d, %d]\n"
			"Size:        [%d x %d]\n"
			"Center:      [%d, %d]",
			hWnd, windowLeft, windowTop, windowWidth, windowHeight, centerX, centerY);
	} else if (command == "mouse") {
		POINT p;
		GetCursorPos(&p);
		c->write("Mouse position: %d, %d", p.x, p.y);

	} else {
		return CR_NOT_DEFINED;
	}
	return CR_SUCCESS;
}
void CALLBACK Console::OnScroll(Scrollbar *scrollbar, float value) {
	Console *c = (Console*)scrollbar->getParent();
	c->m_firstVisibleLine = (int)ceil(value);
}

void CALLBACK Console::OnEngineVariableChange(void *object, string variable, string newValue) {
	Console *c = (Console*)object;
	if (variable == "console_size") {
		if (newValue != "") {
			sscanf(newValue.c_str(), "%d", &c->m_maxLogSize);
			
			while (c->m_log.size() >= (unsigned int)c->m_maxLogSize) {
				c->m_log.erase( c->m_log.begin( ) );
			}
			
			int numOffscreen = (int)(c->m_log.size() - c->m_maxVisibleLines - 1);
			if (numOffscreen > 0 ) {
				c->m_scrollbar.setMaxValue(numOffscreen);
				c->m_scrollbar.scrollToMax();
			}
		} else {
			c->m_maxLogSize = DEFAULT_MAX_LOG_SIZE;
		}
		c->write("ConsoleSize: %d", c->m_maxLogSize);
	}
}

Console::Console() {
	m_consoleFont = NULL;

	m_state = CS_HIDDEN;
	m_movement = 0;
	m_historyIndex = -1;

	m_firstVisibleLine = 0;
	m_maxVisibleLines = 0;
	
	m_commandColor.set(0.251, 0.867, 1.000, 1.000);
	m_logColor.set(1, 1, 1, 1.000);
}

void Console::initialize() {
	m_consoleFont = new Font("Lucida Console", 12);

	Point fontSize = m_consoleFont->getAvgerageCharSize();	
	m_maxVisibleLines = (int)(242 / (fontSize.y)) - 1;

	m_texture = Texture::readTexture("..\\Textures\\console.bmp");
	m_texture->load();

	setupControls();

	addCommandCallback(this, ConsoleCommandWatcher);
	
	EngineVariables::Get()->addCallback("console_size", this, OnEngineVariableChange);
	EngineVariables::Get()->addCallback("history_size", this, OnEngineVariableChange);

	m_maxLogSize = DEFAULT_MAX_LOG_SIZE;
	EngineVariables::Get()->setVariable("console_size", m_maxLogSize);
	EngineVariables::Get()->setVariable("history_size", "10");
	
	processCommand("info", false);
}
void Console::setupControls() {
	// Scrollbar
	m_scrollbar.setPosition(772, 334);
	m_scrollbar.setSize(18, 241);
	m_scrollbar.setParent(this);
	m_scrollbar.setCallback(OnScroll);
	m_scrollbar.setScrollValues(0,0,0);
	
	m_scrollbar.setBarForeColor(RGBA::Null);
	m_scrollbar.setBarBackColor(RGBA::Null);
	
	RGBA hfc = {0.75, 0.75, 0.75, 0.5};
	m_scrollbar.setHandleBackColor(hfc);
	hfc.set(0, 0, 0, 0.5);
	m_scrollbar.setHandleForeColor(hfc);

	// EditBox
	m_editBox.setPosition(5,308);
	m_editBox.setSize(790,20);

	RGBA bcolor = {0, 0, 0, 0};
	m_editBox.setBackColor(bcolor);
	m_editBox.setBorderColor(bcolor);
	
	RGBA fcolor = {0, 0, 0, 1.000};
	m_editBox.setForeColor(fcolor);
	m_editBox.setCaretColor(fcolor);

	m_editBox.setFont("Courier New", 16);
	m_editBox.show();
}

void Console::addCommandCallback(void* object, CommandProc callback) {
	m_parser.addCallback(object, callback);
}

void Console::write(char* format, ...) {
	char buffer[1024];
	va_list parameters;

	va_start (parameters, format);
	    vsprintf (buffer, format, parameters);
	va_end (parameters);

	string message(buffer);
	write(message);
}

void Console::write(string text) {

	int pos = 0;
	int oldPos = -1;
	
	do {
		pos = text.find('\n', oldPos + 1);
		if (pos == -1)
			writeline(text.substr(oldPos + 1));
		else
			writeline(text.substr(oldPos + 1, pos - oldPos - 1));
		oldPos = pos;
	} while (pos != -1);
}

void Console::writeline(string text) {

	while (m_log.size() >= m_maxLogSize) {
		m_log.erase( m_log.begin( ) );
	}
	m_log.push_back(text);

	int numOffscreen = m_log.size() - m_maxVisibleLines - 1;
	if (numOffscreen > 0 ) {
		m_scrollbar.setMaxValue(numOffscreen);
		m_scrollbar.scrollToMax();
	}
}

void Console::show() {
	if (m_state == CS_HIDDEN) {
		m_state = CS_SHOWING;
	}
}
void Console::hide() {
	if (m_state == CS_VISIBLE) {
		m_state = CS_HIDING;
	}
}
void Console::toggleVisible() {
	if (m_state == CS_VISIBLE)
		hide();
	else if (m_state == CS_HIDDEN)
		show();
}

void Console::tick() {
	Timer *t = Timer::Get();

	switch(m_state) {
	case CS_VISIBLE:
		m_editBox.tick();
		break;
	case CS_HIDDEN:
		return;
	case CS_WORKING:
		break;
	case CS_SHOWING:
		m_movement += t->deltaTime;
		if (m_movement >= CONSOLE_SPEED) {
			m_movement = 0;
			m_state = CS_VISIBLE;
		}
		break;
	case CS_HIDING:
		m_movement += t->deltaTime;
		if (m_movement >= CONSOLE_SPEED) {
			m_movement = 0;
			m_state = CS_HIDDEN;
		}
		break;
	}
}

void Console::draw() {

	if (m_state == CS_HIDDEN) return;

	begin2DDrawing(800, 600);


	if (m_state == CS_HIDING || m_state == CS_SHOWING) {
		glPushMatrix();
		glLoadIdentity();
		Transformation matrix;
	
		Point move;
		move.x = move.z = move.y = 0;
		
		if (m_state == CS_SHOWING)
			move.y = (1 - m_movement / (float)CONSOLE_SPEED) * 300;
		else
			move.y = (m_movement / (float)CONSOLE_SPEED) * 300;
		
		matrix.translateBy(move);
		glMultMatrixd(matrix);
	}

	glEnable(GL_TEXTURE_2D);
	m_texture->activate();
	glEnable(GL_BLEND);
	glColor4f(1,1,1,0.75);
	// Console Base
	glEnable(GL_BLEND);
	glBegin(GL_POLYGON);
		glColor4f(1.0, 1.0, 1.0, 0.7);
		glTexCoord2f(0,0);
		glVertex2d(0, 300);

		glTexCoord2f(0.78125, 0);
		glVertex2d(800, 300);

		glTexCoord2f(0.78125, 0.5859);
		glVertex2d(800, 600);

		glTexCoord2f(0, 0.5859);
		glVertex2d(0, 600);
	glEnd();
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);

	// DrawControls
	m_editBox.draw();
	m_scrollbar.draw();	

	int y = 337;
	Point p = m_consoleFont->getAvgerageCharSize();
	if (&m_log != NULL && m_log.size() > 0) {
		int lastLine = m_firstVisibleLine + m_maxVisibleLines;
		if (lastLine > m_log.size()-1) lastLine = m_log.size() - 1;
		
		for(int logIndex = lastLine; logIndex >= m_firstVisibleLine; logIndex--) {
			string message = m_log[logIndex];
			if (message[0] == '>')
				m_commandColor.apply();
			else
				m_logColor.apply();

			m_consoleFont->drawText(8, y, message.c_str() );			
			y += p.y ;
		}
	}
	
	if (m_state == CS_HIDING || m_state == CS_SHOWING) {
		glPopMatrix();
	}

	end2DDrawing();
	glColor3f(1,1,1);
	glEnable(GL_TEXTURE_2D);
}

void Console::processCommand(string command, bool echo) {
	if (command != "") {
		if (echo) writeline(">" + command);
		if (m_parser.parse(command))
			addToHistory(command);
	}
}

int vectorFind(vector<string> collection, string search) {
	if (&collection == NULL || search == "") return -2;
	if (collection.size() == 0) return -1;

	for(int index = 0; index < collection.size(); index++) {
		if (collection[index] == search)
			return index;
	}

	return -1;
}

void Console::addToHistory(string command) {
	if (command == "") return;
	int maxHistorySize = EngineVariables::Get()->getVariableInt("history_size");
	if (maxHistorySize == 0) return;
	
	int index = vectorFind(m_history, command);
	if (index >= 0) {
		m_history.erase(m_history.begin() + index);
		m_history.insert(m_history.begin(), command);
		m_historyIndex = -1;
	} else if (index == -1) {
		m_history.insert(m_history.begin(), command);
		m_historyIndex = -1;
	}
}


bool Console::processKeyDown(unsigned char key, int x, int y) {
	if (m_state != CS_VISIBLE && m_state !=CS_SHOWING) return false;
	
	if (key == 13) {		// Enter
		string command = m_editBox.getText();
		m_editBox.setText("");
		processCommand(command);
	}
	return m_editBox.processKeyDown(key, x, y);
}
bool Console::processSpecialKeyDown(int key, int x, int y) {
	if (m_state != CS_VISIBLE) return false;
	if (key == GLUT_KEY_PAGE_UP) {
		m_scrollbar.scrollUp(1);
		return true;
	} else if (key == GLUT_KEY_PAGE_DOWN) {
		m_scrollbar.scrollDown(1);
		return true;
	} else if (key == GLUT_KEY_UP) {
		if (m_history.size() > 0) {
			m_historyIndex++;
			if (m_historyIndex >= m_history.size()) m_historyIndex = m_history.size() - 1;
			m_editBox.setText(m_history[m_historyIndex]);
		}
		return true;
	} else if (key == GLUT_KEY_DOWN) {
		if (m_history.size() > 0) {
			m_historyIndex --;
			if (m_historyIndex < 0) {
				m_historyIndex = -1;
				m_editBox.setText("");
			} else {
				m_editBox.setText(m_history[m_historyIndex]);
			}
		}
		return true;
	}
	return m_editBox.processSpecialKeyDown(key, x, y);
}

// Command Parser Methods ----------------------------------------------------------
bool Console::CommandParser::parse(string commandline) {
	int pos = 0;
	string command = extractString(commandline, pos);
	vector<string> parameters;

	while (pos < commandline.size()) {
		string param = extractString(commandline, pos);
		if (param != "")
			parameters.push_back(param);
	}
	
	CommandResult ret = sendNotifications(command, parameters);
	if (ret == CR_NOT_DEFINED) 
		Console::Get()->write("Unrecognized Command: '%s'", command.c_str());
		
	return ( (ret & CR_SUCCESS) == CR_SUCCESS);
}
CommandResult Console::CommandParser::sendNotifications(string command, vector<string> parameters) {
	CommandResult result = CR_NOT_DEFINED;

	for(int eventIndex = 0; eventIndex < m_callbacks.size(); eventIndex++) {
		CommandEventInfo *cei = m_callbacks[eventIndex];
		CommandProc cp = cei->m_callback;
		result = (CommandResult)(result | cp(cei->m_object, command, parameters));
	}
	return result;
}

string Console::CommandParser::extractString(string search, int &position) {
	if (position >= search.size()) return "";
	
	bool quoted = false;
	string buffer = "";	
	
	do {
		char nextChar = search.c_str()[position];
		if ( !(buffer == "" && nextChar == ' ')) {		// Skip leading spaces
			
			if (nextChar == '\"') {						// " found
				if (!quoted) {							// if we are not in quotes
					quoted = true;						//   flag that we are now
				} else {								// if we are in quotes
					position ++;						//   then end of string reached
					break;								//   return string
				}
			} else if (nextChar == ' ') {				// <space> found
				if (!quoted) {							// if we are not in quotes
					position ++;						//   then end of string reached
					break;								//   return string
				} else {								// if we are in quotes
					buffer +=nextChar;					//   then add the char to the buffer
				}
			} else {									// other character found
				buffer += nextChar;						//   add it to the buffer
			}
		}
		position++;										// move to next character
		if (position >= search.size()) break;			// break if we reach the end of the string
	} while(true);
	return buffer;
}

void Console::CommandParser::addCallback(void *object, CommandProc callback) {
	CommandEventInfo *cei = new CommandEventInfo(object, callback);
	m_callbacks.push_back(cei);
}
