#include "includes.h"

extern unsigned int VertexShaderHandles [5];
extern unsigned int PixelShaderHandles [5];
extern long registerForTimeValue [5];

bool CALLBACK EP(EditBox *editBox, string text) {
	editBox->setText("");
	editBox->hide();

	//DefaultUI *dui = (DefaultUI*)editBox->m_parent;
	//log("\nFrom EnterProc: %s", text.c_str() );
	return true;
}

DefaultUI::DefaultUI(Game* game)  {
	_isDisposed = false;
	_game = game;
	_camera = new FreeCamera();
	_followCamera = new FollowCamera();	
	_camera->moveTo(0, 1, 5);
	_camera->rotateTo(0, 0, 0);
	_followCamera->moveTo(0,0,10);
	_followCamera->setTarget(_camera);
	_followCamera->m_maxSpeed = 15.0;
	
	setupFonts();
}
DefaultUI::~DefaultUI() {
	::log("\n~DefaultUI");

	if(!_isDisposed)
		dispose();
}

void DefaultUI::setupFonts() {
	HDC hdc = GetDC(NULL);
	_font = new Font("Courier New", 16);
}
void DefaultUI::cleanupFonts() {
	_font->dispose();
	delete _font;
}

bool DefaultUI::processKeyDown(unsigned char key, int x, int y) {
	switch (key) {
		case 'w': case 'W':
			_camera->translateAhead = true;
			break;
		case 's': case 'S':
			_camera->translateBack = true;
			break;
		case 'a': case 'A':
			_camera->translateLeft = true;
			break;
		case 'd': case 'D':
			_camera->translateRight = true;
			break;
		
		case '?':
			//game->_showHelp = !game->_showHelp;
//			game->ui->showHelp = !game->ui->showHelp;
			break;
		case 't':
		case 'T':
			//wireframe = !wireframe;
			//glPolygonMode (GL_FRONT_AND_BACK, wireframe ? GL_LINE : GL_FILL);
			break;
		default:
			return false;
	}
	return true;
}

bool DefaultUI::processKeyUp(unsigned char key, int x, int y) {
	switch (key) {
		case 'w': case 'W':
			_camera->translateAhead = false;
			break;
		case 's': case 'S': 
			_camera->translateBack = false;
			break;
		case 'a': case 'A':
			_camera->translateLeft = false;
			break;
		case 'd': case 'D':
			_camera->translateRight = false;
			break;
		default: 
			return false;			
	}
	return true;
}
bool DefaultUI::processSpecialKeyDown(int key, int x, int y) {
	switch(key) {
	case GLUT_KEY_UP:
			_camera->rotateUp = true;
			break;
		case GLUT_KEY_DOWN:
			_camera->rotateDown = true;
			break;
		case GLUT_KEY_RIGHT:
			_camera->rotateRight = true;			
			break;
		case GLUT_KEY_LEFT:
			_camera->rotateLeft = true;
			break;
		case GLUT_KEY_PAGE_UP:
			_camera->translateUp = true;
			break;
		case GLUT_KEY_PAGE_DOWN:
			_camera->translateDown = true;
			break;
		default:
			return false;
	}
	return true;
}
bool DefaultUI::processSpecialKeyUp(int key, int x, int y) {
	switch(key) {
		case GLUT_KEY_UP:
			_camera->rotateUp = false;
			break;
		case GLUT_KEY_DOWN:
			_camera->rotateDown = false;
			break;
		case GLUT_KEY_RIGHT:
			_camera->rotateRight = false;			
			break;
		case GLUT_KEY_LEFT:
			_camera->rotateLeft = false;
			break;
		case GLUT_KEY_PAGE_UP:
			_camera->translateUp = false;
			break;
		case GLUT_KEY_PAGE_DOWN:
			_camera->translateDown = false;
			break;
		default:
			return false;
	}
	return true;
}
extern int centerX, centerY;
bool DefaultUI::processMouseMove(int x, int y) {
	POINT screenPOINT; GetCursorPos (&screenPOINT); 
	Point point = {screenPOINT.x, screenPOINT.y, 0.0};
	Point center = {centerX , centerY, 0.0};
	double sensitivity = 0.1;
	Point difference = (point - center) * sensitivity;
	Point rotation = {-difference.y, -difference.x, 0.0};
	_camera->rotateBy (rotation); 

	return true;
}
bool DefaultUI::processMouseClick(int button, int state, int x, int y) {
	return false;
}

void DefaultUI::tick() {
	//_time += timer->deltaTime;
	_camera->tick();
	_followCamera->tick();
}

void DefaultUI::draw() {
	Timer *timer = Timer::Get();

	_followCamera->pushMatrix();
	//_camera->pushMatrix();

		if ( _game->useShaders && _game->enableShaders)
		{
			glEnable (GL_VERTEX_PROGRAM_ARB);
			GLExtensions::get()->glBindProgramARB (GL_VERTEX_PROGRAM_ARB, VertexShaderHandles [3]);
					
			if (registerForTimeValue [3] >= 0) {
				float timeValue [4] = {getTimeInSeconds (), 0.0, 0.0, 0.0}; 
				GLExtensions::get()->glProgramLocalParameter4fvARB (GL_VERTEX_PROGRAM_ARB, registerForTimeValue [3], &timeValue [0]);
				}

			glEnable (GL_FRAGMENT_PROGRAM_ARB);
			GLExtensions::get()->glBindProgramARB (GL_FRAGMENT_PROGRAM_ARB, PixelShaderHandles [3]);
		}

		glFrontFace (GL_CW);
		glColor3f (0.03f, 0.53f, 0.96f);
		glTranslated (0.0, 0.5, 0.0);
		glutSolidTeapot (0.5);
		glTranslated(0.0,-0.5,0.0);
		glFrontFace (GL_CCW);

		if( _game->useShaders )
		{
			glDisable (GL_FRAGMENT_PROGRAM_ARB);
			glDisable (GL_VERTEX_PROGRAM_ARB);
		}

		glColor3f(0.5,0.5,0.5);
		glLineWidth(1.0);
		
		glBegin(GL_LINES);
		for(int x=-20; x < 21; x++) {
			glVertex3d(x/2, 0, -10);
			glVertex3d(x/2, 0, 10);
		}
		for(int z = -20; z < 21; z++) {
			glVertex3d(-10, 0, z/2);
			glVertex3d(10, 0, z/2);
		}
		glEnd();

		_camera->draw();

	_followCamera->popMatrix();
	//_camera->popMatrix();		

	
	begin2DDrawing(800,600);
	char buffer[1024];
	sprintf(buffer, "Camera: [%3.f, %3.f, %3.f] <%3.f, %3.f, %3.f>",
		_camera->position.x,
		_camera->position.y,
		_camera->position.z,
		_camera->rotation.x,
		_camera->rotation.y,
		_camera->rotation.z);
	RGBA foreColor = {1, 0, 0, 1};
	foreColor.apply();
	_font->drawText(700, 5, "FPS: %4.f", timer->fps());
	_font->drawText(5,5,buffer);

	end2DDrawing();
	glColor4d(1.0,1.0,1.0,1.0);
}

void DefaultUI::showCursor(bool show) {
}

void DefaultUI::dispose() {
	::log("\n\nDefaultUI Disposed");
	delete _camera;
	delete _followCamera;
	
	cleanupFonts();
	_isDisposed = true;
}