#include "includes.h"

bool EditBox::processKeyDown(unsigned char key, int x, int y) {
	if (!m_visible) return false;
	if (!keyIsValid(key)) return false;

	m_time = 0;					// Show the caret while
	m_showCaret = true;			// typing

	if (key == 8) {				// Backspace
		if (m_caret > 0) {
			string temp;
			temp = m_text.substr(0, m_caret - 1);
			temp += m_text.substr(m_caret);
			m_text = temp;
			m_caret --;
		}
	} else if (key == 127) {	// Delete
		if (m_caret < m_text.length()) {
			string temp;
			temp = m_text.substr(0, m_caret );
			temp += m_text.substr(m_caret + 1);
			m_text = temp;
		}
	} else if (key == 13) {		// Enter
		if (m_enterProc != NULL) {
			return m_enterProc(this, m_text);
		}
	} else {
		stringIterator selStart = m_text.begin() + m_caret;
		m_text.insert(selStart, key);
		m_caret ++;
	}
	return true;
}

bool EditBox::processSpecialKeyDown(int key, int x, int y) {
	if (!m_visible) return false;
	if (key == GLUT_KEY_LEFT) {
		m_caret--;
		if (m_caret < 0) m_caret = 0;

	} else if (key == GLUT_KEY_RIGHT) {
		m_caret++;
		if (m_caret > m_text.length()) m_caret = (int)m_text.length();
	} else if (key == GLUT_KEY_HOME) {
		m_caret = 0;
	} else if (key == GLUT_KEY_END) {
		m_caret = (int)m_text.length();
	} else {
		return false;
	}
	return true;
}

bool EditBox::keyIsValid(unsigned char key) {
	if (key == 8) return true;		// Backspace
	if (key == 13) return true;		// Enter
	
	if (key >= 32 && key <= 127)
		return true;
	
	return false;
}

void EditBox::tick() {
	if (!m_visible) return;

	m_time += Timer::Get()->deltaTime;
	if (m_time >= BLINK_SPEED) {
		m_time = 0;
		m_showCaret = !m_showCaret;
	}
}

void EditBox::draw() {
	if (!m_visible) return;

	glDisable(GL_TEXTURE_2D);
	glLineWidth(1.0);

	glEnable(GL_BLEND);
	
	if (m_backColor.a > 0) {
		m_backColor.apply();
		glBegin(GL_POLYGON);		
			glVertex2i(m_position.x, m_position.y);
			glVertex2i(m_position.x + m_size.x, m_position.y);
			glVertex2i(m_position.x + m_size.x, m_position.y + m_size.y);
			glVertex2i(m_position.x, m_position.y + m_size.y);
		glEnd();
	}
	

	if (m_borderColor.a > 0) {
		m_borderColor.apply();
		glBegin(GL_LINE_LOOP);
			glVertex2i(m_position.x, m_position.y);
			glVertex2i(m_position.x + m_size.x, m_position.y);
			glVertex2i(m_position.x + m_size.x, m_position.y + m_size.y);
			glVertex2i(m_position.x, m_position.y + m_size.y);
		glEnd();
	}

	if (m_font != NULL) {
		m_foreColor.apply();
		m_font->drawText( m_position.x + 5, m_position.y + 5, m_text.c_str());
	}

	if (m_showCaret) {
		m_caretColor.apply();
		Point avg = m_font->getAvgerageCharSize();
		glBegin(GL_LINES);
			glVertex2i( (m_position.x + 6) + 8 * m_caret + 1, m_position.y + 3);
			glVertex2i( (m_position.x + 5) + 8 * m_caret + 1, m_position.y + 3 + (avg.y - 3));
		glEnd();
	}
	glDisable(GL_BLEND);
	glColor4f(1,1,1,1);
}