#pragma once

#define BLINK_SPEED 0.5
#define CLAMP(value, min, max) (value < min ? min : (value > max ? max : value))

class EditBox;

typedef bool (CALLBACK* EditBoxEnterProc)(EditBox*, string);
typedef string::iterator stringIterator;

class EditBox :
	public UIElement, IInputConsumer
	
{
private:
	string m_text;
	UserInterface *m_parent;
	Font *m_font;
	
	unsigned int m_caret;
	
	RGBA m_backColor;
	RGBA m_borderColor;
	RGBA m_foreColor;
	RGBA m_caretColor;

	float m_time;
	bool m_showCaret;
public:
	EditBoxEnterProc m_enterProc;

public:
	virtual bool processKeyUp(unsigned char key, int x, int y) {return false;};
	virtual bool processKeyDown(unsigned char key, int x, int y);
	
	virtual bool processSpecialKeyUp(int key, int x, int y) {return false;};
	virtual bool processSpecialKeyDown(int key, int x, int y);

	virtual bool processMouseMove(int x, int y) {return false;};
	virtual bool processMouseClick(int button, int state, int x, int y) {return false;};

	void tick();
	void draw();

	void onShow() {};
	void onHide() {};

public:
	string getText() {return m_text;};
	void setText(string text) {m_text = text; m_caret = (int)m_text.length();};	

	Font *getFont() {return m_font;};
	void setFont(const char* font, int height) {if (m_font) delete m_font; m_font = new Font(font, height);};

	int getCaretPosition() {return m_caret;};
	void setCaretPosition(int position) {m_caret = CLAMP(position, 0, (int)m_text.length() );};

	RGBA getBackColor() {return m_backColor;};
	void setBackColor(RGBA color) {m_backColor = color;};

	RGBA getBorderColor() {return m_borderColor;};
	void setBorderColor(RGBA color) {m_borderColor = color;};

	RGBA getForeColor() {return m_foreColor;};
	void setForeColor(RGBA color) {m_foreColor = color;};

	RGBA getCaretColor() {return m_caretColor;};
	void setCaretColor(RGBA color) {m_caretColor = color;};

private:
	bool keyIsValid(unsigned char key);

public:
	EditBox() {
		ZeroMemory(&m_position, sizeof(m_position));
		m_size.x = 200;
		m_size.y = 15;

		m_font = NULL;
		m_caret = 0;
		m_text = "";
		m_enterProc = NULL;
		m_parent = NULL;
		m_visible = false;
		m_foreColor.set(0,0,0,1);
		m_backColor.set(1,1,1,0.5);
		m_borderColor.set(0,0,0,1);
		m_caretColor.set(1,0,0,1);
	};

	~EditBox(void) {
		if (m_font) {
			delete m_font;
			m_font = NULL;
		}
	};
};
