#include "includes.h"
#include "enginevariables.h"

/*static*/ EngineVariables EngineVariables::s_variables;

EngineVariables::EngineVariables(void)
{
}

EngineVariables::~EngineVariables(void) {

	EventMapIterator mapIterator = m_events.begin();
	while(mapIterator != m_events.end()) {
		EventProcList *epl = mapIterator->second;

		for(unsigned int eventIndex = 0; eventIndex < epl->size(); eventIndex++) {
			delete (*epl)[eventIndex];
		}
		epl->clear();
		delete epl;
		mapIterator++;
	}
	m_events.clear();
	m_variables.clear();
}

bool EngineVariables::getVariable(string variable, string* pString) {
	EngineVariableMapIterator pos = m_variables.find(variable);
	if (pos != m_variables.end())
	{
		*pString = pos->second;
		return true;
	}
	else
	{
		return false;;
	}
}
bool EngineVariables::getVariable(string variable, float* value) {
	string svalue;
	if(getVariable(variable, &svalue))
	{
		sscanf(svalue.c_str(), "%f", value);
		return true;
	}
	return false;
}
bool EngineVariables::getVariable(string variable, int* value) {
	string svalue;
	if(getVariable(variable, &svalue))
	{
		sscanf(svalue.c_str(), "%d", value);
		return true;
	}
	return false;

}
bool EngineVariables::getVariable(string variable, bool* value) {
	string svalue;
	if(getVariable(variable, &svalue))
	{
		if( svalue == "true")
			*value = true;
		else
			*value = false;
		return true;
	}
	return false;

}

void EngineVariables::setVariable(string variable, string value) {
	EngineVariableMapIterator pos = m_variables.find(variable);
	if (pos != m_variables.end()) {
		pos->second = value;
	} else {
		m_variables.insert(EngineVariableDef(variable, value));
	}
	onVariableChanged(variable, value);
}
void EngineVariables::setVariable(string variable, float value) {
	char buffer[10];
	sprintf(buffer, "%.4f", value);
	
	string s(buffer);
	setVariable(variable, s);
}

void EngineVariables::setVariable(string variable, int value) {
	char buffer[10];
	sprintf(buffer, "%d", value);
	
	string s(buffer);
	setVariable(variable, s);
}

void EngineVariables::setVariable(string variable, bool value) {
	if( value)
		setVariable(variable, string("true"));
	else
		setVariable(variable, string("false"));
}

void EngineVariables::unsetVariable(string variable) {
	EngineVariableMapIterator pos = m_variables.find(variable);
	if (pos != m_variables.end()) {
		m_variables.erase(pos);
	}
	onVariableChanged(variable, "");
}

void EngineVariables::addCallback(string variable, void *object, OnEngineVariableChangeProc proc) {
	
	VariableChangedEventInfo *vcei = new VariableChangedEventInfo();
	vcei->object = object;
	vcei->proc = proc;
	
	EventMapIterator iterator = m_events.find(variable);
	if (iterator != m_events.end()) {
		EventProcList *epl = iterator->second;
		epl->push_back(vcei);
	} else {
		EventProcList *epl = new EventProcList();
		epl->push_back(vcei);
		m_events.insert(VariableEvent(variable, epl));
	}
}

void EngineVariables::onVariableChanged(string variable, string newValue) {
	EventMapIterator iterator = m_events.find(variable);
	if (iterator != m_events.end()) {
		EventProcList *epl = iterator->second;
		for(unsigned int eventIndex = 0; eventIndex < epl->size(); eventIndex++) {
			VariableChangedEventInfo *vcei = (*epl)[eventIndex];
			OnEngineVariableChangeProc proc = vcei->proc;
			proc(vcei->object, variable, newValue);
		}
	}
}