#pragma once

// Engine default variables
#define	FILELOGGING		"file_logging"
#define CONSOLELOGGING	"console_logging"

typedef map<string, string> EngineVariableMap;
typedef EngineVariableMap::iterator EngineVariableMapIterator;
typedef pair <string, string> EngineVariableDef;

typedef void (CALLBACK* OnEngineVariableChangeProc)(void *, string, string);

struct VariableChangedEventInfo {
	void* object;
	OnEngineVariableChangeProc proc;
};

typedef vector<VariableChangedEventInfo*> EventProcList;

typedef map<string, EventProcList*> EventMap;
typedef EventMap::iterator EventMapIterator;
typedef pair<string, EventProcList*> VariableEvent;

class EngineVariables {

protected:
	static EngineVariables s_variables;
	
	EngineVariableMap m_variables;	
	EventMap m_events;

	EngineVariables(void);
	virtual ~EngineVariables(void);

public:
	static EngineVariables *get() {return &s_variables;}

	// Variable Getters
	bool getVariable(string variable, string* value);
	bool getVariable(string variable, float* value);	
	bool getVariable(string variable, int* value);
	bool getVariable(string variable, bool* value);
	
	// Variable Setters
	void setVariable(string variable, string value);
	void setVariable(string variable, float value);
	void setVariable(string variable, int value);
	void setVariable(string variable, bool value);
	// Variable Unsetters
	void unsetVariable(string variable);

	// Event Handlers
	void addCallback(string variable, void* object, OnEngineVariableChangeProc proc);
	void removeCallback(string variable, void* object) {}
	void onVariableChanged(string variable, string newValue);

};
