#include "includes.h"

FollowCamera::FollowCamera(void) {
	m_leadDistance = 0.0;
	m_maxSpeed = 15.0;
	m_pTarget = NULL;
	m_up.x = 0.0; m_up.y = 1.0; m_up.z = 0.0;
	m_viewHeight = 2.0;

	// Default 5 meters viewing distance
	m_viewDistance = 5.0;

	m_currentPosition.x = 0.0;
	m_currentPosition.y = 0.0;
	m_currentPosition.z = 0.0;
}

FollowCamera::~FollowCamera(void) {
}


void FollowCamera::tick() {

	// Make sure there is a target
	if( m_pTarget == NULL)
		return;

	double deltaTime = Timer::Get()->deltaTime;
	Point position;
	Point rotation;
	m_pTarget->getPosition(&position);
	m_pTarget->getRotation(&rotation);

	// Calculate the point to look at and the
	// point where the camera will be

	// Get the point where the orientation will look at
	RPYOrientation o;
	Point front;
	Point at;
	o.setPitch(rotation.x);
	o.setYaw(rotation.y);
	o.setRoll(rotation.z);
	o.getFront(&front);
	
	// Calculate position - front * leadDistance...
	at = position + front * m_leadDistance;

	// Get the position to place the camera at
	m_up.normalize();
	Point eye = position + m_up * m_viewHeight - front * m_viewDistance;
	// Move to the correct position
	double sqDist = eye.squaredDistanceTo(m_currentPosition);
	if( sqDist > (m_maxSpeed * deltaTime) * (m_maxSpeed * deltaTime) )
	{
		Point delta = eye - m_currentPosition;
		m_currentPosition += delta * ( (m_maxSpeed * deltaTime ) / (sqrt(sqDist)));
	}
	else
	{
		m_currentPosition = eye;
	}

	// Create the TargetOrientation
	TargetOrientation to;
	to.lookAt(at, m_currentPosition, m_up);

	/*glPushMatrix();
	glLoadIdentity();
	gluLookAt( m_currentPosition.x, m_currentPosition.y, m_currentPosition.z, at.x, at.y, at.z, m_up.x, m_up.y, m_up.z);
	glGetDoublev(GL_MODELVIEW_MATRIX, (GLdouble*)&m_cameraMatrix);
	glPopMatrix();*/

	m_cameraMatrix.setToIdentity();
	// Create the camera matrix	
	to.getInverseMatrix(&m_cameraMatrix);
	// Translate the matrix by the reverse of the position of the camera
	m_cameraMatrix.translateBy(-m_currentPosition);
	
}

void FollowCamera::pushMatrix()
{
	glPushMatrixd(m_cameraMatrix);
}

void FollowCamera::popMatrix()
{
	glPopMatrix();
}

void FollowCamera::draw() {


}

void FollowCamera::moveBy(double x, double y, double z) {	
}
void FollowCamera::moveBy(Point offset) {
}

void FollowCamera::moveTo(double x, double y, double z) {
	m_currentPosition.x = x;
	m_currentPosition.y = y;
	m_currentPosition.z = z;
}
void FollowCamera::moveTo(Point position) {
	moveTo(position.x, position.y, position.z);
}

void FollowCamera::rotateBy(double x, double y, double z) {
}
void FollowCamera::rotateBy(Point offset) {
}

void FollowCamera::rotateTo(double x, double y, double z) {
}
void FollowCamera::rotateTo(Point position) {
}
