#include "includes.h"
#include "binaryfilestream.h"
#include "transformations.h"
#include "points.h"
#include "font.h"

Font::Font(const char *fontName, int size, bool bold, bool italic) {
	m_isDisposed = false;
	initialize(fontName, size, bold, italic);
}

void Font::initialize(const char* fontName, int size, bool bold, bool italic) {
	m_fontBase = glGenLists(FONT_RANGE_LENGTH);
	HFONT font = CreateFont(
		size,
		0,
		0,
		0,
		bold ? FW_BOLD : FW_NORMAL,
		italic ? TRUE : FALSE,
		FALSE,
		FALSE,
		ANSI_CHARSET,
		OUT_TT_PRECIS,
		CLIP_DEFAULT_PRECIS,
		ANTIALIASED_QUALITY,
		FF_DONTCARE | DEFAULT_PITCH,
		fontName);
	
	HDC hdc = GetDC(NULL);
	HGDIOBJ oldFont = SelectObject(hdc, font);
	wglUseFontBitmaps (hdc, FONT_RANGE_START, FONT_RANGE_LENGTH, m_fontBase);
	calculateAverageSize(hdc);

	SelectObject(hdc, oldFont);
	DeleteObject(font);
	ReleaseDC(NULL, hdc);
}

void Font::drawText(int posX, int posY, const char* message, ...) {
	char text [256]; va_list parameters;
	if (message == NULL) return;

	//Extract variable length parameters and copy into text as in printf.
	va_start (parameters, message);
	    vsprintf (text, message, parameters);
	va_end (parameters);
		
	glRasterPos2i (posX, posY);
	glPushAttrib (GL_LIST_BIT);								//Pushes the display list bits
	glListBase (m_fontBase - FONT_RANGE_START);				//Sets the base character to 32
	glCallLists ((GLsizei)strlen (text), GL_UNSIGNED_BYTE, text);	//Draws the display list text
	glPopAttrib ();											//Pops the display list bits
}

void Font::calculateAverageSize(HDC hdc) {
	char buffer[] = {"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"};
	RECT r;
	SetRect(&r, 0, 0, 0, 0);
	int height = DrawText(hdc, buffer, (int)strlen(buffer), &r, DT_CALCRECT);
	m_avgSize.x = r.right / strlen(buffer);
	m_avgSize.y = r.bottom;
}