#include "includes.h"

void FreeCamera::updateMatrix() {
}

// ICamera Methods -------------------------------------------------

void FreeCamera::moveBy(double x, double y, double z) {	
	position.x += x;
	position.y += y;
	position.z += z;
}
void FreeCamera::moveBy(Point offset) {
	moveBy(offset.x, offset.y, offset.z);
}

void FreeCamera::moveTo(double x, double y, double z) {
	position.x = x;
	position.y = y;
	position.z = z;
}
void FreeCamera::moveTo(Point position) {
	moveTo(position.x, position.y, position.z);
}

void FreeCamera::rotateBy(double x, double y, double z) {
	rotation.x += x;
	rotation.y += y;
	rotation.z += z;
}
void FreeCamera::rotateBy(Point offset) {
	rotateBy(offset.x, offset.y, offset.z);
}

void FreeCamera::rotateTo(double x, double y, double z) {
	rotation.x = x;
	rotation.y = y;
	rotation.z = z;
}
void FreeCamera::rotateTo(Point position) {
	rotateTo(position.x, position.y, position.z);
}

void FreeCamera::tick() {
	//Deal with control keys... 
	const double translationSpeed = 20.0; //meters per second.
	const double rotationSpeed = 90.0; //degrees per second
	
	double deltaTime = Timer::Get()->deltaTime;

	double frameTranslationSpeed = translationSpeed * deltaTime;
	double frameRotationSpeed = rotationSpeed * deltaTime;

	// Need to re-build the camera matrix each tick to
	// avoid incorrect rotation orders.
	
	cameraMatrix.setToIdentity();

	if (rotateLeft)
		rotation.y += frameRotationSpeed;
	if (rotateRight)
		rotation.y -= frameRotationSpeed;
	if (rotateUp)
		rotation.x += frameRotationSpeed;
	if (rotateDown)
		rotation.x -= frameRotationSpeed;
	if (rollLeft)
		rotation.z += frameRotationSpeed;
	if (rollRight)	
		rotation.z -= frameRotationSpeed;

	Point axis;	
		 
	axis = Point::axisZ();
	cameraMatrix.rotateBy(-rotation.z, axis);

	axis = Point::axisX();
	cameraMatrix.rotateBy(-rotation.x, axis);

	axis = Point::axisY();
	cameraMatrix.rotateBy(-rotation.y, axis);	
	
	if (translateDown)
		position.y -= frameTranslationSpeed;
	if (translateUp)
		position.y += frameTranslationSpeed;

	if (translateLeft) {
//		position.x -= frameTranslationSpeed;
		position.x -= frameTranslationSpeed * cameraMatrix.m11;
		position.y -= frameTranslationSpeed * cameraMatrix.m21;
		position.z -= frameTranslationSpeed * cameraMatrix.m31;
	}
	if (translateRight) {
//		position.x += frameTranslationSpeed;
		position.x += frameTranslationSpeed * cameraMatrix.m11;
		position.y += frameTranslationSpeed * cameraMatrix.m21;
		position.z += frameTranslationSpeed * cameraMatrix.m31;
	}

	if (translateAhead) {
//		position.z += frameTranslationSpeed;
		position.x -= frameTranslationSpeed * cameraMatrix.m13;
		position.y -= frameTranslationSpeed * cameraMatrix.m23;
		position.z -= frameTranslationSpeed * cameraMatrix.m33;
	}
	if (translateBack) {
//		position.z -= frameTranslationSpeed;
		position.x += frameTranslationSpeed * cameraMatrix.m13;
		position.y += frameTranslationSpeed * cameraMatrix.m23;
		position.z += frameTranslationSpeed * cameraMatrix.m33;
	}
	cameraMatrix.translateBy(-position);
}

void FreeCamera::draw() {

	// Create the camera local to world transformation
	Transformation camera;
	camera.translateBy(position);
	Point axis;
	axis = Point::axisY();
	camera.rotateBy(rotation.y, axis);
	axis = Point::axisX();
	camera.rotateBy(rotation.x, axis);
	axis = Point::axisZ();
	camera.rotateBy(rotation.z, axis);

	// Push the world transformation
	glPushMatrix();
	// Apply the local to world transformation after the view transformation
	glMultMatrixd(cameraMatrix.inverse);
	//glMultMatrixd(camera);

	glFrontFace (GL_CCW);
	glColor3f (1.00f, 0.53f, 0.0f);
	glutSolidTeapot (1.0);	
	glFrontFace (GL_CCW);

	// Restore the world transformation
	glPopMatrix();
	
}