
#include "includes.h"

// The orientation class methods
//----------------------

// Gets the front vector
void Orientation::getFront( Point* pPoint )
{
	regenerateMatrix();

//	Point zaxis = Point::axisZ();
//	(*pPoint) = zaxis * m_matrix;

	// The front vector is by default the z-vector until transformed
	//pPoint->x = m_matrix.m13;
	//pPoint->y = m_matrix.m23;
	//pPoint->z = m_matrix.m33;
	pPoint->x = -m_matrix.m31;
	pPoint->y = -m_matrix.m32;
	pPoint->z = -m_matrix.m33;
}
// Gets the side vector
void Orientation::getSide( Point* pPoint )
{
	regenerateMatrix();

	// The side vector is by default the x-vector until transformed
	//pPoint->x = m_matrix.m11;
	//pPoint->y = m_matrix.m21;
	//pPoint->z = m_matrix.m31;
	pPoint->x = m_matrix.m11;
	pPoint->y = m_matrix.m12;
	pPoint->z = m_matrix.m13;
}
// Gets the up vector
void Orientation::getUp( Point* pPoint )
{
	regenerateMatrix();
	
	// The up vector is by default the y-vector until transformed
	//pPoint->x = m_matrix.m12;
	//pPoint->y = m_matrix.m22;
	//pPoint->z = m_matrix.m32;
	pPoint->x = m_matrix.m21;
	pPoint->y = m_matrix.m22;
	pPoint->z = m_matrix.m23;
}

void Orientation::getMatrix( Transformation* m )
{
	regenerateMatrix();

	*m = m_matrix;
}

void Orientation::getInverseMatrix( Transformation* m)
{
	regenerateMatrix();

	// return the transpose, since the inverse of a orthogonal ( orientation ) matrices is its transpose
	m_matrix.getTranspose(m);
}

// RPY Orientation methods
//-----------------

RPYOrientation::RPYOrientation()
{
	m_yaw = 0;
	m_pitch = 0;
	m_roll = 0;
	m_dirty = true;
	regenerateMatrix();
}

// Recreates the matrix based on the current state
void RPYOrientation::regenerateMatrix()
{
	// Dont rebuild unless necessary
	if( !m_dirty )
		return;

	// Copy the matrix on the stack
	glPushMatrix();
	// Load the identity
	glLoadIdentity();
	
	//Create the rotation matrix	
	/*glRotated( m_roll, 0.0, 0.0, 1.0 );		
	glRotated( m_pitch, 1.0, 0.0, 0.0 );
	glRotated( m_yaw, 0.0, 1.0, 0.0);*/	
	glRotated( m_yaw, 0.0, 1.0, 0.0);
	glRotated( m_pitch, 1.0, 0.0, 0.0 );
	glRotated( m_roll, 0.0, 0.0, 1.0 );		



	//Save the result
	glGetDoublev( GL_MODELVIEW_MATRIX, (GLdouble*)&m_matrix );
	glPopMatrix();

	m_dirty = false;

}

// Target orientation methods
//-----------
TargetOrientation::TargetOrientation()
{
	m_at.axisZ();
	m_eye.x = 0.0; m_eye.y = 0.0; m_eye.z = 0.0;
	m_up.axisY();
	// Since the identity matrix is the correct matrix for the previous values
	// the Orientation is not dirty
	m_dirty = false;
}


void TargetOrientation::regenerateMatrix()
{
	if( !m_dirty )
		return;

	// Recalculate the look at matrix
	//------------

	// Aim the zaxis along the view direction
	Point zaxis = m_at - m_eye;
	zaxis.normalize();
	Point xaxis = zaxis.cross(m_up);
	//Point xaxis = m_up.cross(zaxis);	
	xaxis.normalize();
	Point yaxis = xaxis.cross(zaxis);	

	// Create the rotation matrix
	//m_matrix.m11 = xaxis.x; m_matrix.m12 = yaxis.x; m_matrix.m13 = zaxis.x;
	//m_matrix.m21 = xaxis.y; m_matrix.m22 = yaxis.y; m_matrix.m23 = zaxis.y;
	//m_matrix.m31 = xaxis.z; m_matrix.m32 = yaxis.z; m_matrix.m33 = zaxis.z;
	m_matrix.m11 = xaxis.x; m_matrix.m21 = yaxis.x; m_matrix.m31 = -zaxis.x;
	m_matrix.m12 = xaxis.y; m_matrix.m22 = yaxis.y; m_matrix.m32 = -zaxis.y;
	m_matrix.m13 = xaxis.z; m_matrix.m23 = yaxis.z; m_matrix.m33 = -zaxis.z;

	m_dirty = false;
}

// Sets the state of the orientation to align with the path specified
void TargetOrientation::lookAt( Point& at, Point& eye, Point& up )
{
	if( m_at != at )
	{
		m_dirty = true;
		m_at = at;
	}
	if( m_eye != eye )
	{
		m_dirty = true;
		m_eye = eye;
	}
	if( m_up != up )
	{
		m_dirty = true;
		m_up = up;
	}
}