#ifndef ORIENTATION_H
#define ORIENTATION_H


class Orientation
{
protected:
	Transformation m_matrix;
public:
	virtual void regenerateMatrix() = 0;

	void getFront( Point* pPoint );
	void getSide( Point* pPoint );
	void getUp( Point* pPoint );
	
	void getMatrix( Transformation* m );
	void getInverseMatrix( Transformation* m);

};

// A wrapper class for the orientation of an object that defines its position by
// yaw, pitch and roll
//------------------------------
class RPYOrientation : public Orientation
{
private:
	// Records whether we should rebuild the matrix or not
	bool   m_dirty;

	double m_yaw;
	double m_pitch;
	double m_roll;

private:
	virtual void regenerateMatrix();

public:
	RPYOrientation();

	void setYaw(double yaw)		{ if( m_yaw != yaw ) { m_dirty = true; m_yaw = yaw; } }
	void setPitch(double pitch) { if( m_pitch != pitch ) { m_dirty = true; m_pitch = pitch; } }
	void setRoll(double roll)	{ if( m_roll != roll) { m_dirty = true; m_roll = roll; } }
	double getYaw()				{ return m_yaw; }
	double getPitch()			{ return m_pitch; }
	double getRoll()			{ return m_roll; } 

};

// A wrapper class for the orientation of something that defines its orientation by a target, position
// and up vector
//--------------------------
class TargetOrientation : public Orientation
{
private:
	// Records whether we should rebuild the matrix or not
	bool   m_dirty;

	Point m_at;
	Point m_eye;
	Point m_up;
public:
	TargetOrientation();

	virtual void regenerateMatrix();

	void lookAt( Point& p, Point& from, Point& up );
};

#endif