#pragma once

class Scrollbar;

typedef void (CALLBACK *ScrollProc)(Scrollbar*, float);

class Scrollbar :
	public UIElement, IInputConsumer
{
private:
	float m_minValue;
	float m_maxValue;
	float m_currentValue;

	RGBA m_barBackColor;
	RGBA m_barForeColor;

	RGBA m_handleBackColor;
	RGBA m_handleForeColor;

	ScrollProc m_callback;

private:
	void onScroll() {
		if (m_callback != NULL)
			m_callback(this, m_currentValue);
	};
public:
	virtual bool processKeyUp(unsigned char key, int x, int y) {return false;};
	virtual bool processKeyDown(unsigned char key, int x, int y) {return false;};
	
	virtual bool processSpecialKeyUp(int key, int x, int y) {return false;};
	virtual bool processSpecialKeyDown(int key, int x, int y);

	virtual bool processMouseMove(int x, int y) {return false;};
	virtual bool processMouseClick(int button, int state, int x, int y) {return false;};

	virtual void tick();
	virtual void draw();

	virtual void onShow() {};
	virtual void onHide() {};

	void setCallback(ScrollProc proc) {m_callback = proc;};

	RGBA getBarBackColor() {return m_barBackColor;};
	void setBarBackColor(RGBA color) {m_barBackColor = color;};

	RGBA getBarForeColor() {return m_barForeColor;};
	void setBarForeColor(RGBA color) {m_barForeColor = color;};

	RGBA getHandleBackColor() {return m_handleBackColor;};
	void setHandleBackColor(RGBA color) {m_handleBackColor = color;};

	RGBA getHandleForeColor() {return m_handleForeColor;};
	void setHandleForeColor(RGBA color) {m_handleForeColor = color;};


	float getMinValue() {return m_minValue;};
	void setMinValue(float min) {
		if (min <= m_maxValue) m_minValue = min;
		if (m_currentValue < m_minValue) m_currentValue = m_minValue;
		onScroll();
	};

	float getMaxValue() {return m_maxValue;};
	void setMaxValue(float max) {
		if (max >= m_minValue) m_maxValue = max;
		if (m_currentValue > m_maxValue) m_currentValue = m_maxValue;
		onScroll();
	};

	float getCurrentValue() {return m_currentValue;};
	void setCurrentValue(float value) {
		m_currentValue = CLAMP(value, m_minValue, m_maxValue);
		onScroll();
	};
	void setScrollValues(float min, float max, float value) {
		m_minValue = min;
		m_maxValue = max;
		m_currentValue = value;
		onScroll();
	};

	void scrollUp(float value) {
		m_currentValue = CLAMP(m_currentValue - value, m_minValue, m_maxValue);
		onScroll();
	};
	void scrollDown(float value) {
		m_currentValue = CLAMP(m_currentValue + value, m_minValue, m_maxValue);
		onScroll();
	};
	void scrollToMin() {
		m_currentValue = m_minValue;
		onScroll();
	};
	void scrollToMax() {
		m_currentValue = m_maxValue;
		onScroll();
	};
public:
	Scrollbar(void);
	~Scrollbar(void);
};
