
//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include "includes.h"

// Shaders
unsigned int VertexShaderHandles [5];
unsigned int PixelShaderHandles [5];
long registerForTimeValue [5];

void setupShaders () {

	char *vertexShaders [] = {"simple shader.vsh", "wobble shader.vsh", "diffuse lighting.vsh",
		"specular lighting.vsh", "texture.vsh"};
	char *pixelShaders [] = {"simple shader.psh", "wobble shader.psh", "diffuse lighting.psh",
		"specular lighting.psh", "texture.psh"};

	for (long index = 0; index < 5; index++) {
		VertexShaderHandles [index] = loadShader (GL_VERTEX_PROGRAM_ARB, vertexShaders [index]);
		PixelShaderHandles [index] = loadShader (GL_FRAGMENT_PROGRAM_ARB, pixelShaders [index]);
		registerForTimeValue [index] = index == 0 ? -1 : 0;
	}
}

void wrapupShaders () {
	for (long index = 0; index < 5; index++) {
		GLExtensions::get()->glDeleteProgramsARB (1, &VertexShaderHandles [index]);
		GLExtensions::get()->glDeleteProgramsARB (1, &PixelShaderHandles [index]);
	}
}
//*****************************************************************************************//
//                                        Game                                             //
//*****************************************************************************************//

Game *game = NULL;
double DT; 
extern long screenWidth, screenHeight;

//HDC Game::deviceContext; GLuint Game::fontBase; 

void setupOpenGL () {
	glEnable (GL_CULL_FACE); glEnable (GL_DEPTH_TEST); glEnable (GL_TEXTURE_2D);
	glLineWidth (3.0);

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective (40.0, 1.0, 1.0, 100.0); //See resizeWindow for parameter explanation.
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();

	GLfloat lightColor [] = {1.0f, 1.0f, 1.0f, 1.0f}; //white
	glLightModeli (GL_LIGHT_MODEL_LOCAL_VIEWER, 1);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, lightColor);
	glLightf (GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.1f);
	glLightf (GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.05f);
	glEnable (GL_LIGHT0); glEnable (GL_LIGHTING);
	glDisable (GL_LIGHTING); //We can deal with our own lighting.
	glEnable (GL_COLOR_MATERIAL); //Track color.

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	//glEnable(GL_LINE_SMOOTH);

	glClearColor (0.0, 0.0, 0.0, 0.0); //black
	glClearDepth (1.0); //Far
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glDrawBuffer (GL_BACK); glReadBuffer (GL_BACK);
	glEnable (GL_DEPTH_TEST); glDepthMask (GL_TRUE); glDepthFunc (GL_LEQUAL);
	glShadeModel (GL_SMOOTH);	

	glFrontFace (GL_CCW); glCullFace (GL_BACK); glEnable (GL_CULL_FACE);

	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); 
	glDisable (GL_ALPHA_TEST);
	glPolygonOffset (0.0, -3.0);

	//Setup materials.
	GLfloat	frontMaterialDiffuse [4] = {0.2f, 0.2f, 0.2f, 1.0f};
	glMaterialfv (GL_FRONT, GL_DIFFUSE, frontMaterialDiffuse);
	GLfloat	frontMaterialAmbient [4] = {0.8f, 0.8f, 0.8f, 1.0f};
	glMaterialfv (GL_FRONT, GL_AMBIENT, frontMaterialAmbient);
	GLfloat frontMaterialSpecular [4] = {0.1f, 0.1f, 0.1f, 1.0f};
	glMaterialfv (GL_FRONT, GL_SPECULAR, frontMaterialSpecular);
	GLfloat	frontMaterialShininess [] = {70.0f};
	glMaterialfv (GL_FRONT, GL_SHININESS, frontMaterialShininess);
	GLfloat	frontMaterialEmission [4] = {0.0f, 0.0f, 0.0f, 1.0f};
	glMaterialfv (GL_FRONT, GL_EMISSION, frontMaterialEmission);

	GLfloat	backMaterialDiffuse [4] = {0.0f, 0.0f, 0.0f, 1.0f};
	glMaterialfv (GL_BACK, GL_DIFFUSE, backMaterialDiffuse);
	GLfloat	backMaterialAmbient [4] = {0.0f, 0.0f, 0.0f, 1.0f};
	glMaterialfv (GL_BACK, GL_AMBIENT, backMaterialAmbient);
	GLfloat	backMaterialSpecular [4] = {0.0f, 0.0f, 0.0f, 1.0f};
	glMaterialfv (GL_BACK, GL_SPECULAR, backMaterialSpecular);
	GLfloat	backMaterialShininess [] = {0.0f};
	glMaterialfv (GL_BACK, GL_SHININESS, backMaterialShininess);
	GLfloat	backMaterialEmission [4] = {0.0f, 0.0f, 0.0f, 1.0f};
	glMaterialfv (GL_BACK, GL_EMISSION, backMaterialEmission);

	glColorMaterial (GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

	//Use a global default texture environment mode.
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

void Game::setup () {
	setupOpenGL ();
	GLExtensions* pExtensions = GLExtensions::create();
	
	game = new Game ();
	if(pExtensions->get()->shadersAvailable())
	{
		game->enableShaders = true;
		game->useShaders	= true;
		setupShaders();
	}
	else
	{
		game->enableShaders = false;
		game->useShaders	= false;
	}
	World::setup ();
}

void Game::wrapup () {
	if( game->enableShaders )
		wrapupShaders ();
	
	delete game;
	World::wrapup ();

	GLExtensions::dispose();
}

CommandResult CALLBACK Game::ConsoleCommandWatcher(void* object, string command, vector<string> parameters) {
	Game *g = (Game*)object;
	if (command == "level") {
		string filename = parameters[0];
		if (filename == "null") {
			g->loadDefaultWorld();
			Console::Get()->write("World Unloaded");
		} else {
			filename = "..\\Worlds\\" + filename + ".wrl";
			FILE *file = fopen(filename.c_str(), "r");
			if (file != NULL) {
				fclose(file);
				g->loadWorld(filename);
				Console::Get()->write("World Loaded: '%s'", filename.c_str());
			} else {
				Console::Get()->write("File Not Found: '%s'", filename.c_str());
				return CR_ERROR;
			}		
		}
	} else if (command == "quit" || command == "exit") {
		Game::wrapup();
		exit(0);
	} else {
		return CR_NOT_DEFINED;
	}
	return CR_SUCCESS;
}
void CALLBACK Game::OnEngineVariableChanged(void* object, string variable, string newValue) {
	if (variable == "wireframe") {
		if (newValue == "" || newValue == "0") {
			glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
		} else {
			glPolygonMode (GL_FRONT_AND_BACK, GL_LINE);
		}
	}
}

bool Game::loadWorld(string filename) {
	unloadWorld();

	currentWorld = World::load (filename.c_str() ); 
	if (currentWorld != NULL) {
		m_currentUI = new WorldUI(game);
		return true;
	} else {
		m_currentUI = new DefaultUI(game);
		return false;
	}	
}
bool Game::loadWorld() {
	unloadWorld();

	currentWorld = World::load (); 
	if (currentWorld != NULL) {
		m_currentUI = new WorldUI(game);
		return true;
	} else {
		m_currentUI = new DefaultUI(game);
		return false;
	}
}
bool Game::unloadWorld() {
	if (currentWorld != NULL) {
		currentWorld->unload ();
		delete currentWorld;
		currentWorld = NULL;
	}
	if(m_currentUI)	{
		delete m_currentUI;
		m_currentUI = NULL;
	}
	return true;
}
bool Game::loadDefaultWorld() {
	unloadWorld();
	m_currentUI = new DefaultUI(game);
	return true;
}

void Game::tick () {
	Timer::Get()->tick();
	
	ConsoleState cstate = m_console->getState();
	if (cstate == CS_HIDDEN) {
		m_currentUI->tick();
	
		if (currentWorld == NULL) return;
		currentWorld->tick ();
	} else {	
		m_console->tick();
	}
}

void Game::draw () {
	m_currentUI->draw();
	m_console->draw();
}
extern void getWindowInfo();
bool Game::processKeyDown(unsigned char key, int x, int y) {
	
	if (key == '`') {
		getWindowInfo();
		m_console->toggleVisible();	
		ConsoleState cstate = m_console->getState();
		if( cstate != CS_HIDDEN )
		{
			SetCursorPos(400,300);
		}
		return true;
	}
	ConsoleState cstate = m_console->getState();
	if (cstate != CS_HIDDEN) {
		return m_console->processKeyDown(key, x, y);
	} else {
		if (m_currentUI)
			return m_currentUI->processKeyDown(key, x, y);
	}
	return false;
}
bool Game::processKeyUp(unsigned char key, int x, int y) {
	ConsoleState cstate = m_console->getState();
	if (cstate != CS_HIDDEN) {
		return m_console->processKeyUp(key, x, y);
	} else {
		if (m_currentUI)
			return m_currentUI->processKeyUp(key, x, y);
	}
	return false;
}
bool Game::processSpecialKeyDown(int key, int x, int y) {
	ConsoleState cstate = m_console->getState();
	if (cstate != CS_HIDDEN) {
		return m_console->processSpecialKeyDown(key, x, y);
	} else {
		if (m_currentUI)
			return m_currentUI->processSpecialKeyDown(key, x, y);
	}
	return false;
}
bool Game::processSpecialKeyUp(int key, int x, int y) {
	ConsoleState cstate = m_console->getState();
	if (cstate != CS_HIDDEN) {
		return m_console->processSpecialKeyUp(key, x, y);
	} else {
		if (m_currentUI)
			return m_currentUI->processSpecialKeyUp(key, x, y);
	}
	return false;
}

bool Game::processMouseMove(int x, int y) {
	ConsoleState cstate = m_console->getState();
	if (cstate != CS_HIDDEN) {
		return m_console->processMouseMove(x, y);
	} else {
		if (m_currentUI)
			return m_currentUI->processMouseMove(x, y);
	}
	return false;
}
bool Game::processMouseClick(int button, int state, int x, int y) {
	ConsoleState cstate = m_console->getState();
	if (cstate != CS_HIDDEN) {
		return m_console->processMouseClick(button, state, x, y);
	} else {
		if (m_currentUI)
			return m_currentUI->processMouseClick(button, state, x, y);
	}
	return false;
}
