//*****************************************************************************************//
//                                        Game                                             //
//*****************************************************************************************//

#ifndef gameModule
#define gameModule 

class Game : public ITickable, public IDrawable, IInputConsumer {
private:
	static CommandResult CALLBACK ConsoleCommandWatcher(void* object, string command, vector<string> parameters);
	static void CALLBACK OnEngineVariableChanged(void*, string, string);

public:
	Game () {
		currentWorld = NULL;
		m_currentUI = new DefaultUI(this);
		m_console = Console::Get();
		m_console->initialize();
		m_console->addCommandCallback(this, ConsoleCommandWatcher);
		EngineVariables::Get()->addCallback("wireframe", this, OnEngineVariableChanged);
	};
	~Game () {
		m_currentUI->dispose();
		if(currentWorld)
		{
			currentWorld->unload();
			delete currentWorld;
		}
		delete m_currentUI;
	};

	World *currentWorld;
	
	bool loadDefaultWorld();
	bool loadWorld();
	bool loadWorld(string filename);
	bool unloadWorld();

	bool useShaders;
	bool enableShaders;



	UserInterface *m_currentUI;
	Console *m_console;

	bool processKeyDown(unsigned char key, int x, int y);
	bool processKeyUp(unsigned char key, int x, int y);
	bool processSpecialKeyDown(int key, int x, int y);
	bool processSpecialKeyUp(int key, int x, int y);

	bool processMouseMove(int x, int y);
	bool processMouseClick(int button, int state, int x, int y);

	static void setup ();
	static void wrapup ();

	void tick ();
	void draw ();
};


extern Game *game;

#endif