#ifndef GLEXTENSIONS_H
#define GLEXTENSIONS_H

// This class encapsulates all the openGL procedures that may or may not be present on the system
//--------------------

inline void *getAddress (const char *name);

class GLExtensions
{
protected:
	static	GLExtensions*	s_pGLExtensions;

	bool					m_multiTexture;
	bool					m_fragmentPrograms;
	bool					m_vertexPrograms;
	bool					m_vertexBuffers;

public:
	static GLExtensions*	create()		{ dispose(); s_pGLExtensions = new GLExtensions(); return s_pGLExtensions; }
	static GLExtensions*	get()			{ return s_pGLExtensions; }
	static void				dispose()		{ if(s_pGLExtensions) { delete s_pGLExtensions; s_pGLExtensions = NULL; } }

	GLExtensions();
	virtual ~GLExtensions();

	bool	multiTextureAvailable()			{ return m_multiTexture; }
	bool	shadersAvailable()				{ return m_fragmentPrograms && m_vertexPrograms; }
	bool	fragmentProgramsAvailable()		{ return m_fragmentPrograms; }
	bool	vertexProgramsAvailable()		{ return m_vertexPrograms; }
	bool	vertexBuffersAvailable()		{ return m_vertexBuffers; }

	// The actual openGL wrappers
	//------------------------------

	//Multitexture functions
	void glActiveTextureARB(GLenum texture);
	void glClientActiveTextureARB(GLenum target);
	void glMultiTexCoord3fARB(GLenum texture, GLfloat s, GLfloat t, GLfloat r);

	// Vertex buffer extension support
	void glBindBufferARB(GLenum target, GLuint buffer);
	void glGenBuffersARB(GLsizei n, GLuint *buffers);
	void glBufferDataARB(GLenum target, GLsizeiptrARB size, const GLvoid *data, GLenum usage);
	void glDeleteBuffersARB(GLsizei n, GLuint *buffers);

	// Fragment and Vertex Program support
	void glProgramStringARB(GLenum target, GLenum format, GLsizei len, const GLvoid *string);
	void glBindProgramARB(GLenum target, GLuint program);
	void glDeleteProgramsARB(GLsizei n, const GLuint *programs);
	void glGenProgramsARB(GLsizei n, GLuint *programs);
	void glProgramLocalParameter4fvARB(GLenum target, GLuint index, const GLfloat* params);

	

};

#endif