
#ifndef interfaceModule
#define interfaceModule

struct Point;
class Font;

class BinaryFileStreamReader;
class BinaryFileStreamWriter;


class IDisposable {
public:
	virtual ~IDisposable() {};

	virtual void dispose() = 0;
};

class ITickable {
public:
	virtual ~ITickable() {};

	virtual void tick() = 0;
};
class IDrawable {
public:
	virtual ~IDrawable() {};

	virtual void draw() = 0;
};

class IInputConsumer {
public:
	virtual ~IInputConsumer() {};

	virtual bool processKeyUp(unsigned char key, int x, int y) = 0;
	virtual bool processKeyDown(unsigned char key, int x, int y) = 0;
	
	virtual bool processSpecialKeyUp(int key, int x, int y) = 0;
	virtual bool processSpecialKeyDown(int key, int x, int y) = 0;

	virtual bool processMouseMove(int x, int y) = 0;
	virtual bool processMouseClick(int button, int state, int x, int y) = 0;
};


class ISerializable {
public:
	virtual ~ISerializable() {};

	virtual bool serialize(BinaryFileStreamWriter *output) = 0;
	virtual bool deserialize(BinaryFileStreamReader *input) = 0;
};

class IMoveable {
public:
	virtual ~IMoveable() {};

	virtual void moveBy(Point offset) = 0;
	virtual void moveBy(double x, double y, double z) = 0;

	virtual void moveTo(Point position) = 0;
	virtual void moveTo(double x, double y, double z) = 0;

	virtual void rotateBy(Point offset) = 0;
	virtual void rotateBy(double x, double y, double z) = 0;
	
	virtual void rotateTo(Point position) = 0;
	virtual void rotateTo(double x, double y, double z) = 0;

	virtual void getPosition(Point* position) = 0;
	virtual void getRotation(Point* rotation) = 0;
};

class IObject : public ITickable, public IDrawable, public ISerializable, public IDisposable {
public:
	virtual ~IObject() {};
};
DECLARECOLLECTION(IObject);

class ILocalObject : public IObject, public IMoveable {
public:
	virtual ~ILocalObject() {};
public:
	//virtual void processProperties() = 0;
};
DECLARECOLLECTION(ILocalObject);

class UserInterface : public IInputConsumer, public ITickable, public IDrawable, public IDisposable {
public:
	virtual ~UserInterface() {};

	virtual void showCursor(bool show) = 0;	

	void begin2DDrawing(int width, int height) {
		glMatrixMode (GL_PROJECTION);
		glPushMatrix ();
		glLoadIdentity ();
		glOrtho (0.0,(GLfloat)width,0.0,(GLfloat)height, -10.0f, 1000.0f);
		glMatrixMode (GL_MODELVIEW);
		glPushMatrix ();
		glLoadIdentity ();
	};
	void end2DDrawing() {
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
	};
protected:
	//True if key is pressed.  Windows Specific
	inline bool keyIsDown (long key) {return (GetAsyncKeyState (key) & 0x8000) != 0;}
};

class ICamera : public IMoveable, public ITickable, public IDrawable {
public:
	virtual ~ICamera() {};

	virtual void pushMatrix() = 0;
	virtual void popMatrix() = 0;
};
#endif
