
//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include "includes.h"
#include "enginevariables.h"
#include "..\vd.h"
#include "logging.h"

//*****************************************************************************************//
//                                       Timing                                            //
//*****************************************************************************************//

long timeNow () {return timeGetTime ();} //Time in milliseconds.

//*****************************************************************************************//
//                                Debugging Facilities                                     //
//*****************************************************************************************//

void clearLog () { 
	static char logFileName [255] = "";
	if (*((char *) &logFileName) == '\0') _fullpath ((char *) &logFileName, ".\\log.txt", sizeof (logFileName));
	FILE *file = fopen ((char *) &logFileName, "w"); if (file == NULL) return;
	fclose (file);					 
}

#define setupStaticBuffer() \
	static char buffer [500]; va_list parameters; \
	va_start (parameters, message); \
	vsprintf (buffer, message, parameters); \
	va_end (parameters) 

void log (const char *message, ...) { 
	//Example use: log ("\nInteger %d float %4.2f hex %8.8x.", 10, 1.2, 16);
	setupStaticBuffer ();
	bool s;
	if( EngineVariables::get()->getVariable( FILELOGGING, &s ) )
	{
		if(s)
		{
			static char logFileName [255] = "";
			if (*((char *) &logFileName) == '\0') 
				_fullpath ((char *) &logFileName, ".\\log.txt", sizeof (logFileName));
			//MessageBox (GetActiveWindow (), buffer, "      Message      ", MB_OK);
			FILE *file = fopen ((char *) &logFileName, "a");
			if (file == NULL) return;
			fprintf (file, "%s", buffer);
			fprintf(file, "\n");
			fclose (file);
		}
	}

	//Log to console if enabled
	if( EngineVariables::get()->getVariable( CONSOLELOGGING, &s ) )
	{
		if(s && VDConsole::get())
		{
			VDConsole::get()->printToConsole(string(buffer));
		}
	}
		
}

void prompt (const char *message, ...) { 
	//Use like log.
	setupStaticBuffer ();
	MessageBox (/*GetActiveWindow ()*/ NULL, buffer, "      Message      ", MB_OK);
}

void quit (const char *message, ...) { 
	//Use like log.
	setupStaticBuffer ();
	log ("\n%s", buffer);
	MessageBox (/*GetActiveWindow ()*/ NULL, buffer, "      Message      ", MB_OK);
	exit (0);
}
 
char *asString (const char *message, ...) { 
	//Use like log.
	setupStaticBuffer ();
	return &buffer [0]; //Careful: Two asStrings in a row use the same buffer.
}
