
// ***********************************************
// This file contains the starting point for the application.
// It uses the GLUT toolkit to create and receive windows messages.
// 
// The DEVS window is interfaced to GLUT through the code in this file.
// 

#include "includes.h"
#include "enginevariables.h"
#include "..\vd.h"
#include "..\vc.h"

HWND hWnd = NULL;
VDMainWindow* vdwin = NULL;

static VD_KEY convertGLUTKeyToVDKey( int character )
{
	if( character == GLUT_KEY_F1 )
		return VD_F1;
	if( character == GLUT_KEY_F2 )
		return VD_F2;
	if( character == GLUT_KEY_F3 )
		return VD_F3;
	if( character == GLUT_KEY_F4 )
		return VD_F4;
	if( character == GLUT_KEY_F5 )
		return VD_F5;
	if( character == GLUT_KEY_F6 )
		return VD_F6;
	if( character == GLUT_KEY_F7 )
		return VD_F7;
	if( character == GLUT_KEY_F8 )
		return VD_F8;
	if( character == GLUT_KEY_F9 )
		return VD_F9;
	if( character == GLUT_KEY_F10 )
		return VD_F10;
	if( character == GLUT_KEY_F11 )
		return VD_F11;
	if( character == GLUT_KEY_F12 )
		return VD_F12;
	if( character == GLUT_KEY_LEFT )
		return VD_LEFT;
	if( character == GLUT_KEY_RIGHT )
		return VD_RIGHT;
	if( character == GLUT_KEY_UP )
		return VD_UP;
	if( character == GLUT_KEY_DOWN )
		return VD_DOWN;
	if( character == GLUT_KEY_PAGE_UP )
		return VD_PAGE_UP;
	if( character == GLUT_KEY_PAGE_DOWN )
		return VD_PAGE_DOWN;
	if( character == GLUT_KEY_HOME )
		return VD_HOME;
	if( character == GLUT_KEY_INSERT )
		return VD_INSERT;
	if( character == GLUT_KEY_END )
		return VD_END;
	
	return character;
}

void resizeWindow (int width, int height) 
{
	if(vdwin)
	{
		VDResizeEvent e(width, height);
		vdwin->processEvent(&e);
		VDRedisplayEvent v;
		vdwin->processEvent(&v);		
	}
}

void displayWindow () 
{
	if(vdwin)
	{
		// Delete all the disposables on a display
		VDGarbageCan::get()->empty();

		VDRedisplayEvent e;
		vdwin->processEvent(&e);
	}
}

void idle () 
{
}

void visibilityChanged (int visible) 
{
}

void specialKeyPressed (int character, int x, int y) 
{
	if(vdwin)
	{
		VDKey key(convertGLUTKeyToVDKey(character));
		if( glutGetModifiers() == GLUT_ACTIVE_SHIFT )
			key.applyShift();
		if( glutGetModifiers() == GLUT_ACTIVE_ALT )
			key.applyAlt();
		if( glutGetModifiers() == GLUT_ACTIVE_CTRL )
			key.applyControl();
		VDKeyDownEvent e(key);
		vdwin->processEvent(&e);
	}
}

void specialKeyReleased (int character, int x, int y) 
{
	if(vdwin)
	{
		VDKey key(convertGLUTKeyToVDKey(character));
		VDKeyUpEvent e(key);
		vdwin->processEvent(&e);
	}
}
    
void normalKeyPressed (unsigned char character, int x, int y) 
{
	if(vdwin)
	{
		VDKey* key = NULL;

		// **HACK** control c == 3 for some reason
		if( glutGetModifiers() == GLUT_ACTIVE_CTRL )
		{
			if( character == 3 )
			{
				key = new VDKey(VD_C);
			}
			else if( character == 26 )
			{
				key = new VDKey(VD_Z);
			}
			else if( character == 24 )
			{
				key = new VDKey(VD_X);
			}
			else if( character == 22 )
			{
				key = new VDKey(VD_V);
			}
			else if( character == 25 )
			{
				key = new VDKey(VD_Y);
			}
			else if( character == 1 )
			{
				key = new VDKey(VD_A);
			}
			else if( character == 6 )
			{
				key = new VDKey(VD_F);
			}
		}

		if( !key )
			key = new VDKey(character & VD_ASCII_MASK);
		
		if( glutGetModifiers() == GLUT_ACTIVE_SHIFT )
			key->applyShift();
		if( glutGetModifiers() == GLUT_ACTIVE_ALT )
			key->applyAlt();
		if( glutGetModifiers() == GLUT_ACTIVE_CTRL )
			key->applyControl();
		VDKeyDownEvent e(*key);
		vdwin->processEvent(&e);
	}
}

void normalKeyReleased (unsigned char character, int x, int y) 
{
	if(vdwin)
	{
		VDKey key(character);
		VDKeyUpEvent e(key);
		vdwin->processEvent(&e);
	}
}

void mousePressed (int button, int state, int x, int y) 
{
	if(vdwin)
	{
		int buttonNum;
		if( button == GLUT_LEFT_BUTTON )
			buttonNum = VD_BUTTON_LEFT;
		else if( button == GLUT_RIGHT_BUTTON )
			buttonNum = VD_BUTTON_RIGHT;
		else if( button == GLUT_MIDDLE_BUTTON )
			buttonNum = VD_BUTTON_MIDDLE;

		VDMouseButtonEvent* pE = NULL;
		if(state == GLUT_UP)
			pE = new VDMouseUpEvent(buttonNum, x, y);
		else
			pE = new VDMouseDownEvent(buttonNum, x, y);

		vdwin->processEvent(pE);

		if( pE)
		{
			delete pE;
			pE = NULL;
		}
	}
}

void mouseMoved (int x, int y) 
{
	if(vdwin)
	{
		VDMouseMoveEvent e(x, y);
		vdwin->processEvent(&e);
	}

}

int main (int parametersSize, char **parameters) {	
	
	// clear the log
	clearLog();

	// Setup GLUT Window
	glutInitDisplayMode (GLUT_RGBA | GLUT_ALPHA | GLUT_DEPTH | GLUT_DOUBLE | GLUT_STENCIL | GLUT_MULTISAMPLE);
	glutInitWindowSize (800, 600);
	glutInit (&parametersSize, parameters);
	glutCreateWindow ("DEVSView");

	// Get the window handle for any window specific functions
	hWnd = FindWindow("GLUT", "OpenGL Engine");

	// Create the default environment variables
	EngineVariables::get()->setVariable(CONSOLELOGGING, true);
	EngineVariables::get()->setVariable(FILELOGGING, true);

	// Create the garbage can
	if(!VDGarbageCan::create())
	{
		::log("Unable to create garbage collector");
		return 0;
	}

	// Create the ViewerDisplay Window
	vdwin = VDMainWindow::create(800, 600);
	if( !vdwin )
	{
		::log("Unable to create Viewer Display Window");
		return 0;
	}

	// Setup the GLExtensions
	if(!GLExtensions::create())
	{
		::log("Unable to create glextension object");
		return 0;
	}

	// Create the theme store
	if(!VDThemeStore::create())
	{
		::log("Unable to create Theme store");
		return 0;
	}

	// Create the simulation control object
	if(!VCSimulation::create())
	{
		::log("Unable to create simulation control object");
		return 0;
	}


	// Specify GLUT function handlers.
	glutDisplayFunc (displayWindow);
	glutReshapeFunc (resizeWindow);
	glutKeyboardFunc (normalKeyPressed);
	glutSpecialFunc (specialKeyPressed);
	glutMouseFunc (mousePressed);
	glutMotionFunc (mouseMoved);
	glutVisibilityFunc (visibilityChanged);
    glutPassiveMotionFunc (mouseMoved); 
    glutSpecialUpFunc (specialKeyReleased);
    glutKeyboardUpFunc (normalKeyReleased);

	// Log the startup succeeded
	::log("Startup successful...");

	// Start the process
	glutMainLoop ();

	// Close down the resources by executing the exit command
	VDExitCommand e;
	e.execute();

	return 0;
}
