
//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include "binaryfilestream.h"
#include "transformations.h"
#include "points.h"

//*****************************************************************************************//
//                                   Points/Vectors                                       //
//*****************************************************************************************//

Point Point::operator * (const Transformation &t) const {
	//or Vector = Vector * transformation.
	Point result = {
		x*t.m11 + y*t.m21 + z*t.m31 + t.m41, 
		x*t.m12 + y*t.m22 + z*t.m32 + t.m42, 
		x*t.m13 + y*t.m23 + z*t.m33 + t.m43}; 
	float w = x*t.m14 + y*t.m24 + z*t.m34 + t.m44;
	float absoluteW = absolute (w);
	if ((absoluteW - 1.0) < epsilon ()) return result; //usually, w is 1.0
	if (absoluteW < epsilon ()) return result * inverseEpsilon (); //w is 0.0 (use epsilon).
	return result * (1.0 / w); //the complicated case.
}

bool Point::deserialize(BinaryFileStreamReader *input) {
	string tag;
	input->readTag(&tag);
	x = (float)input->readFloat();
	y = (float)input->readFloat();
	z = (float)input->readFloat();
	return true;
}