
//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include "includes.h"

//*****************************************************************************************//
//                                      TextureList                                        //
//*****************************************************************************************//

/*static*/ TextureList TextureList::s_textureList;

// Load the texture names from a file
bool TextureList::LoadTextureNames( BinaryFileStreamReader *file )
{	

	// buffer allocation
	string buffer;

	// Read the identifier
	file->readTag(&buffer); //TEX
	if ( buffer != TAG_TEXTURELIST )
	{
		::log("Tried to load a texture list chunk from something other than a texture list");
		return false;		
	}

	// Read the number of textures
	
	int numTextures = file->readInteger();
	if(numTextures < 1 || numTextures > 2048)
	{
		::log("Invalid number of textures specified in file texture list");
		return false;
	}

	// Read the texture names
	string textureName;
	int i;
	for( i = 0; i < numTextures; i++)
	{
		file->readString(&textureName);
		if( textureName.size() == 0 )
		{
			::log("Invalid Texture name encountered");
		}
		m_textureNames.push_back(textureName);
	}

	return true;
}

// Save the texture names to a file
bool TextureList::SaveTextureNames( ::ofstream &file )
{

	// Write the identifier
	file << "\n" << TEXTURELIST_FILEID;

	// Write the number of textures
	file << "\n" << (unsigned int)m_textureNames.size();

	// Write the texture names
	unsigned int i;
	for( i = 0; i < m_textureNames.size(); i++)
	{
		file  << "\n" << m_textureNames[i].c_str();
	}

	return true;
}

// Attempt to load the textures into system memory
bool TextureList::LoadTextures()
{
	if( !m_textures.empty() )
	{
		::log("texture list was not unloaded before loading");
		return false;
	}
	Texture *texture;
	LOOPVECTOR(index, m_textureNames)		
		texture = Texture::readTexture(((char*)((std::string)m_textureNames[index]).c_str()));
		if(!texture)
		{
			char buffer[255];
			sprintf(buffer, "Unable to load texture: %s", ((std::string)m_textureNames[index]).c_str());
			MessageBox(NULL, buffer, "Error", MB_OK);
		}
		m_textures.push_back(texture);	
	ENDLOOP

	return true;
}

// Attempt to load the list into video card memory
bool TextureList::RestoreTextures()
{
	Texture* texture;
	::log("\n\nLoading Textures --------------------------------------");
	// Load the textures into video memory
	LOOPVECTOR(index, m_textures)
		texture = ((Texture*)m_textures[index]);
		if(texture)
			texture->load();
		::log("\n[%3x] Texture Loaded -> %s", index, texture->textureName);
	ENDLOOP
	::log("\n\nLoading Textures Complete -----------------------------");

	return true;
}

// Unload the textures from video card memory
void TextureList::InvalidateTextures()
{
	Texture* texture;
	::log("\n\nUnoading Textures ------------------------------------");
	// Load the textures into video memory
	LOOPVECTOR(index, m_textures)
		texture = ((Texture*)m_textures[index]);
		texture->unload();
		::log("\n[%3x] Texture Unloaded -> %s", index, texture->textureName);
	ENDLOOP
	::log("\n\nUnoading Textures Complete ---------------------------");
}

// Unload the textures from system memory
void TextureList::UnloadTextures()
{	
	// Delete the textures
	LOOPVECTOR(index, m_textures)
		delete (Texture*)m_textures[index];
		m_textures[index] = NULL;
	ENDLOOP

	m_textureNames.clear();
	m_textures.clear();

}

// Retrieves the index associated with the specified texture name
int TextureList::GetIndex(char* name)
{
	// Check invalid strings
	if( name == NULL || strlen(name) == 0)
		return TEXTURELIST_INVALIDINDEX;

	// Loop through the texture names searching for the index of the name passed in
	unsigned int i;
	for(i = 0; i < m_textureNames.size(); i++)
	{
		if( ((std::string)m_textureNames[i]) == name )
			return i;
	}

	return TEXTURELIST_INVALIDINDEX;

}