//*****************************************************************************************//
//                                      TextureList                                        //
// Description: This class encapsulates the functionality of managing a list of textures   //
//*****************************************************************************************//

#define TEXTURELIST_FILEID	"Textures"
#define TEXTURELIST_INVALIDINDEX -1

class TextureList
{
private:
	// Singleton object
	static TextureList s_textureList;

public:
	// Get the texture list
	static TextureList *	Get() { return &s_textureList; }

	// Load the texture names from a file
	bool LoadTextureNames( BinaryFileStreamReader *file );

	// Save the texture names to a file
	bool SaveTextureNames( ::ofstream &ifile );

	// Attempt to load the textures into system memory
	bool LoadTextures();

	// Attempt to load the list into video card memory
	bool RestoreTextures();

	// Unload the textures from video card memory
	void InvalidateTextures();

	// Unload the textures from system memory
	void UnloadTextures();

	// Get the index of a texture name
	int GetIndex(char* name);

	// Add a texture name
	int AddTextureName( char* name ) { m_textureNames.push_back( std::string(name) ); return (int)m_textureNames.size()-1; };

	// Get a pointer to a texture object
	Texture* GetTexture(int index) { 
		return (Texture*)m_textures[index]; 
	}

private:

	// A vector of texture names
	std::vector<std::string>	m_textureNames;

	// The list of textures
	TextureCollection			m_textures;

};