#include "sdvisualization.h"
#include "vd.h"
#include "vc.h"

/*static*/ VCSimulation* VCSimulation::s_pSimulationControl = NULL;

VCSimulation::VCSimulation(void)
{
	m_pCurrentVisualization = NULL;
	m_pEventListControl = NULL;
	m_pModelListControl = NULL;

}

VCSimulation::~VCSimulation(void)
{
	if( m_pCurrentVisualization )
		closeVisualization();
}

bool VCSimulation::createNewVisualization( string name )
{
	if( SDVisualization::get() )
	{
		VDConsole::get()->printToConsole(string("Cannot create new visualization. Please close open visualization first"));
		return false;
	}

	m_pCurrentVisualization = SDVisualization::create();
	m_pCurrentVisualization->setName(name);

	// Create the model and event list controls
	m_pEventListControl = new VCEventList(m_pCurrentVisualization->getEventList());
	m_pModelListControl = new VCModelList(m_pCurrentVisualization->getModelList());

	return true;
}

void VCSimulation::closeVisualization()
{
	if( SDVisualization::get() )
	{
		if(m_pEventListControl)
			delete m_pEventListControl;
		if(m_pModelListControl)
			delete m_pModelListControl;

		SDVisualization::destroy();
		m_pCurrentVisualization = NULL;
	}
}

bool VCSimulation::importLogFile( string file )
{
	if( !SDVisualization::get() )
	{
		::log("Unable to import from log file: no visualization currently exists.");
		return false;
	}

	// Create the simulation link object
	VCSimulationLink* pLink = new VCSimulationLink(m_pModelListControl, m_pEventListControl);	

	return pLink->loadLogFile( file );	

	// Delete the simulation link object
	delete pLink;

}