
#include "source\includes.h"
#include "sd.h"
#include "sl.h"
#include "vc.h"

VCSimulationLink::VCSimulationLink(VCModelList* pModelListControl, VCEventList* pEventListControl)
{
	m_pModelListControl = pModelListControl;
	m_pEventListControl = pEventListControl;

	// Not created yet	
	m_pLink				= NULL;
}

VCSimulationLink::~VCSimulationLink(void)
{
}

bool VCSimulationLink::loadLogFile(string logFile)
{
	if( m_pLink != NULL)
	{
		::log("Unable to load log file as a simulation link since the simulation link is already linked to something else");
		return false;
	}

	// Create a SLLogFile Link
	m_pLink = new SLLogFile(this, logFile);

	// Connect to this link

	// Note that this will immediately load all 
	// the data from the log file into the 
	// visualization. An interactive technique
	// will allocate a thread to the simulation link
	// and keep track of the events allocated to this visualization
	// Gonna need a cross platform thread library.

	if(!m_pLink->connect())
	{
		delete m_pLink;
		m_pLink = NULL;
		return false;
	}

	delete m_pLink;
	m_pLink = NULL;
	return true;
}

bool VCSimulationLink::notifyDEVSModel(SDDevsModel* pModel)
{
	if( m_pModelListControl )
	{
		return m_pModelListControl->addModel( pModel );
	}

	return false;
}

bool VCSimulationLink::notifyEvent(SDEvent* pEvent)
{
	if( m_pEventListControl )
	{
		return m_pEventListControl->addEvent( pEvent );
	}

	return false;
}
