#include "vd.h"

VDButton::VDButton(void)
{
	m_mouseOver = false;
	m_buttonPressed = false;
}

VDButton::~VDButton(void)
{
}

bool VDButton::processEvent(VDEvent *pEvent)
{
	if( pEvent->getType() == MOUSEMOVE )
	{

		VDMouseMoveEvent* pMove = dynamic_cast<VDMouseMoveEvent*>(pEvent);
		// Check if the mouse is over this button
		if( m_bounds.containsPoint(pMove->getMouseX(), pMove->getMouseY() ) )
		{
			if(!m_mouseOver)
			{
				m_mouseOver = true;
				VDRedisplayRequestEvent e;
				sendEvent(&e);
			}
		}
		else
		{
			if(m_mouseOver)
			{
				m_mouseOver = false;
				m_buttonPressed = false;
				VDRedisplayRequestEvent e;
				sendEvent(&e);
			}
		}

	}
	else if( pEvent->getType() == MOUSEDOWN )
	{

		VDMouseDownEvent* pDown = dynamic_cast<VDMouseDownEvent*>(pEvent);
		// Check if the mouse is over this button
		if( pDown->getButton() == VD_BUTTON_LEFT && m_bounds.containsPoint(pDown->getMouseX(), pDown->getMouseY() ) )
		{			
			if(!m_buttonPressed)
			{
				m_buttonPressed = true;
				VDRedisplayRequestEvent e;
				sendEvent(&e);
			}
		}

	}
	else if( pEvent->getType() == MOUSEUP )
	{
		VDMouseUpEvent* pUp = dynamic_cast<VDMouseUpEvent*>(pEvent);
		// Check if the mouse is over this button
		if( pUp->getButton() == VD_BUTTON_LEFT && m_bounds.containsPoint(pUp->getMouseX(), pUp->getMouseY() ) )
		{
			if( m_buttonPressed)
			{
				m_buttonPressed = false;

				VDRedisplayRequestEvent r;
				sendEvent(&r);

				// The button was clicked.. Send the event
				VDButtonPressEvent e(m_id);
				sendEvent(&e);

				return true;
			}
			
		}
	}
	else if( m_hasFocus )
	{
		if( pEvent->getType() == KEYDOWN )
		{
			VDKeyDownEvent*	e = dynamic_cast<VDKeyDownEvent*>(pEvent);
			if( e->getKey().getKey() == VD_ENTER )
			{
				if(!m_buttonPressed)
				{
					m_buttonPressed = true;
					VDRedisplayRequestEvent e;
					sendEvent(&e);
				}
			}
		}
		else if( pEvent->getType() == KEYUP )
		{
			VDKeyUpEvent*	e = dynamic_cast<VDKeyUpEvent*>(pEvent);
			if( e->getKey().getKey() == VD_ENTER )
			{
				if( m_buttonPressed)
				{
					m_buttonPressed = false;

					VDRedisplayRequestEvent r;
					sendEvent(&r);

					// The button was clicked.. Send the event
					VDButtonPressEvent e(m_id);
					sendEvent(&e);

					return true;
				}
			}
		}
	}

	return sendEvent(pEvent);
}

VDTextButton::VDTextButton(void)
{
}

VDTextButton::~VDTextButton(void)
{
}

bool VDTextButton::processEvent(VDEvent* pEvent)
{
	if( pEvent->getType() == REDISPLAY )
	{
		// Get the theme and draw the button
		VDThemeStore::get()->getCurrentTheme()->drawTextButton(this);
	}

	if(__super::processEvent(pEvent))
		return true;

	return sendEvent(pEvent);
}

