#include "vd.h"

/*static*/ VDConsole* VDConsole::s_pConsole = NULL;

VDConsole::VDConsole(void)
{	
	m_controlsSetup = false;
	m_pConsoleParser = new VDConsoleParser();
	m_pEditBox = NULL;
	m_pTextBox = NULL;
}

VDConsole::~VDConsole(void)
{
}

void VDConsole::setupControls()
{
	setupControls(1000);
}

void VDConsole::setupControls(int maxLines)
{
	if(m_controlsSetup)
		return;
	m_controlsSetup = true;

	m_visible = true;

	// Set the console name
	m_name = "Console";

	// Set up the edit box
	m_pEditBox = new VDEditBox();
	m_focusWindow = addChildWindow(m_pEditBox);
	m_pEditBox->setFocus(true);
	m_pEditBox->setParentWindow(this);

	// Set up the text box
	m_pTextBox = new VDTextBox(true);
	addChildWindow(m_pTextBox);
	m_pTextBox->setParentWindow(this);
	m_pTextBox->setNumberOfLines(maxLines);

	resizeControls(m_bounds);
	m_pTextBox->setupControls();

	// Setup the history
	m_hisIndex = m_history.begin();

}

void VDConsole::printToConsole( string s )
{
	if(!m_controlsSetup)
		return;
	m_pTextBox->printLine(s);
}

bool VDConsole::processEvent(VDEvent *pEvent)
{
	if( !m_visible )
		return false;

	if( pEvent->getType() == KEYDOWN )
	{
		VDKeyDownEvent* e = dynamic_cast<VDKeyDownEvent*>(pEvent);
		// Handle the keys differently based on which control has the focus
		if( m_pEditBox->hasFocus() )
		{		
			if ( e == NULL )
			{
				::log("Invalid key down event sent to console");
				return true;
			}

			if( e->getKey().getKey() == VD_TAB )
			{
				// Print the possible commands to the console if there are multiple.
				// if there is one.. then simply fill in the rest of the command
				vector<string> vc = m_pConsoleParser->getCommandList( m_pEditBox->getContents() );

				if( (int)vc.size() > 1)
				{
					//List the commands
					printToConsole(string());
					printToConsole("Command List:");
					printToConsole("-------------");
					int i;							
					for(i = 0; i < (int)vc.size(); i++)
						printToConsole(vc[i]);

					// Redisplay
					VDRedisplayRequestEvent r;
					sendEvent(&r);

					return true;
				}
				else if( vc.size() != 0 )
				{
					// Set the command
					string s = vc[0];
					m_pEditBox->setContents(s);
					// Set the cursor
					m_pEditBox->setCaretPosition((int)s.length());

					// Redisplay
					VDRedisplayRequestEvent r;
					sendEvent(&r);

					return true;
				}
				else
				{
					// Dont handle this tab in the default way
					return true;
				}
				
			}
			else if( e->getKey().getKey() == VD_ENTER && m_pEditBox->getContents().length() != 0)
			{
				// If its enter, then process the command on the console
				VDConsoleCommand*	c;
				string				result;
				if( m_pConsoleParser->parseLine(m_pEditBox->getContents(), &c, &result) )
				{
					// Print out the command result
					printToConsole(string());
					printToConsole(result);

					// Execute the command
					c->execute();
				}
				else
				{
					// Print out the command result
					printToConsole(string());
					printToConsole(result);
				}

				// Append the command to the history
				m_history.push_back(m_pEditBox->getContents());
				m_hisIndex = m_history.end();
				if( m_history.size() > HISTORYSIZE )
					m_history.pop_front();

				//Erase the edit box line
				m_pEditBox->setContents(string());

				// Redisplay
				VDRedisplayRequestEvent r;
				sendEvent(&r);
			}
			else if ( e->getKey().getKey() == VD_DOWN)
			{
				if( m_hisIndex != m_history.end() )
				{
					m_hisIndex++;
					if(m_hisIndex == m_history.end())
						m_hisIndex--;

					m_pEditBox->setContents(*m_hisIndex);
					// Set the cursor
					m_pEditBox->setCaretPosition((int)(*m_hisIndex).length());

					// Redisplay
					VDRedisplayRequestEvent r;
					sendEvent(&r);
				}
			}
			else if ( e->getKey().getKey() == VD_UP)
			{
				if(m_hisIndex != m_history.begin())
				{
					m_hisIndex--;
					m_pEditBox->setContents(*m_hisIndex);
					// Set the cursor
					m_pEditBox->setCaretPosition((int)(*m_hisIndex).length());
					// Redisplay
					VDRedisplayRequestEvent r;
					sendEvent(&r);
				}
			}
		}
		else if( m_pTextBox->hasFocus() )
		{
			if( e->getKey().getKey() == VD_UP ) 
			{
				//Scroll up
				m_pTextBox->setCurrentLine(m_pTextBox->getCurrentLine()+1);

			}
			else if( e->getKey().getKey() == VD_DOWN)
			{
				//Scroll down
				m_pTextBox->setCurrentLine(m_pTextBox->getCurrentLine()-1);
			}
			else if( e->getKey().getKey() == VD_PAGE_DOWN || e->getKey().getKey() == VD_PAGE_UP)
			{
				int numLines = VDThemeStore::get()->getCurrentTheme()->getNumberOfLines(m_pTextBox);
				if( e->getKey().getKey() == VD_PAGE_DOWN )
					m_pTextBox->setCurrentLine(m_pTextBox->getCurrentLine() - numLines);
				else
					m_pTextBox->setCurrentLine(m_pTextBox->getCurrentLine() + numLines);
			}	
		}
	}

	// Pass the event on to the children
	if(sendEvent(pEvent))
		return true;

	// Pass the event on to the default processing.
	if( __super::processEvent(pEvent) )
		return true;

	return false;
}

void VDConsole::resizeControls(VDRectangleContainer<float>& newBounds)
{	
	m_bounds = newBounds;
	
	if( m_pEditBox )
	{
		VDRectangleContainer<float> bounds;		
		bounds.setBounds(m_bounds.getX1() + m_resizeBorderWidth, m_bounds.getY1() + m_resizeBorderWidth, m_bounds.getX2() - m_resizeBorderWidth, m_bounds.getY1() + m_resizeBorderWidth + 30 );
		m_pEditBox->resizeControls(bounds);
	}

	if( m_pTextBox )
	{
		VDRectangleContainer<float> bounds;
		bounds.setBounds(m_bounds.getX1() + m_resizeBorderWidth, m_bounds.getY1() + 30 + m_resizeBorderWidth, m_bounds.getX2()-m_resizeBorderWidth, m_bounds.getY2() - m_moveBarHeight - m_resizeBorderWidth);	
		m_pTextBox->resizeControls(bounds);
	}
}

void VDConsole::hide()
{ 
	if(m_visible) 
	{
		m_visible = false;
		VDRedisplayRequestEvent e;
		sendEvent(&e);
	}
}

void VDConsole::show()
{
	if(!m_visible)
	{
		m_visible = true;
		VDRedisplayRequestEvent e;
		sendEvent(&e);
	}
}

