#include "sd.h"
#include "vc.h"
#include "vd.h"

void VDExitCommand::messageBoxResponse( MESSAGEBOXRESPONSE r)
{
	// Add this object to the garbage can
	VDGarbageCan::get()->dispose(this);

	if( r == YES )
	{

		// Destroy the simulation control object
		VCSimulation::destroy();

		// Destroy the glextensions object
		GLExtensions::dispose();

		// Destroy the themes
		VDThemeStore::destroy();

		// Destroy the Window
		VDMainWindow::destroy();

		// Destroy the garbage can
		VDGarbageCan::destroy();

		//exit the program. GLUT should take care of cleaning up itself!
		exit(0);
	}

}

void VDExitCommand::execute()
{
	VDMainWindow::get()->messageBox(YESNO, string("Are you sure you want to exit?"), this);
}

void VDNewCommand::execute()
{
	VCSimulation::get()->createNewVisualization(m_visName);
	// Add this object to the garbage can
	VDGarbageCan::get()->dispose(this);
}

void VDCloseCommand::execute()
{
	VCSimulation::get()->closeVisualization();
	// Add this object to the garbage can
	VDGarbageCan::get()->dispose(this);
}

void VDPrintObjectCommand::execute()
{
	if( m_objectId == VDPRINTOBJECT_VISUALIZATION )
	{
		//Print out the state of the current visualization object
		SDVisualization* pVis = SDVisualization::get();
		if( pVis )
		{
			VDConsole* pConsole = VDConsole::get();
			if(pConsole)
			{
				char buffer[255];
				string s;
				pConsole->printToConsole("----------------------");
				pConsole->printToConsole("Current Visualization:");
				s = "Name: ";
				s.append(pVis->getName());
				pConsole->printToConsole(s);

				int hours, minutes, seconds, milliseconds;
				SDTime t = pVis->getTime();
				t.getTime(&hours, &minutes, &seconds, &milliseconds);
				sprintf(buffer, "%02d:%02d:%02d:%03d", hours, minutes, seconds, milliseconds);

				s = "Current Time: ";
				s.append(buffer);
				pConsole->printToConsole(s);
				pConsole->printToConsole("----------------------");				
			}
		}
		else
		{
			VDConsole::get()->printToConsole("Unable to print out visualization. Visualization does not exist.");
		}
	}
	else if( m_objectId == VDPRINTOBJECT_VISUALIZATION_NODELIST )
	{
		//Print out the state of the current visualization object
		SDVisualization* pVis = SDVisualization::get();
		if( pVis )
		{
			VDConsole* pConsole = VDConsole::get();
			if(pConsole)
			{
				SDNodeList* pList = pVis->getNodeList();
				char buffer[255];
				string s;
				pConsole->printToConsole("----------------------");
				pConsole->printToConsole("Node List:");
				s = "List Size: ";
				int size = pList->getNodeListSize();
				sprintf(buffer, "%d", size);
				s.append(buffer);
				pConsole->printToConsole(s);
				pConsole->printToConsole("----------------------");
				int i;				
				for(i = 0; i < size; i++)
				{
					sprintf(buffer, "Node %d", i);
					s = buffer;
					pConsole->printToConsole(s);
				}
				pConsole->printToConsole("----------------------");
			}
		}
		else
		{
			VDConsole::get()->printToConsole("Unable to print out visualization node list. Visualization does not exist.");
		}
	}

	// Add this object to the garbage can
	VDGarbageCan::get()->dispose(this);
}

void VDAddDetailNodeCommand::execute()
{
	// Create the detail node
	if( m_nodeType == VDADDDETAILNODE_CUBE )
	{
		SDVisualization* pVis = SDVisualization::get();
		if( pVis )
		{
			// Create the simple detail node
			SDDetailNode* pNode = new SDDetailNode();
			pNode->setPosition(m_position);
			pNode->setSize(m_size);

			// Add the node to the visualization
			pVis->addNode(pNode);
		}
		else
		{
			VDConsole::get()->printToConsole("Unable to add detail node. Visualization does not exist.");
		}
	}
	// Add this object to the garbage can
	VDGarbageCan::get()->dispose(this);
}

void VDImportLogFileCommand::execute()
{
	// Import the log file
	VCSimulation::get()->importLogFile(m_logFile);
	// Add this object to the garbage can
	VDGarbageCan::get()->dispose(this);

}

VDConsoleParser::VDConsoleParser(void)
{
	// Create the global list of commands
	m_commandList.push_back(VDCOMMAND_EXIT);
	m_commandList.push_back(VDCOMMAND_NEW);
	m_commandList.push_back(VDCOMMAND_CLOSE);
	m_commandList.push_back(VDCOMMAND_PRINTOBJECT);
	m_commandList.push_back(VDCOMMAND_ADDDETAILNODE);
	m_commandList.push_back(VDCOMMAND_IMPORTLOGFILE);
}

VDConsoleParser::~VDConsoleParser(void)
{
}


vector<string>	VDConsoleParser::tokenize(string s, string delim)
{
	vector<string>	tokens;

	int start = 0;
	int i = 0;
	while( i < (int)s.length() )
	{
		if( delim.find(s.at(i)) != string::npos )
		{
			tokens.push_back(s.substr(start, i-start));
			// Skip the delimiter
			start = i + 1;
		}
		i++;
	}

	// Grab the end of the string
	if ( i != start && start < (int)s.length() )
		tokens.push_back(s.substr(start));

	return tokens;
}

bool VDConsoleParser::parseLine( string line, VDConsoleCommand** ppCommand, string* result)
{
	// Grab the first space separated token
	vector<string> tokens;
	tokens = tokenize(line, string(" "));
	if(tokens.size() != 0)
	{
		if( tokens[0] == VDCOMMAND_EXIT )
		{
			if( tokens.size() == 1 )
			{
				(*result) = "Exiting...";
				*ppCommand = new VDExitCommand();
				return true;
			}
			else
			{
				(*result) = "usage: ";
				(*result).append(VDCOMMAND_EXIT);
				*ppCommand = NULL;
				return false;
			}
		}
		else if( tokens[0] == VDCOMMAND_NEW )
		{
			if( tokens.size() == 2 )
			{
				if( tokens[1].find(".vis") == string::npos )
					tokens[1].append(".vis");

				(*result) = "Creating new visualization context named: ";
				(*result).append(tokens[1]);

				*ppCommand = new VDNewCommand();
				((VDNewCommand*)(*ppCommand))->setVisName(tokens[1]);
				return true;
			}			
			else
			{
				(*result) = "usage: ";
				(*result).append(VDCOMMAND_NEW);
				(*result).append(" $visualization_name");
				*ppCommand = NULL;
				return false;
			}
		}
		else if( tokens[0] == VDCOMMAND_IMPORTLOGFILE )
		{
			if( tokens.size() == 2 )
			{
				(*result) = "Importing simulation results from log file: ";
				(*result).append(tokens[1]);

				*ppCommand = new VDImportLogFileCommand();
				((VDImportLogFileCommand*)(*ppCommand))->setLogFile(tokens[1]);
				return true;
			}			
			else
			{
				(*result) = "usage: ";
				(*result).append(VDCOMMAND_IMPORTLOGFILE);
				(*result).append(" $logfile");
				*ppCommand = NULL;
				return false;
			}
		}
		else if( tokens[0] == VDCOMMAND_CLOSE )
		{
			if( tokens.size() == 1)
			{
				(*result) = "Closing visualization context...";
				*ppCommand = new VDCloseCommand();
				return true;
			}			
			else
			{
				(*result) = "usage: ";
				(*result).append(VDCOMMAND_CLOSE);
				*ppCommand = NULL;
				return false;
			}
		}
		else if( tokens[0] == VDCOMMAND_PRINTOBJECT )
		{
			if( tokens.size() == 2 && ((tokens[1] == VDPRINTOBJECT_VISUALIZATION) || (tokens[1] == VDPRINTOBJECT_VISUALIZATION_NODELIST)))
			{
				(*result) = "Printing object info...";
				*ppCommand = new VDPrintObjectCommand();
				((VDPrintObjectCommand*)(*ppCommand))->setObjectId(tokens[1]);
				return true;
			}			
			else
			{
				(*result) = "usage: ";
				(*result).append(VDCOMMAND_PRINTOBJECT);
				(*result).append(" ");
				(*result).append(VDPRINTOBJECT_VISUALIZATION);
				(*result).append(" | ");
				(*result).append(VDPRINTOBJECT_VISUALIZATION_NODELIST);
				*ppCommand = NULL;
				return false;
			}
		}
		else if( tokens[0] == VDCOMMAND_ADDDETAILNODE )
		{
			if(( tokens.size() == 6 ) && ( tokens[1] == VDADDDETAILNODE_CUBE ))
			{
				(*result) = "Adding node object...";
				*ppCommand = new VDAddDetailNodeCommand();
				float x,y,z,size;
				sscanf( tokens[2].c_str(), "%f", &x );
				sscanf( tokens[3].c_str(), "%f", &y );
				sscanf( tokens[4].c_str(), "%f", &z );
				sscanf( tokens[5].c_str(), "%f", &size );

				((VDAddDetailNodeCommand*)(*ppCommand))->setNodeType(VDADDDETAILNODE_CUBE);
				((VDAddDetailNodeCommand*)(*ppCommand))->setLocation(x,y,z);
				((VDAddDetailNodeCommand*)(*ppCommand))->setSize(size);
				return true;
			}
			else
			{
				// If it was not handled then show the usage
				(*result) = "usage: ";
				(*result).append(VDCOMMAND_ADDDETAILNODE);
				(*result).append(" (");
				(*result).append(VDADDDETAILNODE_CUBE);
				(*result).append(") $x_pos $y_pos $z_pos $size");
				*ppCommand = NULL;
				return false;
			}
		}
	}	
	
	*ppCommand = NULL;
	(*result) = "Unknown command: ";
	if( tokens.size() != 0 )
		(*result).append(tokens[0]);
	return false;
}

vector<string>	VDConsoleParser::getCommandList()
{
	return m_commandList;
}

vector<string>	VDConsoleParser::getCommandList(string match)
{
	vector<string> commands;
	int i;

	// return the commandlist if the match is empty
	if( match.size() == 0 )
		return m_commandList;

	for(i=0; i<(int)m_commandList.size(); i++)
	{
		string s;
		s = m_commandList[i];
		if( match == s.substr(0, match.size()))
			commands.push_back(s);
	}

	return commands;
}