#pragma once

#include "source\binaryfilestream.h"
#include "source\transformations.h"
#include "source\points.h"

// List of specific commands
#define VDCOMMAND_EXIT			"exit"
#define VDCOMMAND_NEW			"new_visualization"
#define VDCOMMAND_CLOSE			"close_visualization"
#define VDCOMMAND_PRINTOBJECT	"print_object"
	#define VDPRINTOBJECT_VISUALIZATION				"VIS"
	#define VDPRINTOBJECT_VISUALIZATION_NODELIST	"VIS_NL"
#define VDCOMMAND_ADDDETAILNODE "add_detail_node"
	#define VDADDDETAILNODE_CUBE	"CUBE"
#define VDCOMMAND_IMPORTLOGFILE	"import_logfile"

class VDConsoleCommand : public VDDisposable
{
protected:
	string m_name;

public:
	VDConsoleCommand(string name)		{ m_name = name; }
	virtual ~VDConsoleCommand()			{}

	virtual string	getName()			{ return m_name; }
	virtual void	execute() = 0;

};

class VDExitCommand : public VDConsoleCommand, public VDMessageBoxCallBack
{
public:
	VDExitCommand() : VDConsoleCommand(VDCOMMAND_EXIT)	{}
	virtual ~VDExitCommand()							{}

	virtual void messageBoxResponse( MESSAGEBOXRESPONSE r);

	virtual void execute();

};

class VDNewCommand : public VDConsoleCommand
{
protected:
	string	m_visName;
public:
	VDNewCommand() : VDConsoleCommand(VDCOMMAND_NEW){}
	virtual ~VDNewCommand()							{}

	void	setVisName(string n) { m_visName = n; }
	virtual void execute();

};

class VDCloseCommand : public VDConsoleCommand
{
public:
	VDCloseCommand() : VDConsoleCommand(VDCOMMAND_CLOSE){}
	virtual ~VDCloseCommand()							{}

	virtual void execute();

};

class VDPrintObjectCommand : public VDConsoleCommand
{
protected:
	string	m_objectId;
public:
	VDPrintObjectCommand() : VDConsoleCommand(VDCOMMAND_PRINTOBJECT)	{}
	virtual ~VDPrintObjectCommand()										{}

	virtual void setObjectId(string id) { m_objectId = id; }

	virtual void execute();
};

class VDAddDetailNodeCommand : public VDConsoleCommand
{
protected:
	string	m_nodeType;
	Point	m_position;
	float	m_size;
public:
	VDAddDetailNodeCommand() : VDConsoleCommand(VDCOMMAND_ADDDETAILNODE)	{}
	virtual ~VDAddDetailNodeCommand()										{}

	virtual void setNodeType(string type) { m_nodeType = type; }
	virtual void setLocation(float x, float y, float z) { m_position.x = x; m_position.y = y; m_position.z = z;}
	virtual void setSize(float size) { m_size = size; }

	virtual void execute();
};

class VDImportLogFileCommand : public VDConsoleCommand
{
protected:
	string	m_logFile;
public:
	VDImportLogFileCommand() : VDConsoleCommand(VDCOMMAND_IMPORTLOGFILE)		{}
	virtual ~VDImportLogFileCommand()											{}

	virtual void setLogFile(string logFile) { m_logFile = logFile; }

	virtual void execute();
};

class VDConsoleParser
{
protected:
	vector<string>	m_commandList;

	vector<string>	VDConsoleParser::tokenize(string s, string delim);
public:
	VDConsoleParser(void);
	virtual ~VDConsoleParser(void);

	virtual bool	parseLine( string line, VDConsoleCommand** ppCommand, string* result);

	virtual vector<string>		getCommandList();
	virtual vector<string>		getCommandList(string match);
};
