#include "source\includes.h"
#include "source\points.h"
#include "source\font.h"
#include "vd.h"

VDDefaultTheme::VDDefaultTheme(void)
{
}

VDDefaultTheme::~VDDefaultTheme(void)
{
}

void VDDefaultTheme::setup()
{
	// Focus color. If an object has this color then it has the focus
	m_focusColor.r = 1.0;m_focusColor.g = 1.0;m_focusColor.b = 1.0;m_focusColor.a = 1.0;
	// Mouse over color. If an object has this color then it has the focus
	m_mouseOverColor.r = 0.2;m_mouseOverColor.g = 0.2;m_mouseOverColor.b = 0.2;m_mouseOverColor.a = 1.0;
	// Selected color. If an object has this color then it has the focus
	m_selectedColor.r = 0.5;m_selectedColor.g = 0.7;m_selectedColor.b = 0.5;m_selectedColor.a = 0.5;

	// Setup the edit box info	
	m_editBoxBackgroundColor.r = 0.3;m_editBoxBackgroundColor.g = 0.3;m_editBoxBackgroundColor.b = 0.3;m_editBoxBackgroundColor.a = 1.0;
	m_editBoxTextColor.r = 0.9;m_editBoxTextColor.g = 0.9;m_editBoxTextColor.b = 0.9;m_editBoxTextColor.a = 1.0;
	m_editBoxOutlineColor.r = 0.5;m_editBoxOutlineColor.g = 0.5;m_editBoxOutlineColor.b = 0.5;m_editBoxOutlineColor.a = 0.5;

	// Setup the label info
	m_labelTextColor.r = 0.8;m_labelTextColor.g = 0.8;m_labelTextColor.b = 0.8;m_labelTextColor.a = 1.0;

	// Setup the panel colors
	m_panelBorderColor.r = 0.6;m_panelBorderColor.g = 0.65;m_panelBorderColor.b = 0.6;m_panelBorderColor.a = 0.8;
	m_panelMainColor.r = 0.0;m_panelMainColor.g = 0.0;m_panelMainColor.b = 0.0;m_panelMainColor.a = 0.4;
	m_panelUpperLeftColor.r = 1.0;m_panelUpperLeftColor.g = 1.0;m_panelUpperLeftColor.b = 1.0;m_panelUpperLeftColor.a = 0.8;
	m_panelLowerRightColor.r = 0.4;m_panelLowerRightColor.g = 0.4;m_panelLowerRightColor.b = 0.4;m_panelLowerRightColor.a = 0.8;

	// Button colors
	m_buttonMainColor.r = 0.5;m_buttonMainColor.g = 0.5;m_buttonMainColor.b = 0.5;m_buttonMainColor.a = 1.0;
	m_buttonUpperLeftColor.r = 0.7;m_buttonUpperLeftColor.g = 0.7;m_buttonUpperLeftColor.b = 0.7;m_buttonUpperLeftColor.a = 1.0;
	m_buttonLowerRightColor.r = 0.3;m_buttonLowerRightColor.g = 0.3;m_buttonLowerRightColor.b = 0.3;m_buttonLowerRightColor.a = 1.0;

	m_pTitleFont	= new Font("Lucida Console", 14, true, false);
	m_pTitleFont->getAverageCharSize();
	m_pStandardFont = new Font("Lucida Console", 12);
	m_fontSize = m_pStandardFont->getAverageCharSize();

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_LINE_SMOOTH);

	glClearDepth (1.0);

	glDrawBuffer (GL_BACK);
	glReadBuffer (GL_BACK);
	glEnable (GL_DEPTH_TEST);
	glDepthMask (GL_TRUE);
	glDepthFunc (GL_LEQUAL);
	glShadeModel (GL_SMOOTH);	

	glFrontFace (GL_CCW);
	glCullFace (GL_BACK);
	glEnable (GL_CULL_FACE);
	glEnable (GL_DEPTH_TEST);

	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glPolygonOffset (0.0, -3.0);

	//Use a global default texture environment mode.
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

}

void VDDefaultTheme::wrapup()
{
	if(m_pStandardFont)
	{
		delete m_pStandardFont;
		m_pStandardFont = NULL;
	}
}

DRAWTYPE VDDefaultTheme::drawEditBoxImp(VDEditBox *pEditBox)
{

	// Get the bounds of the edit box
	VDRectangleContainer<float> bounds;
	pEditBox->getBounds(&bounds);
	
	glColor4f(m_editBoxBackgroundColor.r, m_editBoxBackgroundColor.g, m_editBoxBackgroundColor.b, m_editBoxBackgroundColor.a);
	glBegin(GL_POLYGON);
		bounds.setGLVertex2f();
	glEnd();	

	if( pEditBox->hasFocus() )
		glColor4f(m_focusColor.r, m_focusColor.g, m_focusColor.b, m_focusColor.a);
	else
		glColor4f(m_editBoxOutlineColor.r, m_editBoxOutlineColor.g, m_editBoxOutlineColor.b, m_editBoxOutlineColor.a);
	glLineWidth(2.0f);
	glBegin(GL_LINE_LOOP);
		bounds.setGLVertex2f();
	glEnd();	
	glLineWidth(1.0f);

	glColor4f(m_editBoxTextColor.r, m_editBoxTextColor.g, m_editBoxTextColor.b, m_editBoxTextColor.a);
	m_pStandardFont->drawText( bounds.getX1() + 5, bounds.getY1() + 5, pEditBox->getContents().c_str() );
	
	glBegin(GL_LINES);
		glVertex2d( (bounds.getX1() + 6) + (m_fontSize.x + 0.01) * pEditBox->getCaretPosition(), bounds.getY1() + 3);
		glVertex2d( (bounds.getX1() + 5) + (m_fontSize.x + 0.01) * pEditBox->getCaretPosition(), bounds.getY1() + 9 + (m_fontSize.y));
	glEnd();

	if ( pEditBox->getSelectPosition() != -1)
	{
		glDisable (GL_CULL_FACE);
		glColor4f(m_selectedColor.r, m_selectedColor.g, m_selectedColor.b, m_selectedColor.a);
		glBegin(GL_POLYGON);
				glVertex2d( (bounds.getX1() + 6) + (m_fontSize.x) * pEditBox->getCaretPosition(), bounds.getY1() );		
				glVertex2d( (bounds.getX1() + 6) + (m_fontSize.x) * pEditBox->getSelectPosition(), bounds.getY1() );
				glVertex2d( (bounds.getX1() + 5) + (m_fontSize.x) * pEditBox->getSelectPosition(), bounds.getY1() + 9 + (m_fontSize.y));
				glVertex2d( (bounds.getX1() + 5) + (m_fontSize.x) * pEditBox->getCaretPosition(), bounds.getY1() + 9 + (m_fontSize.y));							
		glEnd();	

		glColor4f(m_selectedColor.r, m_selectedColor.g, m_selectedColor.b, m_selectedColor.a);
		glBegin(GL_LINE_LOOP);
				glVertex2d( (bounds.getX1() + 6) + (m_fontSize.x) * pEditBox->getCaretPosition(), bounds.getY1() );
				glVertex2d( (bounds.getX1() + 6) + (m_fontSize.x) * pEditBox->getSelectPosition(), bounds.getY1() );
				glVertex2d( (bounds.getX1() + 5) + (m_fontSize.x) * pEditBox->getSelectPosition(), bounds.getY1() + 9 + (m_fontSize.y));
				glVertex2d( (bounds.getX1() + 5) + (m_fontSize.x) * pEditBox->getCaretPosition(), bounds.getY1() + 9 + (m_fontSize.y));				
		glEnd();
		glEnable (GL_CULL_FACE);
	}
	return STATIC;
}

DRAWTYPE VDDefaultTheme::drawLabelImp( VDLabel* pLabel )
{
	// Get the bounds of the label
	VDRectangleContainer<float> bounds;
	pLabel->getBounds(&bounds);
	
	glColor4f(m_labelTextColor.r, m_labelTextColor.g, m_labelTextColor.b, m_labelTextColor.a);
	m_pStandardFont->drawText( bounds.getX1() + 5, bounds.getY1() + 5, pLabel->getContents().c_str() );
	return STATIC;
}

DRAWTYPE VDDefaultTheme::drawTextBoxImp( VDTextBox* pTextBox )
{

	// Get the bounds of the TextBox
	VDRectangleContainer<float> bounds;
	pTextBox->getBounds(&bounds);
	bounds.setX2(bounds.getX2() - pTextBox->getVScrollWidth());

	// Draw the background and outline
	glColor4f(m_editBoxBackgroundColor.r, m_editBoxBackgroundColor.g, m_editBoxBackgroundColor.b, m_editBoxBackgroundColor.a);
	glBegin(GL_POLYGON);
		bounds.setGLVertex2f();
	glEnd();

	if( pTextBox->hasFocus() )
		glColor4f(m_focusColor.r, m_focusColor.g, m_focusColor.b, m_focusColor.a);
	else
		glColor4f(m_editBoxOutlineColor.r, m_editBoxOutlineColor.g, m_editBoxOutlineColor.b, m_editBoxOutlineColor.a);
	glLineWidth(2.0f);
	glBegin(GL_LINE_LOOP);
		bounds.setGLVertex2f();
	glEnd();
	glLineWidth(1.0f);

	// Draw the multiple lines of text
	VDStringLog* pLines = pTextBox->getText();
	list<string>::iterator i = pLines->getBegin();
	int j = 0;
	int pos = 0;
	while(i != pLines->getEnd())
	{	
		if( pTextBox->getCurrentLine() <= pos )
		{
			//Draw this string if it fits, if not we need to clip it somehow.. stencil buffer?
			float y = bounds.getY1() + 5 + (j * (m_fontSize.y+2));
			if( (y + m_fontSize.y) > bounds.getY2() )
				break;
			glColor4f(m_editBoxTextColor.r, m_editBoxTextColor.g, m_editBoxTextColor.b, m_editBoxTextColor.a);
			if( (*i).length() * m_fontSize.x < (bounds.getWidth() - 5))
				m_pStandardFont->drawText( bounds.getX1() + 5, y, (*i).c_str() );
			else
			{
				int numChars = ((bounds.getWidth()-5) / m_fontSize.x);
				m_pStandardFont->drawText(bounds.getX1() + 5, y, (*i).substr(0, numChars).c_str() );
			}
			j++;
		}

		i++;
		pos++;
	}

	return STATIC;
}

DRAWTYPE VDDefaultTheme::drawMainWindowBackgroundImp()
{
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	return STATIC;
}

DRAWTYPE VDDefaultTheme::drawPanelImp( VDPanel*	pPanel )
{

	// Get the bounds of the Panel
	VDRectangleContainer<float> bounds;
	pPanel->getBounds(&bounds);

	// Draw the main window
	glColor4f(m_panelBorderColor.r, m_panelBorderColor.g, m_panelBorderColor.b, m_panelBorderColor.a);
	glBegin(GL_POLYGON);
		bounds.setGLVertex2f();
	glEnd();	

	// Draw the upper left outline
	glColor4f(m_panelUpperLeftColor.r, m_panelUpperLeftColor.g, m_panelUpperLeftColor.b, m_panelUpperLeftColor.a);
	glBegin(GL_LINES);
		glVertex2f(bounds.getX1(), bounds.getY1());
		glVertex2f(bounds.getX1(), bounds.getY2());

		glVertex2f(bounds.getX1(), bounds.getY2());
		glVertex2f(bounds.getX2(), bounds.getY2());
	glEnd();

	// Draw the lower right outline
	glColor4f(m_panelLowerRightColor.r, m_panelLowerRightColor.g, m_panelLowerRightColor.b, m_panelLowerRightColor.a);
	glBegin(GL_LINES);
		glVertex2f(bounds.getX1(), bounds.getY1());
		glVertex2f(bounds.getX2(), bounds.getY1());

		glVertex2f(bounds.getX2(), bounds.getY1());
		glVertex2f(bounds.getX2(), bounds.getY2());
	glEnd();

	// Draw the panel area
	glColor4f(m_panelMainColor.r, m_panelMainColor.g, m_panelMainColor.b, m_panelMainColor.a);
	int border	= pPanel->getBorderWidth();
	int bar		= pPanel->getBarHeight();
	glBegin(GL_POLYGON);
		glVertex2f(bounds.getX1() + border, bounds.getY1() + border);
		glVertex2f(bounds.getX2() - border, bounds.getY1() + border);		
		glVertex2f(bounds.getX2() - border, bounds.getY2() - bar - border );
		glVertex2f(bounds.getX1() + border, bounds.getY2() - bar - border );
	glEnd();
	
	// Draw the mouse over states
	glLineWidth(3.0f);
	glColor4f(m_mouseOverColor.r, m_mouseOverColor.g, m_mouseOverColor.b, m_mouseOverColor.a);
	if( pPanel->getMouseOverFlags() & PANELFLAG_LEFT)
	{
		glBegin(GL_LINES);
			glVertex2f(bounds.getX1(), bounds.getY1());
			glVertex2f(bounds.getX1(), bounds.getY2());
		glEnd();
	}
	if( pPanel->getMouseOverFlags() & PANELFLAG_RIGHT)
	{
		glBegin(GL_LINES);
			glVertex2f(bounds.getX2(), bounds.getY1());
			glVertex2f(bounds.getX2(), bounds.getY2());
		glEnd();
	}
	if( pPanel->getMouseOverFlags() & PANELFLAG_UP)
	{
		glBegin(GL_LINES);
			glVertex2f(bounds.getX1(), bounds.getY2());
			glVertex2f(bounds.getX2(), bounds.getY2());
		glEnd();
	}
	if( pPanel->getMouseOverFlags() & PANELFLAG_DOWN)
	{
		glBegin(GL_LINES);
			glVertex2f(bounds.getX1(), bounds.getY1());
			glVertex2f(bounds.getX2(), bounds.getY1());
		glEnd();
	}	
	if( pPanel->getMouseOverFlags() & PANELFLAG_MOVING)
	{
		glBegin(GL_LINE_LOOP);
			glVertex2f(bounds.getX1() + border, bounds.getY2() - border);
			glVertex2f(bounds.getX2() - border, bounds.getY2() - border);
			glVertex2f(bounds.getX2() - border, bounds.getY2() - bar - border);
			glVertex2f(bounds.getX1() + border, bounds.getY2() - bar - border);
		glEnd();
	}
	glLineWidth(1.0f);

	if( pPanel->getName().length() > 0 )
	{
		if( pPanel->getMouseOverFlags() & PANELFLAG_MOVING)
			glColor4f(m_mouseOverColor.r, m_mouseOverColor.g, m_mouseOverColor.b, m_mouseOverColor.a);
		else
			glColor4f(m_labelTextColor.r, m_labelTextColor.g, m_labelTextColor.b, m_labelTextColor.a);

		m_pTitleFont->drawText( bounds.getX1() + border + 5, bounds.getY2() - border - 15, pPanel->getName().c_str() );
	}

	return STATIC;
}

DRAWTYPE VDDefaultTheme::drawScrollBarImp( VDScrollBar* pScrollBar)
{

	// Get the bounds of the Scrollbar
	VDRectangleContainer<float> bounds;
	pScrollBar->getBounds(&bounds);

	// Draw the main window
	glColor4f(0.0f, 0.0f, 0.0f, 0.5f);
	glBegin(GL_POLYGON);
		bounds.setGLVertex2f();
	glEnd();	

	// Draw the slider
	pScrollBar->getSliderBounds(&bounds);
	glColor4f(m_panelUpperLeftColor.r, m_panelUpperLeftColor.g, m_panelUpperLeftColor.b, m_panelUpperLeftColor.a);
	glBegin(GL_POLYGON);
		bounds.setGLVertex2f();
	glEnd();

	// Draw the slider outlines
	// Draw the upper left outline
	glColor4f(m_panelUpperLeftColor.r, m_panelUpperLeftColor.g, m_panelUpperLeftColor.b, m_panelUpperLeftColor.a);
	glBegin(GL_LINES);
		glVertex2f(bounds.getX1(), bounds.getY1());
		glVertex2f(bounds.getX1(), bounds.getY2());

		glVertex2f(bounds.getX1(), bounds.getY2());
		glVertex2f(bounds.getX2(), bounds.getY2());
	glEnd();

	// Draw the lower right outline
	glColor4f(m_panelLowerRightColor.r, m_panelLowerRightColor.g, m_panelLowerRightColor.b, m_panelLowerRightColor.a);
	glBegin(GL_LINES);
		glVertex2f(bounds.getX1(), bounds.getY1());
		glVertex2f(bounds.getX2(), bounds.getY1());

		glVertex2f(bounds.getX2(), bounds.getY1());
		glVertex2f(bounds.getX2(), bounds.getY2());
	glEnd();

	return STATIC;
}

DRAWTYPE VDDefaultTheme::drawTextButtonImp( VDTextButton* pTextButton)
{
	// Get the bounds of the Text button
	VDRectangleContainer<float> bounds;
	pTextButton->getBounds(&bounds);

	// Draw the button object
	if( pTextButton->hasMouseOver() )
		glColor4f(m_mouseOverColor.r, m_mouseOverColor.g, m_mouseOverColor.b, m_mouseOverColor.a);
	else
		glColor4f(m_buttonMainColor.r, m_buttonMainColor.g, m_buttonMainColor.b, m_buttonMainColor.a);
	glEnable(GL_BLEND);
	glBegin(GL_POLYGON);
		bounds.setGLVertex2f();
	glEnd();	

	// Draw the button upper left
	if( pTextButton->isPressed())
		glColor4f(m_buttonLowerRightColor.r, m_buttonLowerRightColor.g, m_buttonLowerRightColor.b, m_buttonLowerRightColor.a);
	else
		glColor4f(m_buttonUpperLeftColor.r, m_buttonUpperLeftColor.g, m_buttonUpperLeftColor.b, m_buttonUpperLeftColor.a);		
	glBegin(GL_LINES);
		glVertex2f(bounds.getX1(), bounds.getY1());
		glVertex2f(bounds.getX1(), bounds.getY2());

		glVertex2f(bounds.getX1(), bounds.getY2());
		glVertex2f(bounds.getX2(), bounds.getY2());
	glEnd();	

	// Draw the button lower right
	if( !pTextButton->isPressed())
		glColor4f(m_buttonLowerRightColor.r, m_buttonLowerRightColor.g, m_buttonLowerRightColor.b, m_buttonLowerRightColor.a);
	else
		glColor4f(m_buttonUpperLeftColor.r, m_buttonUpperLeftColor.g, m_buttonUpperLeftColor.b, m_buttonUpperLeftColor.a);
	glBegin(GL_LINES);
		glVertex2f(bounds.getX1(), bounds.getY1());
		glVertex2f(bounds.getX2(), bounds.getY1());

		glVertex2f(bounds.getX2(), bounds.getY1());
		glVertex2f(bounds.getX2(), bounds.getY2());
	glEnd();

	if(pTextButton->hasFocus())
		glColor4f(m_focusColor.r, m_focusColor.g, m_focusColor.b, m_focusColor.a);
	else
		glColor4f(m_labelTextColor.r, m_labelTextColor.g, m_labelTextColor.b, m_labelTextColor.a);
	float x_t, y_t;
	if( pTextButton->isPressed())
	{
		x_t = 11;
		y_t = 5;
	}
	else
	{
		x_t = 10;
		y_t = 6;
	}

	m_pStandardFont->drawText( bounds.getX1() + x_t, bounds.getY1() + y_t, pTextButton->getText().c_str() );

	return STATIC;
}

int	VDDefaultTheme::getNumberOfLines( VDTextBox* pTextBox )
{
	VDRectangleContainer<float> bounds;
	pTextBox->getBounds(&bounds);
	return (int)(floor(bounds.getHeight() / m_fontSize.y));
}