#pragma once

#include <math.h>
#include "source\transformations.h"
#include "source\binaryfilestream.h"
#include "source\points.h"

#define DEFAULTTHEME	"default"

class Font;
struct Point;

class VDDefaultTheme : public VDTheme
{
protected:

	// Edit box visual appearance - put in EditBoxAppearance class? Maybe 
	COLORVALUE	m_editBoxBackgroundColor;
	COLORVALUE	m_editBoxTextColor;
	COLORVALUE	m_editBoxOutlineColor;

	// Focus appearance
	COLORVALUE	m_focusColor;
	COLORVALUE	m_mouseOverColor;
	COLORVALUE	m_selectedColor;

	// Label visual appearance
	COLORVALUE	m_labelTextColor;

	// Panel colours
	COLORVALUE	m_panelBorderColor;
	COLORVALUE	m_panelMainColor;
	COLORVALUE	m_panelUpperLeftColor;
	COLORVALUE	m_panelLowerRightColor;

	// Button colours
	COLORVALUE	m_buttonMainColor;
	COLORVALUE	m_buttonUpperLeftColor;
	COLORVALUE	m_buttonLowerRightColor;


	Font*		m_pTitleFont;
	Font*		m_pStandardFont;
	Point		m_fontSize;

public:
	VDDefaultTheme(void);
	virtual ~VDDefaultTheme(void);

	virtual void setup();
	virtual void wrapup();

	virtual void beginThemeDrawing(float width, float height) {

		glDisable(GL_TEXTURE_2D);
		glLineWidth(1.0);

		// Setup default 2d drawing mode
		glMatrixMode (GL_PROJECTION);
		glPushMatrix ();
		glLoadIdentity ();
		glOrtho (0.0,(GLfloat)width,0.0, (GLfloat)height, -10.0f, 100.0f);
		glMatrixMode (GL_MODELVIEW);
		glPushMatrix ();
		glLoadIdentity ();
		m_draw = STATIC;

		glEnable(GL_BLEND);

	}
	virtual DRAWTYPE endThemeDrawing() {

		// Return to saved drawing mode
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();

		glDisable(GL_BLEND);
		glColor4f(1,1,1,1);

		return STATIC;
	}

	// Positioning information
	virtual int		 getNumberOfLines( VDTextBox* pTextBox );

	virtual DRAWTYPE drawMainWindowBackgroundImp();
	virtual DRAWTYPE drawEditBoxImp( VDEditBox* pEditBox );
	virtual DRAWTYPE drawLabelImp( VDLabel* pLabel );
	virtual DRAWTYPE drawTextBoxImp( VDTextBox* pTextBox );
	virtual DRAWTYPE drawPanelImp( VDPanel*	pPanel );
	virtual DRAWTYPE drawScrollBarImp( VDScrollBar* pScrollBar);
	virtual DRAWTYPE drawTextButtonImp( VDTextButton* pTextButton);

	virtual bool isCutKey(VDKey* key)		{ return (key->getKey() == (VD_CONTROL | VD_X)) || (key->getKey() == (VD_SHIFT | VD_DELETE )); }
	virtual bool isCopyKey(VDKey* key)		{ return (key->getKey() == (VD_CONTROL | VD_INSERT)) || (key->getKey() == (VD_CONTROL | VD_C )); }
	virtual bool isPasteKey(VDKey* key)		{ return (key->getKey() == (VD_CONTROL | VD_V)) || (key->getKey() == (VD_SHIFT | VD_INSERT )); }
	virtual bool isFocusButton(int button)	{ return button == VD_BUTTON_LEFT; }
	virtual bool isFocusIncKey(VDKey* key)	{ return (key->getKey() == VD_RIGHT) || (key->getKey() == VD_TAB); }
	virtual bool isFocusDecKey(VDKey* key)	{ return (key->getKey() == VD_LEFT) || (key->getKey() == (VD_TAB | VD_SHIFT)); }
	virtual bool isResizeButton(int button)	{ return button == VD_BUTTON_LEFT; }
	virtual bool isMoveButton(int button)	{ return button == VD_BUTTON_LEFT; }
	virtual bool isScrollButton(int button) { return button == VD_BUTTON_LEFT; }
	virtual bool isScrollDown(VDKey* key)	{ return (key->getKey() == VD_DOWN); }


};
