#include "vd.h"

VDEditBox::VDEditBox(void)
{	
	m_caretPos = 0;
	m_selectionPos = -1;
	m_isSelecting = false;
	m_allowSelect = true;
	m_maxCharacters = 50;
	m_hasFocus = false;
}

VDEditBox::~VDEditBox(void)
{
}

bool VDEditBox::processEvent(VDEvent *pEvent)
{

	if( pEvent->getType() == REDISPLAY )	
	{
		if( !VDThemeStore::get() )
		{
			::log("Unable to get theme store in edit box process event: Editbox ID = %i", m_id);
			return true;
		}

		VDTheme* pTheme = VDThemeStore::get()->getCurrentTheme();
		if( !pTheme )
		{
			::log("Unable to get theme in edit box process event: Editbox ID = %i", m_id);
			return true;
		}

		pTheme->drawEditBox(this);
	}
	else if( pEvent->getType() == KEYDOWN )
	{
		VDKeyDownEvent*	e = dynamic_cast<VDKeyDownEvent*>(pEvent);
		if( e == NULL )
		{
			::log("Unable to case event to key down event even though its type says its one");
			return true;
		}
		
		// Edit boxes only recieve input when they have the focux
		if(!m_hasFocus)
		{
			return false;
		}

		VDKey k = e->getKey();

		// Handle selection state
		if( k.getCodeOnly() == VD_LEFT || k.getCodeOnly() == VD_RIGHT || k.getCodeOnly() == VD_HOME || k.getCodeOnly() == VD_END || k.getCodeOnly() == VD_BACKSPACE)
		{
			if(m_allowSelect)
			{
				if( !m_isSelecting && k.withShift() )
				{
					// Start selecting in this region
					m_selectionPos = m_caretPos;
					m_isSelecting = true;
				}
				else if( !k.withShift() )
				{
					m_isSelecting = false;
					m_selectionPos = -1;
				}
			}
		}

		// Handle text editing
		if( k.getCodeOnly() == VD_LEFT )
		{
			// Move the caret left if it can move			
			m_caretPos--;
			if( m_caretPos < 0)
				m_caretPos = 0;			

		}
		else if( k.getCodeOnly() == VD_RIGHT )
		{
			// Move the caret right if it can move
			int	size = (int)m_contents.size();
			m_caretPos++;
			if(m_caretPos > size)
				m_caretPos = size;
		}
		else if(k.getCodeOnly() == VD_HOME )
		{
			m_caretPos = 0;
		}
		else if(k.getCodeOnly() == VD_END )
		{
			m_caretPos = (int)m_contents.size();
		}
		else if(k.getCodeOnly() == VD_DELETE )
		{
			if ( m_isSelecting )
			{
			
				// Delete the section 
				if (m_caretPos <= (int)m_contents.length() && m_selectionPos <= (int)m_contents.length()) {
					string temp;
					// Sort the caret and selection position
					int a, b;
					a = m_caretPos <  m_selectionPos ? m_caretPos : m_selectionPos;
					b = m_caretPos >= m_selectionPos ? m_caretPos : m_selectionPos;

					// Copy to the clipboard if shift is down
					if ( VDThemeStore::get()->getCurrentTheme()->isCutKey(&k) )
						VDClipBoard::get()->copyString(m_contents.substr(a, b-a));

					// starting at a to a - 0 elements
					temp = m_contents.substr(0, a - 0);
					temp += m_contents.substr( b );
					m_contents = temp;
					
					// Reset the caret to a valid position
					if ( m_caretPos == b)
						m_caretPos -= (b-a);
				
					// Disable selecting mode
					m_isSelecting = false;
					m_selectionPos = -1;
				}
				else
				{
					::log("Unable to delete selected section of edit box");
				}
			}
			else
			{
				// Delete the next character
				if (m_caretPos < (int)m_contents.length()) {
					string temp;
					temp = m_contents.substr(0, m_caretPos );
					temp += m_contents.substr(m_caretPos + 1);
					m_contents = temp;
				}
			}
		}
		else if(k.getKey() == VD_BACKSPACE )
		{
			if (m_caretPos > 0) {
				string temp;
				temp = m_contents.substr(0, m_caretPos - 1);
				temp += m_contents.substr(m_caretPos);
				m_contents = temp;
				m_caretPos --;
			}

		}
		else if( VDThemeStore::get()->getCurrentTheme()->isPasteKey(&k) )
		{			
			// Overflows the max characters
			if(  (int)(m_contents.length() + VDClipBoard::get()->getString().length()) > m_maxCharacters )
				return true;

			string clipboard = VDClipBoard::get()->getString();
			m_contents.insert(m_caretPos, clipboard);			
		}
		else if( VDThemeStore::get()->getCurrentTheme()->isCopyKey(&k) )
		{
			if ( m_isSelecting )
			{
				// Sort the caret and selection position
				int a, b;
				a = m_caretPos <  m_selectionPos ? m_caretPos : m_selectionPos;
				b = m_caretPos >= m_selectionPos ? m_caretPos : m_selectionPos;

				// Copy to the clipboard
				VDClipBoard::get()->copyString(m_contents.substr(a, b-a));
			}
		}
		else if( k.isChar() )
		{
			// Overflows the max characters
			if (((int)m_contents.length()) >= m_maxCharacters )
				return true;

			string::iterator selStart = m_contents.begin() + m_caretPos;
			m_contents.insert(selStart, k.getChar());
			m_caretPos ++;

			m_isSelecting = false;
			m_selectionPos = -1;
		}		
		else
		{
			// Dont handle this event, give to other controls if they can use them without focus
			return false;
		}		

		// Request a redisplay
		VDRedisplayRequestEvent r;
		sendEvent(&r);

		// Gobble the events we handle
		return true;

	}

	// Didn't handle thie sone
	return false;
}