#include "vd.h"

VDEventGenerator::VDEventGenerator(void)
{
	m_pParentListener = NULL;
}

VDEventGenerator::~VDEventGenerator(void)
{
}

bool VDEventGenerator::sendEventToWindow( VDEvent* pEvent, int i )
{
	VDEventListener* pListener = (VDEventListener*)m_listeners[i];
	if( (pEvent->getDirection() == DOWN || pEvent->getDirection() == BOTH) && pListener->isActive() ) 
	{
		return pListener->processEvent( pEvent );			
	}
	return false;
}

bool VDEventGenerator::sendEventToAll( VDEvent* pEvent)
{
	// Send the event to the appropriate locations
	EVENTDIRECTION d = pEvent->getDirection();
	if( d == UP || d == BOTH)
		sendEventUp(pEvent);

	if( d == DOWN || d == BOTH )
		sendEventDown(pEvent);

	return false;
}
bool VDEventGenerator::sendEventDown( VDEvent* pEvent)
{

	// Send the event to each listener
	vector<VDEventListener*>::iterator i;
	for( i = m_listeners.begin(); i != m_listeners.end(); i++ )
	{
		VDEventListener* pListener = (VDEventListener*)(*i);
		if( pListener->isActive() ) 
		{
			pListener->processEvent( pEvent );			
		}
	}

	return false;

}

bool VDEventGenerator::sendEventUp( VDEvent* pEvent)
{
	// Send the event to the parent	
	if( m_pParentListener && m_pParentListener->isActive() )
	{
		m_pParentListener->processEvent( pEvent );			
	}

	return false;
}

void VDEventGenerator::unrequestEvents( VDEventListener* listener )
{

	// Remove from parent listener if it is one
	if(m_pParentListener == listener)
		m_pParentListener = NULL;

	// Remove from the list of listeners
	vector<VDEventListener*>::iterator i;
	for( i = m_listeners.begin(); i != m_listeners.end(); i++ )
	{
		if( (VDEventListener*)(*i) == listener ) 
		{
			i = m_listeners.erase(i);
			break;
		}
	}	
}

void VDEventGenerator::requestEventsDown( VDEventListener* listener )
{
	// Make sure this listener is not subscribed already
	unrequestEvents( listener );

	m_listeners.push_back( listener );

}

void VDEventGenerator::requestEventsUp( VDEventListener* listener )
{
	m_pParentListener = listener;
}