#pragma once

#include <list>
using namespace std;

#include "source\logging.h"

// Static singleton for throwing out objects when they ask to be thrown out
// Some objects are aware of when they should be deleted ( such as command objects, and message boxes )
// This class allows the programmer to dispose of these objects in a safe and easy manner

class VDDisposable
{	
public:
	VDDisposable() {}
	virtual ~VDDisposable()
	{ 
		::log("Disposing object with pointer %d", this);
	}
};

class VDGarbageCan
{
protected:
	static VDGarbageCan*	s_pGarbageCan;

	// The list of disposables to dispose
	list<VDDisposable*>		m_disposables;

	// This boolean ensures the garbage can is not called
	// recursively
	bool					m_emptying;

	VDGarbageCan(void);
	virtual ~VDGarbageCan(void);
public:
	static VDGarbageCan*	create()	{ if( !s_pGarbageCan ) s_pGarbageCan = new VDGarbageCan(); return s_pGarbageCan; }
	static VDGarbageCan*	get()		{ return s_pGarbageCan; }
	static void				destroy()	{ if( s_pGarbageCan) { s_pGarbageCan->empty(); delete s_pGarbageCan; s_pGarbageCan = NULL; } }

	void	dispose(VDDisposable* pDisposable) { m_disposables.push_back(pDisposable); }
	void	empty();
};
