#pragma once

typedef unsigned int VD_KEY;

//Modifiers
#define VD_SHIFT		0x1000
#define VD_CONTROL		0x2000
#define VD_ALT			0x4000

// Modifier Mask
#define VD_MODIFIER_MASK	0xF000
// Code mask
#define VD_CODE_MASK		(~VD_MODIFIER_MASK)
// Ascii mask
#define VD_ASCII_MASK		0xFF

//Actual key code
#define VD_F1			0x0101
#define VD_F2			0x0102 
#define VD_F3			0x0103 
#define VD_F4			0x0104 
#define VD_F5			0x0105 
#define VD_F6			0x0106 
#define VD_F7			0x0107 
#define VD_F8			0x0108 
#define VD_F9			0x0109 
#define VD_F10			0x010A 
#define VD_F11			0x010B 
#define VD_F12			0x010C 
#define VD_LEFT			0x010D 
#define VD_UP			0x010E 
#define VD_RIGHT		0x010F 
#define VD_DOWN			0x0100 
#define VD_PAGE_UP		0x0110 
#define VD_PAGE_DOWN	0x0111 
#define VD_HOME			0x0112 
#define VD_END			0x0113 
#define VD_INSERT		0x0114

// Define ASCII codes here
#define VD_DELETE		127
#define VD_BACKSPACE	8
#define VD_ENTER		13
#define VD_TAB			9
#define VD_ESCAPE		27
#define VD_SPACE		32
#define VD_TILDE		96

#define VD_A			97
#define VD_B			98
#define VD_C			99
#define VD_D			100
#define VD_E			101
#define VD_F			102
#define VD_G			103
#define VD_H			104
#define VD_V			118
#define VD_X			120
#define VD_Y			121
#define VD_Z			122

class VDKey
{
protected:
	VD_KEY	m_keyCode;

public:
	VDKey()					{ m_keyCode = 0;		}
	VDKey(VD_KEY keyCode)	{ m_keyCode = keyCode;	}
	virtual ~VDKey(void)	{}

	bool withAlt()		{ return (m_keyCode & VD_ALT) != 0;		}
	bool withControl()	{ return (m_keyCode & VD_CONTROL) != 0;	}
	bool withShift()	{ return (m_keyCode & VD_SHIFT) != 0;		}

	void applyAlt()			{ m_keyCode |= VD_ALT;		}
	void applyShift()		{ m_keyCode |= VD_SHIFT;	}
	void applyControl()		{ m_keyCode |= VD_CONTROL; 	}

	void removeAlt()		{ m_keyCode &= ~VD_ALT;		}
	void removeShift()		{ m_keyCode &= ~VD_SHIFT;	}
	void removeControl()	{ m_keyCode &= ~VD_CONTROL;	}

	void toggleAlt()		{ m_keyCode ^= VD_ALT;		}
	void toggleShift()		{ m_keyCode ^= VD_SHIFT;	}
	void toggleControl()	{ m_keyCode ^= VD_CONTROL;	}

	VD_KEY	getCodeOnly()	{ return m_keyCode & VD_CODE_MASK; }
	VD_KEY	getKey()		{ return m_keyCode; }	

	// returns true if the key can be converted to an ascii representation
	bool	isChar();
	unsigned char	getChar()		{ return (unsigned char)(m_keyCode & VD_ASCII_MASK); }

};
