#include "vd.h"

VDPanel::VDPanel(void)
{
	m_moveable					= true;
	m_resizable					= true;
	m_moveBarHeight				= 20;
	m_resizeBorderWidth			= 5;
	m_isMoving					= false;	

	m_mouseOverFlags			= 0;
	m_resizingFlags				= 0;
}

VDPanel::~VDPanel(void)
{
}

bool VDPanel::processEvent(VDEvent* pEvent)
{
		
	if( pEvent->getType() == REDISPLAY )
	{
		// Draw this panel again
		VDThemeStore::get()->getCurrentTheme()->drawPanel(this);
		
	}
	else if( pEvent->getType() == MOUSEMOVE )
	{
		VDMouseMoveEvent* pMove = dynamic_cast<VDMouseMoveEvent*>(pEvent);

		// Is the panel moving
		if( m_isMoving )
		{
			// Move the left window bound to the cursor
			VDRectangleContainer<float> bounds;
			// Calculate offset from window upper left
			float x1 = pMove->getMouseX();
			float y1 = pMove->getMouseY();
			bounds = m_bounds;
			bounds.translate(x1 - m_memX, y1 - m_memY);
			m_memX = x1;
			m_memY = y1;
			if( bounds.getX1() >= 0 &&  bounds.getY1() >= 0)
			{
				resizeControls(bounds);

				VDRedisplayRequestEvent e;
				sendEvent(&e);
			}

		}
		else if( m_resizingFlags != 0 )// Is the panel resizing?
		{
			// take care of the resizing
			if( m_resizingFlags & PANELFLAG_LEFT)
			{			
				// Move the left window bound to the cursor
				VDRectangleContainer<float> bounds;				
				float x1 = pMove->getMouseX();
				if( x1 > (m_bounds.getX2() - 32*m_resizeBorderWidth) )
					x1 = (m_bounds.getX2() - 32*m_resizeBorderWidth);
				if( x1 < 0 )
					x1 = 0;
				bounds = m_bounds;
				bounds.setX1(x1);
				resizeControls(bounds);
			}
			else if( m_resizingFlags & PANELFLAG_RIGHT)
			{			
				// Move the left window bound to the cursor
				VDRectangleContainer<float> bounds;				
				float x2 = pMove->getMouseX();
				if( x2 < (m_bounds.getX1() + 32*m_resizeBorderWidth) )
					x2 = (m_bounds.getX1() + 32*m_resizeBorderWidth);
				bounds = m_bounds;
				bounds.setX2(x2);
				resizeControls(bounds);
			}

			if( m_resizingFlags & PANELFLAG_UP)
			{			
				// Move the left window bound to the cursor
				VDRectangleContainer<float> bounds;				
				float y2 = pMove->getMouseY();
				if( y2 < (m_bounds.getY1() + 32*m_resizeBorderWidth) )
					y2 = (m_bounds.getY1() + 32*m_resizeBorderWidth);
				bounds = m_bounds;
				bounds.setY2(y2);
				resizeControls(bounds);
			}
			else if( m_resizingFlags & PANELFLAG_DOWN)
			{			
				// Move the left window bound to the cursor
				VDRectangleContainer<float> bounds;				
				float y1 = pMove->getMouseY();
				if( y1 > (m_bounds.getY2() - 32*m_resizeBorderWidth) )
					y1 = (m_bounds.getY2() - 32*m_resizeBorderWidth);
				if( y1 < 0 )
					y1 = 0;
				bounds = m_bounds;
				bounds.setY1(y1);
				resizeControls(bounds);
			}

			VDRedisplayRequestEvent e;
			sendEvent(&e);

		}
		else
		{
			MOUSEOVERFLAGS newFlags = 0;			

			if( m_moveable )
			{
				// Handle mouse over states
				// check if the mouse is pressed on the border
				VDRectangleContainer<float>	m_bar;
				// Test left/right
				m_bar.setBounds( m_bounds.getX1() + m_resizeBorderWidth, m_bounds.getY2() - m_resizeBorderWidth - m_moveBarHeight,
									m_bounds.getX2() - m_resizeBorderWidth, m_bounds.getY2() - m_resizeBorderWidth);
				if( m_moveable && m_bar.containsPoint( pMove->getMouseX(), pMove->getMouseY() ))
					newFlags |= PANELFLAG_MOVING;

			}

			if( m_resizable )
			{
				VDRectangleContainer<float>	m_border;
				// Test left/right
				m_border.setBounds( m_bounds.getX1(), m_bounds.getY1(), m_bounds.getX1() + m_resizeBorderWidth, m_bounds.getY2());
				if( m_border.containsPoint( pMove->getMouseX(), pMove->getMouseY() ))
					newFlags |= PANELFLAG_LEFT;
				else
				{
					m_border.setBounds( m_bounds.getX2() - m_resizeBorderWidth, m_bounds.getY1(), m_bounds.getX2(), m_bounds.getY2());
					if( m_border.containsPoint( pMove->getMouseX(), pMove->getMouseY() ))
						newFlags |= PANELFLAG_RIGHT;
				}
				// Test up/down
				m_border.setBounds( m_bounds.getX1(), m_bounds.getY1(), m_bounds.getX2(), m_bounds.getY1() + m_resizeBorderWidth);
				if( m_border.containsPoint( pMove->getMouseX(), pMove->getMouseY() ))
					newFlags |= PANELFLAG_DOWN;
				else
				{
					m_border.setBounds( m_bounds.getX1(), m_bounds.getY2() - m_resizeBorderWidth, m_bounds.getX2(), m_bounds.getY2());
					if( m_border.containsPoint( pMove->getMouseX(), pMove->getMouseY() ))
						newFlags |= PANELFLAG_UP;
				}
			}

			if( m_mouseOverFlags != newFlags )
			{
				m_mouseOverFlags = newFlags;
				VDRedisplayRequestEvent e;
				sendEvent(&e);
			}
	
		}

	}
	else if( pEvent->getType() == MOUSEDOWN )
	{
		VDMouseDownEvent* pDown = dynamic_cast<VDMouseDownEvent*>(pEvent);
		if( m_moveable && VDThemeStore::get()->getCurrentTheme()->isMoveButton(pDown->getButton()) )
		{
			m_isMoving = false;
			// check if the mouse is pressed on the border
			VDRectangleContainer<float>	m_bar;
			// Test left/right
			m_bar.setBounds( m_bounds.getX1() + m_resizeBorderWidth, m_bounds.getY2() - m_resizeBorderWidth - m_moveBarHeight,
								m_bounds.getX2() - m_resizeBorderWidth, m_bounds.getY2() - m_resizeBorderWidth);
			if( m_bar.containsPoint( pDown->getMouseX(), pDown->getMouseY() ))
			{
				m_isMoving = true;
				m_memX = pDown->getMouseX();
				m_memY = pDown->getMouseY();
			}
		}

		if( m_resizable && VDThemeStore::get()->getCurrentTheme()->isResizeButton(pDown->getButton()) )
		{
			// Reset the resizing flags			
			m_resizingFlags = 0;

			// check if the mouse is pressed on the border
			VDRectangleContainer<float>	m_border;
			// Test left/right
			m_border.setBounds( m_bounds.getX1(), m_bounds.getY1(), m_bounds.getX1() + m_resizeBorderWidth, m_bounds.getY2());
			if( m_border.containsPoint( pDown->getMouseX(), pDown->getMouseY() ))
				m_resizingFlags |= PANELFLAG_LEFT;
			else
			{
				m_border.setBounds( m_bounds.getX2() - m_resizeBorderWidth, m_bounds.getY1(), m_bounds.getX2(), m_bounds.getY2());
				if( m_border.containsPoint( pDown->getMouseX(), pDown->getMouseY() ))
					m_resizingFlags |= PANELFLAG_RIGHT;
			}
			// Test up/down
			m_border.setBounds( m_bounds.getX1(), m_bounds.getY1(), m_bounds.getX2(), m_bounds.getY1() + m_resizeBorderWidth);
			if( m_border.containsPoint( pDown->getMouseX(), pDown->getMouseY() ))
				m_resizingFlags |= PANELFLAG_DOWN;
			else
			{
				m_border.setBounds( m_bounds.getX1(), m_bounds.getY2() - m_resizeBorderWidth, m_bounds.getX2(), m_bounds.getY2());
				if( m_border.containsPoint( pDown->getMouseX(), pDown->getMouseY() ) )
					m_resizingFlags |= PANELFLAG_UP;
			}

		}
	}
	else if( pEvent->getType() == MOUSEUP )
	{
		VDMouseUpEvent* pUp = dynamic_cast<VDMouseUpEvent*>(pEvent);
		if( m_resizingFlags && VDThemeStore::get()->getCurrentTheme()->isResizeButton(pUp->getButton()) )
		{
			m_resizingFlags = 0;
		}
		if( m_isMoving && VDThemeStore::get()->getCurrentTheme()->isMoveButton(pUp->getButton()) )
		{
			m_isMoving = false;
		}
	}

	if(sendEvent(pEvent))
		return true;

	// Handle the default focus stuff
	return __super::processEvent(pEvent);

}
