#pragma once

// This class organizes a set of controls into a group
class VDWindow;

typedef unsigned int RESIZINGFLAGS;
typedef unsigned int MOUSEOVERFLAGS;

// Valid resizing flags and mouse over flags
#define PANELFLAG_LEFT			1
#define PANELFLAG_RIGHT			2
#define PANELFLAG_UP			4
#define PANELFLAG_DOWN			8

// Mouse over flag only
#define PANELFLAG_MOVING		16

class VDPanel : public VDWindow
{
protected:
	// Indicates this panel can be move around with the mouse
	bool	m_moveable;

	// Indicates this panel is resizeable
	bool	m_resizable;

	// The height of the move bar
	float	m_moveBarHeight;

	// The resize border width
	float	m_resizeBorderWidth;

	// The name of the panel
	string	m_name;

	// Is the panel moving?
	bool	m_isMoving;

	// Is the panel resizing
	RESIZINGFLAGS	m_resizingFlags;

	// Mouse over state
	MOUSEOVERFLAGS	m_mouseOverFlags;

	// Position memory for movement and resizing
	float	m_memX, m_memY;

public:
	VDPanel(void);
	virtual ~VDPanel(void);

	virtual bool processEvent(VDEvent* pEvent);

	void setName(string name)				{ m_name = name; }
	void setBarHeight(float height)			{ m_moveBarHeight = height; }
	void setBorderWidth(float width)		{ m_resizeBorderWidth = width; }
	void setMoveable(bool yes)				{ m_moveable = yes; }
	void setResizeable(bool yes)			{ m_resizable = yes; }

	string	getName()					{ return m_name; }
	float	getBarHeight()				{ return m_moveBarHeight; }
	float	getBorderWidth()			{ return m_resizeBorderWidth; }
	bool	isMoveable()				{ return m_moveable; }
	bool	isResizeable()				{ return m_resizable; }

	bool			isMoving()					{ return m_isMoving; }
	RESIZINGFLAGS	getResizingFlags()			{ return m_resizingFlags; }
	MOUSEOVERFLAGS	getMouseOverFlags()			{ return m_mouseOverFlags; }

};
