#include "vd.h"

VDScrollBar::VDScrollBar()
{
	m_sliderPos = 0;
	m_sliderMax = 0;
	m_isScrolling = false;
}

VDScrollBar::~VDScrollBar()
{
}

void VDScrollBar::getSliderBounds(VDRectangleContainer<float>* bounds)
{
	float sliderSteps = (m_bounds.getHeight() - m_sliderSize) / (float)m_sliderMax;
	if( m_direction == VERTICAL )
	{
		bounds->setBounds(m_bounds.getX1(), m_bounds.getY2() - m_sliderPos*sliderSteps - m_sliderSize, m_bounds.getX2(), m_bounds.getY2() - m_sliderPos*sliderSteps );
	}
	else
	{
		bounds->setBounds(m_bounds.getX1() + m_sliderPos*sliderSteps, m_bounds.getY2(), m_bounds.getX1() + m_sliderPos*sliderSteps + m_sliderSize, m_bounds.getY2());
	}
}

void VDScrollBar::updateSlideSize()
{

	int sliderSizeInt = m_sliderMax;
	if( sliderSizeInt < 2)
		sliderSizeInt = 2;
	if( sliderSizeInt > 20)
		sliderSizeInt = 20;

	// Set the slider size based on the max size
	if( m_direction == VERTICAL )
		m_sliderSize = m_bounds.getHeight() / sliderSizeInt;
	else if( m_direction == HORIZONTAL )
		m_sliderSize = m_bounds.getWidth() / sliderSizeInt;
}

void VDScrollBar::setScrollPos(int pos)
{ 
	if( m_sliderPos != pos) 
	{ 
		m_sliderPos = pos;
		if( m_sliderPos < 0)
			m_sliderPos = 0;
		else if( m_sliderPos > m_sliderMax)
			m_sliderPos = m_sliderMax;

		VDScrollValueEvent e(m_id, m_sliderPos);
		sendEvent(&e);
	}
}

bool VDScrollBar::processEvent(VDEvent* pEvent)
{
	
	if( pEvent->getType() == REDISPLAY )
	{
		// Draw this scrollbar
		VDThemeStore::get()->getCurrentTheme()->drawScrollBar(this);
		
	}
	else if( pEvent->getType() == MOUSEMOVE )
	{
		VDMouseMoveEvent* pMove = dynamic_cast<VDMouseMoveEvent*>(pEvent);

		// Check if we are moving the scrollbar
		if( m_isScrolling && m_bounds.containsPoint(pMove->getMouseX(), pMove->getMouseY()))
		{
			// Set the new position
			//***************

			// Calculate offset from window upper left
			float x1 = pMove->getMouseX();
			float y1 = pMove->getMouseY();
			if( m_direction == VERTICAL )
			{
				setScrollPos(m_memsliderPos + ( m_memPos - y1) * ( (float)m_sliderMax / (m_bounds.getHeight() - m_sliderSize) ));
			}
			else
			{
				setScrollPos(m_memsliderPos + ( x1 - m_memPos ) * ( (float)m_sliderMax / (m_bounds.getWidth() - m_sliderSize) ));
			}

			VDRedisplayRequestEvent e;
			sendEvent(&e);
		}
	}
	else if( pEvent->getType() == MOUSEDOWN )
	{
		VDMouseDownEvent* pDown = dynamic_cast<VDMouseDownEvent*>(pEvent);

		if( VDThemeStore::get()->getCurrentTheme()->isScrollButton( pDown->getButton() ) )
		{
			// Check if the button was pressed on the slider
			VDRectangleContainer<float>	bounds;
			getSliderBounds(&bounds);
			if( bounds.containsPoint( pDown->getMouseX(), pDown->getMouseY() ) )
			{
				m_isScrolling = true;
				m_memsliderPos = m_sliderPos;
				if( m_direction == VERTICAL)
					m_memPos = pDown->getMouseY();
				else 
					m_memPos = pDown->getMouseX();
			}
		}
	}
	else if( pEvent->getType() == MOUSEUP )
	{
		VDMouseUpEvent* pUp = dynamic_cast<VDMouseUpEvent*>(pEvent);
		if( VDThemeStore::get()->getCurrentTheme()->isScrollButton( pUp->getButton() ) )
		{
			m_isScrolling = false;
		}		
	}

	if(sendEvent(pEvent))
		return true;

	// Handle the default focus stuff
	return __super::processEvent(pEvent);


	return false;
}

void VDScrollBar::resizeControls(VDRectangleContainer<float>& bounds)
{
	m_bounds = bounds;
	updateSlideSize();
}