#include "vd.h"

VDTextBox::VDTextBox(bool vscroll)
{
	m_vscroll = vscroll;
	if( m_vscroll )
		m_vscrollWidth = 30.0f;
	else
		m_vscrollWidth = 0.0f;
	
	m_pVScrollBar = NULL;
	m_currentLine = 0;
}

VDTextBox::~VDTextBox(void)
{
}

void VDTextBox::setupControls()
{
	if( m_vscroll )
	{
		m_pVScrollBar = new VDScrollBar();
		m_focusWindow = addChildWindow(m_pVScrollBar);
		m_pVScrollBar->setFocus(true);
		m_pVScrollBar->setParentWindow(this);
		m_pVScrollBar->setDirection(VERTICAL);
		m_pVScrollBar->setScrollMax(m_text.getSize());
		m_pVScrollBar->setScrollPos(m_text.getSize());

	}

	resizeControls(m_bounds);

}

void VDTextBox::resizeControls( VDRectangleContainer<float>& newBounds )
{
	m_bounds = newBounds;
	
	if( m_vscroll && m_pVScrollBar)
	{
		VDRectangleContainer<float> bounds;		
		bounds.setBounds(m_bounds.getX2() - m_vscrollWidth, m_bounds.getY1(), m_bounds.getX2(), m_bounds.getY2() );
		m_pVScrollBar->resizeControls(bounds);
	}

}

void VDTextBox::printLine( string line )
{
	m_text.addString(line);
	if( m_vscroll)
	{
		m_pVScrollBar->setScrollMax(m_text.getSize() - 1);
		m_pVScrollBar->setScrollPos(m_text.getSize() - m_currentLine - 1);
	}
}

bool VDTextBox::processEvent(VDEvent* pEvent)
{
	if ( pEvent->getType() == REDISPLAY )
	{
		//Draw the text box
		VDThemeStore::get()->getCurrentTheme()->drawTextBox(this);

	}
	else if( pEvent->getType() == SCROLLVALUE )
	{		
		VDScrollValueEvent *pScroll = dynamic_cast<VDScrollValueEvent*>(pEvent);
		m_currentLine = m_text.getSize() - 1 - pScroll->getScrollValue();

		VDRedisplayRequestEvent e;
		sendEvent(&e);

		return true;
	}

	return sendEvent(pEvent);
}

void VDTextBox::setCurrentLine(int line)
{
	m_currentLine = line;
	if( m_currentLine < 0 )
		m_currentLine = 0;
	else if( m_currentLine > (m_text.getSize()) )
		m_currentLine = (m_text.getSize());

	if( m_vscroll)
	{
		m_pVScrollBar->setScrollPos(m_text.getSize() - 1 - m_currentLine);
	}
}