#include "source\includes.h"
#include "vd.h"

VDWindow::VDWindow(void)
{
	m_bounds.setBounds(0, 0, 1, 1);
	m_mouseOver = false;
	m_focusWindow = 0;
	m_hasFocus = false;
}

VDWindow::~VDWindow(void)
{
	// Delete child windows
	vector<VDWindow*>::iterator i = NULL;
	for(i=m_subWindows.begin(); i!=m_subWindows.end(); i++)
	{
		if( ((VDWindow*)(*i)) != NULL )
		{
			// Remove from event list as well
			unrequestEvents(((VDWindow*)(*i)));
			delete ((VDWindow*)(*i));
		}
			
	}
}

bool VDWindow::processEvent(VDEvent *pEvent)
{
	// First off check if this window is active
	if(!m_isActive)
	{
		::log("Inactive window sent a message");
		return false;
	}

	// Check if the focus should change
	if ( pEvent->getType() == MOUSEDOWN )
	{
		// Check if we left clicked on a child window
		VDMouseDownEvent* e = dynamic_cast<VDMouseDownEvent*>(pEvent);
		if( e == NULL )
		{
			::log("Invalid mouse down event recieved");
			return true;
		}

		int button	= e->getButton();
		int x		= e->getMouseX();
		int y		= e->getMouseY();

		if( VDThemeStore::get()->getCurrentTheme()->isFocusButton(button) )
		{
			int j = 0;
			vector<VDWindow*>::iterator i = m_subWindows.begin();
			while( i != m_subWindows.end() )
			{
				VDWindow* pWin = (VDWindow*)(*i);
				VDRectangleContainer<float> bounds;
				pWin->getBounds(&bounds);
				if( bounds.containsPoint(x, y) )
				{
					// Set the focus window... Find the closest one for layers
					m_focusWindow = j;
					break;
				}
				j++;
				i++;
			}
		}
	}
	else if( pEvent->getType() == KEYDOWN )
	{
		VDKeyDownEvent* e = dynamic_cast<VDKeyDownEvent*>(pEvent);
		if( e == NULL )
		{
			::log("Invalid key down event received");
			return false;
		}

		VDKey k = e->getKey();
		if ( VDThemeStore::get()->getCurrentTheme()->isFocusIncKey(&k) )
		{
			m_focusWindow++;
			if(m_focusWindow > ((int)m_subWindows.size() - 1))
				m_focusWindow = 0;

		}
		if ( VDThemeStore::get()->getCurrentTheme()->isFocusDecKey(&k) )
		{
			m_focusWindow--;
			if(m_focusWindow < 0)
				m_focusWindow = ((int)m_subWindows.size() - 1);
		}
	}

	return false;
}

void VDWindow::setupControls()
{
}

bool VDWindow::sendEvent(VDEvent* pEvent)
{

	// Ensure there is a focus window
	if( m_focusWindow >= 0 && m_focusWindow < (int)m_subWindows.size() )
	{
		// Check if the focus has changed	
		VDWindow* pWin = m_subWindows[m_focusWindow];
		if( !pWin->hasFocus() )
		{
			// Reset all the windows
			vector<VDWindow*>::iterator i = m_subWindows.begin();
			while( i != m_subWindows.end() )
			{
				VDWindow* pAll = (VDWindow*)(*i);
				pAll->setFocus(false);
				i++;
			}

			// Set the focus window
			pWin->setFocus(true);

			// Send a redisplay event to show the new focus
			VDRedisplayRequestEvent e;
			sendEvent(&e);
		}
		
		if(sendEventToWindow(pEvent, m_focusWindow))
			return true;
	}

	// The focus window didn't gobble it.
	// send it to the others to gobble
	return sendEventToAll(pEvent);
	
}

void VDWindow::resizeControls( VDRectangleContainer<float>& newBounds )
{

	// Get translation
	float translationX = newBounds.getX1() - m_bounds.getX1();
	float translationY = newBounds.getY1() - m_bounds.getY1();

	// Get scale
	float scaleX = newBounds.getWidth() / m_bounds.getWidth();
	float scaleY = newBounds.getHeight() / m_bounds.getHeight();

	// Loop through the child windows resizing each of them
	vector<VDWindow*>::iterator i = m_subWindows.begin();
	VDRectangleContainer<float>	childBounds;
	VDRectangleContainer<float>	newChildBounds;

	while( i != m_subWindows.end() )
	{
		VDWindow* pWin = (VDWindow*)(*i);	
		pWin->getBounds(&childBounds);

		// Calculate the new bounds
		newChildBounds.setX1( childBounds.getX1() + translationX );
		newChildBounds.setY1( childBounds.getY1() + translationY );
		newChildBounds.setX2( newChildBounds.getX1() + childBounds.getWidth() * scaleX );
		newChildBounds.setY2( newChildBounds.getY1() + childBounds.getHeight() * scaleY );

		pWin->resizeControls(newChildBounds);
		
		i++;
	}

	// Set the new bounds
	m_bounds = newBounds;

}
