#pragma once

class VDEvent;
class VDMouse;
template class VDRectangleContainer<float>;

class VDWindow : public VDEventListener, public VDEventGenerator
{
private:
	// Parent window
	VDWindow*					m_pParentWindow;
	// List of controls in this window
	vector<VDWindow*>			m_subWindows;

protected:	
	VDRectangleContainer<float>	m_bounds;
	WINDOWID					m_id;

	// Whether this window has the cursor over it
	bool						m_mouseOver;
	// Whether this window has the focus
	bool						m_hasFocus;
	// The sub window to send input to before handling in the parent window
	int							m_focusWindow;


public:
	VDWindow(void);
	virtual ~VDWindow(void);
	
	virtual bool sendEvent(VDEvent* pEvent);

	virtual bool processEvent(VDEvent *pEvent);
	virtual void setupControls();

	virtual WINDOWID				getID()											{ return m_id; }	
	void							getBounds(VDRectangleContainer<float>* bounds)	{ bounds->setBounds(m_bounds.getX1(),m_bounds.getY1(),m_bounds.getX2(),m_bounds.getY2()); }
	bool							getMouseOverState()								{ return m_mouseOver; }
	bool							hasFocus()										{ return m_hasFocus; }

	void							setFocus(bool yes)	{ m_hasFocus = yes; }
	void							setID(WINDOWID id)	{ m_id = id; }

	// Default resizing functionality
	virtual void					resizeControls( VDRectangleContainer<float>& newBounds );

	// Parent window functionality
	void	setParentWindow(VDWindow* pWindow)	{ m_pParentWindow = pWindow; requestEventsUp(pWindow); }
	void	unsetParentWindow()					{ unrequestEvents(m_pParentWindow); m_pParentWindow = NULL; }

	// Child window functionality
	int		addChildWindow(VDWindow* pWindow)	{ m_subWindows.push_back(pWindow); requestEventsDown(pWindow); return ((int)m_subWindows.size() - 1); }
	bool	removeChildWindow(VDWindow* pWindow)
	{
		vector<VDWindow*>::iterator i = NULL;
		for(i = m_subWindows.begin(); i != m_subWindows.end(); i++)
		{
			if( (VDWindow*)(*i) == pWindow )
			{
				i = m_subWindows.erase(i);
				break;
			}
		}

		// Remove from event list as well
		unrequestEvents(pWindow);
	}
	vector<VDWindow*>::iterator getChildWindowIterator() { return m_subWindows.begin(); }

};
