#include "includes.h"

// World Object -----------------------------------
void WorldObject::tick() {
	Face *face = NULL;
	LOOPVECTOR(faceIndex, m_mesh.m_faces)
		face = (Face*)m_mesh.m_faces[faceIndex];
		face->tick();
	ENDLOOP
}
void WorldObject::draw() {
	m_mesh.draw();
}

bool WorldObject::deserialize(BinaryFileStreamReader *input) {
	return m_mesh.deserialize(input);
}
// ------------------------------------------------


// LocalObject-------------------------------------
bool LocalObject::deserialize(BinaryFileStreamReader *input) {
	int meshID = input->readInteger();
	m_mesh = MeshList::Get()->getMesh(meshID);
	
	m_position.deserialize(input);
	m_rotation.deserialize(input);
	
	string tag;
	
	input->readTag(&tag); //LOP
	int propertyCount = input->readInteger();
	for(int propertyIndex = 0; propertyIndex < propertyCount; propertyIndex++) {
		
		string key, value;
		input->readString( &key);
		input->readString( &value);
		
		char* newValue = new char[strlen(value.c_str() ) + 1];
		strcpy(newValue, value.c_str() );

		m_properties.insert(Property(key, newValue));		
	}

	return true;
}

void LocalObject::draw() {
	Transformation matrix;
	matrix.setToIdentity();
		
	matrix.translateBy(m_position);
	matrix.rotateBy(m_rotation);
		
	glPushMatrix();
	glMultMatrixd(matrix);
	
	if(m_mesh)
		m_mesh->draw();

	glPopMatrix();
}
// ------------------------------------------------