#pragma once

/*
	My Idea for handling objects
	------------------------------
	IObject	
		- base class for all objects
	WorldObject	: IObject
		- objects that dont move and are part of the level
	ILocalObject : IObject
		- base class for moveable items
	LocalObject	: ILocalObject	
		- defines position, rotation, and has a FaceCollecion
	LocalObjectGroup :ILocalObject
		- an object that consists of multiple LocalObjects
		- defines ILocalObjectCollection;

	Then Object 'classes' such as Vehicle would inherit from one of the above.
*/

#define CLASSKEY	"classname"

typedef map<string, char*> PropertyMap;
typedef PropertyMap::iterator PropertyMapIterator;
typedef pair <string, char*> Property;


class LocalObject : public ILocalObject {
protected:
	PropertyMap m_properties;

public:
	Mesh *m_mesh;
	Point m_position;
	Vector m_rotation;

public:	

	void tick() {};
	void draw();

	bool serialize(BinaryFileStreamWriter *output) {return false;};
	bool deserialize(BinaryFileStreamReader *input);
	
	char* getProperty(string key) {
		PropertyMapIterator iterator = m_properties.find(key);
		if (iterator != m_properties.end()) {
			return iterator->second;
		} else {
			return NULL;
		}
		//return m_properties[key]; 
	}
	char* getClass() { 
		return getProperty(CLASSKEY);
	}

	void dispose() {
		// DE-allocate the map contents
		PropertyMapIterator iter;
		for(iter = m_properties.begin(); iter != m_properties.end(); iter++)
		{
			//Property* p = *iter;
			delete [] (char*)iter->second;
		}
	};

	virtual void moveBy(Point offset) {moveBy(offset.x, offset.y, offset.x);};
	virtual void moveBy(double x, double y, double z) {
		m_position.x +=x;
		m_position.y +=y;
		m_position.z +=z;
	};

	virtual void moveTo(Point position) {moveTo(position.x, position.y, position.z);};
	virtual void moveTo(double x, double y, double z) {
		m_position.x =x;
		m_position.y =y;
		m_position.z =z;
	};

	virtual void rotateBy(Point offset) {rotateBy(offset.x, offset.y, offset.z);};
	virtual void rotateBy(double x, double y, double z) {
		m_rotation.x +=x;
		m_rotation.y +=y;
		m_rotation.z +=z;
	};
	
	virtual void rotateTo(Point position) {
		rotateTo(position.x, position.y, position.z);
	};
	virtual void rotateTo(double x, double y, double z) {
		m_rotation.x =x;
		m_rotation.y =y;
		m_rotation.z =z;
	};

	virtual void getPosition(Point* position) {*position = m_position;};
	virtual void getRotation(Point* rotation) {*rotation = m_rotation;};
};
DECLARECOLLECTION(LocalObject);

class LocalObjectGroup : public ILocalObject {
protected:
	ILocalObjectCollection m_objects;
public:
	Point m_position;
	Vector m_rotation;

};

class WorldObject : public IObject {
protected:
	Mesh m_mesh;
public:
	void tick();
	void draw();
	void dispose() {};
	
	// Serialize is only needed in the Builder
	bool serialize(BinaryFileStreamWriter *output) {return false;};
	bool deserialize(BinaryFileStreamReader *input);
};
DECLARECOLLECTION(WorldObject);