#include "includes.h"
extern int centerX, centerY;

WorldUI::WorldUI(Game *game) {
	_game = game;
	_camera = &_freeCamera;
	_followCamera.m_maxSpeed = 10.0;
	_camMode = FREECAMERA;
	_cursorVisible = true;
	_isDisposed = false;
	setupFonts();

	if (_game->currentWorld != NULL) {
		// recenter mouse so it doesnt affect rotation
		SetCursorPos(centerX,centerY);
		_camera->moveTo(_game->currentWorld->startPosition);
		_camera->rotateTo(_game->currentWorld->startRotation);		
	}
}

void WorldUI::setupFonts() {
	HDC hdc = GetDC(NULL);
	_debugFont = new Font("Courier New", 16);
	_smallFont = new Font("Courier New", 13);
}
void WorldUI::cleanupFonts() {
	_debugFont->dispose();
	_smallFont->dispose();
	delete _debugFont;
	delete _smallFont;
}


bool WorldUI::processKeyDown(unsigned char key, int x, int y) {
	FreeCamera *fc = (FreeCamera*)_camera;
	if (_camMode == FOLLOWCAMERA) {
		// Fix Later
		// followCamer.m_pTarget returns an IMOVEABLE

		LocalObject *obj = (LocalObject*)_followCamera.m_pTarget;
		if (obj == NULL || strcmp(obj->getClass(), "Jeep") != 0)
			return false;

		Jeep *j = (Jeep*)_followCamera.m_pTarget;
		switch (key) {
			case 'w': case 'W':
				j->driveForward = true;
				break;
			case 's': case 'S':
				j->driveBackward = true;
				break;
			case 'a': case 'A':
				j->turnLeft = true;
				break;
			case 'd': case 'D':
				j->turnRight = true;
				break;
			default:
				return false;
		}
	} else {
		switch (key) {
			case 'w': case 'W':
				fc->translateAhead = true;
				break;
			case 's': case 'S':
				fc->translateBack = true;
				break;
			case 'a': case 'A':
				fc->translateLeft = true;
				break;
			case 'd': case 'D':
				fc->translateRight = true;
				break;
			case '?':
				//game->_showHelp = !game->_showHelp;
	//			game->ui->showHelp = !game->ui->showHelp;
				break;
			case 't':
			case 'T':
				//wireframe = !wireframe;
				//glPolygonMode (GL_FRONT_AND_BACK, wireframe ? GL_LINE : GL_FILL);
				break;
			default:
				return false;
		}
	}
	return true;
}

bool WorldUI::processKeyUp(unsigned char key, int x, int y) {
	FreeCamera *fc = (FreeCamera*)_camera;
	if (_camMode == FOLLOWCAMERA) {
		LocalObject *obj = (LocalObject*)_followCamera.m_pTarget;
		if (obj == NULL || strcmp(obj->getClass(), "Jeep") != 0)
			return false;

		Jeep *j = (Jeep*)_followCamera.m_pTarget;
		switch (key) {
			case 'w': case 'W':
				j->driveForward = false;
				break;
			case 's': case 'S':
				j->driveBackward = false;
				break;
			case 'a': case 'A':
				j->turnLeft = false;
				break;
			case 'd': case 'D':
				j->turnRight = false;
				break;
			case 'f': case 'F':
				// Switch to the free camera
				_camera = &_freeCamera;
				Point p;
				_followCamera.getPosition(&p);
				_camera->moveTo(p);
				
				_followCamera.m_pTarget->getRotation(&p);
				_camera->rotateTo(p);
				_camMode = FREECAMERA;
				break;
			default:
				return false;
		}
	} else {
		switch (key) {
			case 'w': case 'W':
				fc->translateAhead = false;
				break;
			case 's': case 'S': 
				fc->translateBack = false;
				break;
			case 'a': case 'A':
				fc->translateLeft = false;
				break;
			case 'd': case 'D':
				fc->translateRight = false;
				break;
			
			case 'f': case 'F':
				{
					Point cameraPosition;
					_camera->getPosition(&cameraPosition);
					Point localObjectPosition;
					LocalObjectCollection *localObjects = &_game->currentWorld->m_localObjects;
					// Find the target if there is one.
					LOOPVECTOR(objectIndex, (*localObjects))
						LocalObject* lobj = (LocalObject*)((*localObjects)[objectIndex]);
						lobj->getPosition(&localObjectPosition);
						char* className = lobj->getClass();	
						if (className != NULL && strcmp(className, "Jeep") == 0)  {
							Jeep *j = (Jeep*)lobj;
							double aoi = j->m_areaOfInfluence;
							aoi *= aoi;
							
							if( cameraPosition.squaredDistanceTo(localObjectPosition) < aoi )
							{
								Point p;
								_freeCamera.getPosition(&p);
								_followCamera.moveTo(p);

								_followCamera.setTarget(lobj);
								_camera = &_followCamera;
								_camMode = FOLLOWCAMERA;
								break;
							}
						}
					ENDLOOP
				}
				break;				
			default: 
				return false;
		}
	}
	return true;
}
bool WorldUI::processSpecialKeyDown(int key, int x, int y) {
	FreeCamera *fc = (FreeCamera*)_camera;
	switch(key) {
		case GLUT_KEY_UP:
			fc->rotateUp = true;
			break;
		case GLUT_KEY_DOWN:
			fc->rotateDown = true;
			break;
		case GLUT_KEY_RIGHT:
			fc->rotateRight = true;
			break;
		case GLUT_KEY_LEFT:
			fc->rotateLeft = true;
			break;
		case GLUT_KEY_PAGE_UP:
			fc->translateUp = true;
			break;
		case GLUT_KEY_PAGE_DOWN:
			fc->translateDown = true;
			break;
		default:
			return false;
	}
	return true;
}
bool WorldUI::processSpecialKeyUp(int key, int x, int y) {
	FreeCamera *fc = (FreeCamera*)_camera;
	switch(key) {
		case GLUT_KEY_UP:
			fc->rotateUp = false;
			break;
		case GLUT_KEY_DOWN:
			fc->rotateDown = false;
			break;
		case GLUT_KEY_RIGHT:
			fc->rotateRight = false;
			break;
		case GLUT_KEY_LEFT:
			fc->rotateLeft = false;
			break;
		case GLUT_KEY_PAGE_UP:
			fc->translateUp = false;
			break;
		case GLUT_KEY_PAGE_DOWN:
			fc->translateDown = false;
			break;
		default:
			return false;
	}
	return true;
}

bool WorldUI::processMouseMove(int x, int y) {
	POINT screenPOINT; GetCursorPos (&screenPOINT); 
	Point point = {screenPOINT.x, screenPOINT.y, 0.0};
	Point center = {screenWidth >> 1, screenHeight >> 1, 0.0};
	double sensitivity = 0.1;
	Point difference = (point - center) * sensitivity;
	Point rotation = {-difference.y, -difference.x, 0.0};
	_camera->rotateBy (rotation); 

	return true;
}

bool WorldUI::processMouseClick(int button, int state, int x, int y) {
	return false;
}

void WorldUI::tick() {
	_time += Timer::Get()->deltaTime;
	_camera->tick();
}

void WorldUI::draw() {
	if (_game->currentWorld != NULL) {
		_camera->pushMatrix();
		_game->currentWorld->draw();
		_camera->popMatrix();
	}
	RGBA colorWhite = {1, 1, 1, 1};

	begin2DDrawing(800,600);
	glDisable(GL_TEXTURE_2D);
	

	// Bottom Bar
	glEnable(GL_BLEND);
	glBegin(GL_POLYGON);
		glColor4d(0.0,0.0,0.0, 0.5);
		glVertex2d(0,0);
		glVertex2d(800, 0);
		glVertex2d(800,25);
		glVertex2d(0,25);
	glEnd();
	glDisable(GL_BLEND);

	glLineWidth(2.0);
	glBegin(GL_LINES);
		glColor4d(0.0,0.0,0.0, 0.5);
		glVertex2d(0,25);
		glVertex2d(800,25);
	glEnd();
	colorWhite.apply();
	_debugFont->drawText(700, 7, "FPS: %4.f", Timer::Get()->fps() );
	
	FreeCamera *fc = (FreeCamera*)_camera;
	char buffer[1024];	
	sprintf(buffer, "Camera: [%3.f, %3.f, %3.f] <%3.f, %3.f, %3.f>",
		fc->position.x,
		fc->position.y,
		fc->position.z,
		fc->rotation.x,
		fc->rotation.y,
		fc->rotation.z);
	_debugFont->drawText(7,7,buffer);

	// Side Bar
	glEnable(GL_BLEND);
	glBegin(GL_POLYGON);
		glColor4d(0.0,0.0,0.0, 0.5);
		glVertex2d(675,600);
		glVertex2d(675, 25);
		glVertex2d(800, 25);
		glVertex2d(800,600);
	glEnd();
	glDisable(GL_BLEND);

	glLineWidth(2.0);
	glBegin(GL_LINES);
		glColor4d(0.0,0.0,0.0, 0.5);
		glVertex2d(675,25);
		glVertex2d(675,600);
	glEnd();
	
	colorWhite.apply();
	_debugFont->drawText(710, 580, "Objects");
	LocalObjectCollection loc = _game->currentWorld->m_localObjects;
	int y = 2;
	
	const int LINE_HEIGHT = 13;
	LOOPVECTOR(objectIndex, loc)
		LocalObject *obj = loc[objectIndex];
		char* className = obj->getClass();
		if (className != NULL) {
			_smallFont->drawText(685, 600-(++y*LINE_HEIGHT), className);
			if (strcmp(className, "Jeep") ==0) {
				Jeep *j = (Jeep*)obj;
				sprintf(buffer, "AOI: %.f", j->m_areaOfInfluence);
				_smallFont->drawText(750, 600-y*LINE_HEIGHT, buffer);
			}
		}

		sprintf(buffer, "[%4.f, %4.f, %4.f]", obj->m_position.x, obj->m_position.y, obj->m_position.z);
		_smallFont->drawText( 685, 600-(++y*LINE_HEIGHT), buffer);
		
		sprintf(buffer, "[%4.f, %4.f, %4.f]", obj->m_rotation.x, obj->m_rotation.y, obj->m_rotation.z);
		_smallFont->drawText( 685, 600-(++y*LINE_HEIGHT), buffer);
		y++;
	ENDLOOP

	
	// Cursor
	if (_cursorVisible) {
		glEnable(GL_BLEND);
		glColor4f(1,0,0,0.5);
		glBegin(GL_TRIANGLES);
			glVertex2d(400,305);
			glVertex2d(405,315);
			glVertex2d(395,315);

			glVertex2d(400, 295);
			glVertex2d(395, 285);
			glVertex2d(405, 285);

			glVertex2d(395,300);
			glVertex2d(385, 305);
			glVertex2d(385, 295);

			glVertex2d(405, 300);
			glVertex2d(415, 295);
			glVertex2d(415, 305);
		glEnd();
		glDisable(GL_BLEND);

		glBegin(GL_LINE_LOOP);
			glVertex2d(400,305);
			glVertex2d(405,315);
			glVertex2d(395,315);
		glEnd();
		glBegin(GL_LINE_LOOP);
			glVertex2d(400, 295);
			glVertex2d(395, 285);
			glVertex2d(405, 285);
		glEnd();
		glBegin(GL_LINE_LOOP);
			glVertex2d(395,300);
			glVertex2d(385, 305);
			glVertex2d(385, 295);
		glEnd();
		glBegin(GL_LINE_LOOP);
			glVertex2d(405, 300);
			glVertex2d(415, 295);
			glVertex2d(415, 305);
		glEnd();
	}

	
	glLineWidth(1.0);
	glColor4d(1.0,1.0,1.0,1.0);
	end2DDrawing();
}

void WorldUI::showCursor(bool show) {
	_cursorVisible = show;
}