#ifndef TERRAIN_H
#define TERRAIN_H

struct TERRAINPOINT
{
	float x, y, z;
};

struct TERRAINTEX
{
	float tx, ty;
};

class Terrain : public LocalObject
{
protected:
	// The location of the terrain in world space
	Point			m_location;

	// The height range of the terrain. The height of any vertex on the height map is given by the m_location.y + m_height * 
	float			m_height;

	//Texture id for the color map
	int				m_colorMap;

	// Height map information
	string			m_heightMapFile;
	HeightMap*		m_pHeightMap;

	// The size constraints of the heightmap
	float			m_sizeX;
	float			m_sizeZ;

	// The length of space inbetween successive vertices
	float			m_gridSizeX;
	float			m_gridSizeZ;

	// The indices into the vertex range
	unsigned int*	m_pIndices;

	// The array of heightmap vertices in system memory
	TERRAINPOINT*	m_pVertices;
	// The array of heightmap texture coordinates in system memory
	TERRAINTEX*		m_pTexCoords;

	// Vertex buffer id
	GLuint			m_vertexBuffer;
	// Index buffer id
	GLuint			m_indexBuffer;
	// Tex Coord buffer
	GLuint			m_textureBuffer;

	void processProperties();

public:

	Terrain()
	{
		m_rotation.x = 0.0;
		m_rotation.y = 0.0;
		m_rotation.z = 0.0;
		m_height = 0.0;
		m_colorMap = TEXTURELIST_INVALIDINDEX;
		m_pHeightMap = NULL;
		m_sizeX = 0.0;
		m_sizeZ = 0.0;
		m_gridSizeX = 0.0;
		m_gridSizeZ = 0.0;
		m_gridSizeX = 0.0;
		m_gridSizeZ = 0.0;
		m_pIndices = NULL;
		m_pVertices = NULL;
		m_pTexCoords = NULL;
		m_vertexBuffer = 0;
		m_indexBuffer = 0;
		m_textureBuffer = 0;
		
	};
	virtual ~Terrain();
	void setProperties(Point position, string heightMapFile, int colorMap, double heightRange, double sizeX, double sizeZ)
	{
		m_position = position;
		m_heightMapFile = heightMapFile;
		m_colorMap = colorMap;
		m_height = heightRange;
		m_sizeX = sizeX;
		m_sizeZ = sizeZ;
	};

	void initialize();

	virtual void draw();
	virtual void tick() {}
	virtual void dispose() {}


	// Perhaps put this into a common class between Jeep, Terrain, and all the other LocalObject's that have the same format
	// in the WRL file.
	bool deserialize(BinaryFileStreamReader *input) {
		__super::deserialize(input);
		processProperties();
		initialize();
		return true;
	}

};



#endif