
//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include "includes.h"

//*****************************************************************************************//
//                                       World                                             //
//*****************************************************************************************//

World *world = NULL;

void World::setup () {
	//Nothing to do so far.
}

void World::wrapup () {
}

void World::tick () {
	// Tick all Static Objects
	LOOPVECTOR(objectIndex, m_worldObjects)
		((WorldObject*)m_worldObjects[objectIndex])->tick();
	ENDLOOP

	// Tick All Dynamic Objects
	LOOPVECTOR(objectIndex, m_localObjects)
		((LocalObject*)m_localObjects[objectIndex])->tick();
	ENDLOOP
}

void World::draw () {
	LOOPVECTOR(objectIndex, m_worldObjects)
		((WorldObject*)m_worldObjects[objectIndex])->draw();
	ENDLOOP
	
	LOOPVECTOR(objectIndex, m_localObjects)
		((LocalObject*)m_localObjects[objectIndex])->draw();
	ENDLOOP
	
}
World *World::load() {
	char *filename = promptForWorldRead ();
	if (filename == NULL) return NULL;
	return World::load(filename);
}
World *World::load (const char* filename) {

	//Prompt for a ".wrl" file (THIS IS YOUR OWN PERSONAL FORMAT) and reads it in to
	//a brand new world. You should also load all the textures you ultimately need
	//into the game card (see class Texture for how to do this to an individual texture).

	//Note: many faces will have the same texture name... Don't read in the texture bits
	//multiple times. The best way to do this is have collected the texture names together 
	//in one collection and output them before any of the objects are output.
	//Then, when you read the names in here, you can create a corresponding texture object.
	//Subsequently, when you read a face in, you can either look up the texture object 
	//by name if a name was exported with the face or you can simply index the collection
	//if you exported an index instead of a name (much easier).
 
	

	// create the new world
	World *world = new World;

	// Open a reader for the WRL file
	BinaryFileStreamReader *input = new BinaryFileStreamReader(filename);
	if (input->isBad ()) {prompt ("Unable to open input file \"%s\".", filename); return NULL;}

	string tag;
	input->readTag(&tag); //WRL
	char version = input->readChar();
	if (version != WRL_VERSION) {
		prompt("Incorrect File Version.\nReader Version:\t%i\nFile Version:\t%i\n\nPlease rebuild the world.", WRL_VERSION, version);
		input->close();
		delete input;
		return NULL;
	}

	TextureList::Get()->LoadTextureNames(input);
	TextureList::Get()->LoadTextures();
	TextureList::Get()->RestoreTextures();

	MeshList::Get()->deserialize(input);

	world->startPosition.deserialize(input);
	world->startRotation.deserialize(input);

	input->readTag(&tag); //WSL
	int objectCount = input->readInteger();
	for(int objectIndex = 0; objectIndex < objectCount; objectIndex ++) {
		WorldObject *wo = new WorldObject();
		wo->deserialize(input);
		world->m_worldObjects.push_back(wo);
	}

	LocalObjectFactory *factory = LocalObjectFactory::Get();
	input->readTag(&tag); //LOL
	int localCount = input->readInteger();
	for(int localIndex = 0; localIndex < localCount; localIndex ++) {
		LocalObject *lo = factory->deserialize(input);
		//LocalObject *lo = new LocalObject();
		//lo->deserialize(input);
		if (lo != NULL)
			world->m_localObjects.push_back(lo);
	}
	
	input->close ();
	delete input;
	return world;
}

void World::unload () {
	LOOPVECTOR(objectIndex, m_worldObjects)
		((WorldObject*)m_worldObjects[objectIndex])->dispose();
		delete ((WorldObject*)m_worldObjects[objectIndex]);
	ENDLOOP
	m_worldObjects.clear();

	LOOPVECTOR(objectIndex, m_localObjects)
		((LocalObject*)m_localObjects[objectIndex])->dispose();
		delete ((LocalObject*)m_localObjects[objectIndex]);
	ENDLOOP
	m_localObjects.clear();

	TextureList::Get()->InvalidateTextures();
	TextureList::Get()->UnloadTextures();

	MeshList::Get()->dispose();
}